
/* GENERATED CODE... DO NOT EDIT */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 *
 * Copyright (C) 2012 Lanedo GmbH
 * Copyright (C) 2012-2015 Aleksander Morgado <aleksander@aleksander.es>
 */


#include <string.h>

#include "qmi-wds.h"
#include "qmi-enum-types.h"
#include "qmi-enum-types-private.h"
#include "qmi-flags64-types.h"
#include "qmi-error-types.h"
#include "qmi-device.h"
#include "qmi-utils.h"

#define QMI_STATUS_SUCCESS 0x0000
#define QMI_STATUS_FAILURE 0x0001


typedef enum {
    QMI_MESSAGE_WDS_RESET = 0x0000,
    QMI_MESSAGE_WDS_ABORT = 0x0002,
    QMI_MESSAGE_WDS_GET_SUPPORTED_MESSAGES = 0x001E,
    QMI_MESSAGE_WDS_START_NETWORK = 0x0020,
    QMI_MESSAGE_WDS_STOP_NETWORK = 0x0021,
    QMI_MESSAGE_WDS_GET_PACKET_SERVICE_STATUS = 0x0022,
    QMI_MESSAGE_WDS_GET_PACKET_STATISTICS = 0x0024,
    QMI_MESSAGE_WDS_GET_PROFILE_LIST = 0x002A,
    QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS = 0x002B,
    QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS = 0x002C,
    QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS = 0x002D,
    QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS = 0x0034,
    QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY = 0x0037,
    QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY = 0x0044,
    QMI_MESSAGE_WDS_SET_IP_FAMILY = 0x004D,
    QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS = 0x0051,
    QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO = 0x006C,
} QmiMessageWds;


typedef enum {
    QMI_INDICATION_WDS_PACKET_SERVICE_STATUS = 0x0022,
} QmiIndicationWds;


/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Reset */


/* --- Input -- */

/* Note: no fields in the Input container */

static QmiMessage *
__qmi_message_wds_reset_request_create (
    guint16 transaction_id,
    guint8 cid,
    gpointer unused,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_RESET);

    return self;
}

/* --- Output -- */

/**
 * QmiMessageResult:
 * @error_status: a #guint16.
 * @error_code: a #guint16.
 *
 * A QmiMessageResult struct.
 */
typedef struct _QmiMessageResult {
    guint16 error_status;
    guint16 error_code;
} QmiMessageResult;

struct _QmiMessageWdsResetOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;
};

#define QMI_MESSAGE_WDS_RESET_OUTPUT_TLV_RESULT 0x02

/**
 * qmi_message_wds_reset_output_get_result:
 * @self: a QmiMessageWdsResetOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_reset_output_get_result (
    QmiMessageWdsResetOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

GType
qmi_message_wds_reset_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsResetOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_reset_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_reset_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_reset_output_ref:
 * @self: a #QmiMessageWdsResetOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsResetOutput *
qmi_message_wds_reset_output_ref (QmiMessageWdsResetOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_reset_output_unref:
 * @self: a #QmiMessageWdsResetOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_reset_output_unref (QmiMessageWdsResetOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsResetOutput, self);
    }
}

static gboolean
qmi_message_result_validate (
    const guint8 *buffer,
    guint16 buffer_len)
{
    static const guint expected_len = 4;

    if (buffer_len < expected_len) {
        g_warning ("Cannot read the 'Result' TLV: expected '%u' bytes, but only got '%u' bytes",
                   expected_len, buffer_len);
        return FALSE;
    }

    return TRUE;
}

static gchar *
qmi_message_result_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    const guint8 *buffer;
    guint16 buffer_len;

    buffer = qmi_message_get_raw_tlv (self,
                                      QMI_MESSAGE_WDS_RESET_OUTPUT_TLV_RESULT,
                                      &buffer_len);
    if (buffer) {
        GString *printable;
        guint16 error_status;
        guint16 error_code;

        printable = g_string_new ("");
        qmi_utils_read_guint16_from_buffer (
            &buffer,
            &buffer_len,
            QMI_ENDIAN_LITTLE,
            &error_status);
        qmi_utils_read_guint16_from_buffer (
            &buffer,
            &buffer_len,
            QMI_ENDIAN_LITTLE,
            &error_code);

        g_warn_if_fail (buffer_len == 0);

        if (error_status == QMI_STATUS_SUCCESS)
            g_string_append (printable, "SUCCESS");
        else
            g_string_append_printf (printable,
                                    "FAILURE: %s",
                                    qmi_protocol_error_get_string ((QmiProtocolError) error_code));

        return g_string_free (printable, FALSE);
    }

    return NULL;
}

struct message_reset_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_reset_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_reset_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_RESET_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_reset_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Reset\" (0x0000)\n",
                            line_prefix);

    {
        struct message_reset_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_reset_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsResetOutput *
__qmi_message_wds_reset_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsResetOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_RESET, NULL);

    self = g_slice_new0 (QmiMessageWdsResetOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_RESET_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_reset_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_reset_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Abort */


/* --- Input -- */

/**
 * QmiMessageWdsAbortInput:
 *
 * The #QmiMessageWdsAbortInput structure contains private data and should only be accessed
 * using the provided API.
 */
typedef struct _QmiMessageWdsAbortInput QmiMessageWdsAbortInput;
static GType qmi_message_wds_abort_input_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_MESSAGE_WDS_ABORT_INPUT (qmi_message_wds_abort_input_get_type ())

struct _QmiMessageWdsAbortInput {
    volatile gint ref_count;

    /* Transaction ID */
    gboolean arg_transaction_id_set;
    guint16 arg_transaction_id;
};

#define QMI_MESSAGE_WDS_ABORT_INPUT_TLV_TRANSACTION_ID 0x01

static gboolean qmi_message_wds_abort_input_get_transaction_id (
    QmiMessageWdsAbortInput *self,
    guint16 *value_transaction_id,
    GError **error);

/**
 * qmi_message_wds_abort_input_get_transaction_id:
 * @self: a #QmiMessageWdsAbortInput.
 * @value_transaction_id: a placeholder for the output #guint16, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Transaction ID' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
static gboolean
qmi_message_wds_abort_input_get_transaction_id (
    QmiMessageWdsAbortInput *self,
    guint16 *value_transaction_id,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_transaction_id_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Transaction ID' was not found in the message");
        return FALSE;
    }

    if (value_transaction_id)
        *value_transaction_id = self->arg_transaction_id;

    return TRUE;
}

static gboolean qmi_message_wds_abort_input_set_transaction_id (
    QmiMessageWdsAbortInput *self,
    guint16 value_transaction_id,
    GError **error);

/**
 * qmi_message_wds_abort_input_set_transaction_id:
 * @self: a #QmiMessageWdsAbortInput.
 * @value_transaction_id: a #guint16.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Transaction ID' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
static gboolean
qmi_message_wds_abort_input_set_transaction_id (
    QmiMessageWdsAbortInput *self,
    guint16 value_transaction_id,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_transaction_id = value_transaction_id;
    self->arg_transaction_id_set = TRUE;

    return TRUE;
}

static QmiMessageWdsAbortInput *qmi_message_wds_abort_input_ref (QmiMessageWdsAbortInput *self);
static void qmi_message_wds_abort_input_unref (QmiMessageWdsAbortInput *self);
static QmiMessageWdsAbortInput *qmi_message_wds_abort_input_new (void);

static GType
qmi_message_wds_abort_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsAbortInput"),
                                          (GBoxedCopyFunc) qmi_message_wds_abort_input_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_abort_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_abort_input_ref:
 * @self: a #QmiMessageWdsAbortInput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
static QmiMessageWdsAbortInput *
qmi_message_wds_abort_input_ref (QmiMessageWdsAbortInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_abort_input_unref:
 * @self: a #QmiMessageWdsAbortInput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
static void
qmi_message_wds_abort_input_unref (QmiMessageWdsAbortInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsAbortInput, self);
    }
}

/**
 * qmi_message_wds_abort_input_new:
 *
 * Allocates a new #QmiMessageWdsAbortInput.
 *
 * Returns: the newly created #QmiMessageWdsAbortInput. The returned value should be freed with qmi_message_wds_abort_input_unref().
 */
static QmiMessageWdsAbortInput *
qmi_message_wds_abort_input_new (void)
{
    QmiMessageWdsAbortInput *self;

    self = g_slice_new0 (QmiMessageWdsAbortInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_wds_abort_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageWdsAbortInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_ABORT);

    /* All TLVs are optional, we allow NULL input */
    if (!input)
        return self;

    /* Try to add the 'Transaction ID' TLV */
    if (input->arg_transaction_id_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_ABORT_INPUT_TLV_TRANSACTION_ID, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Transaction ID': ");
            goto error_out;
        }

        /* Write the guint16 variable to the buffer */
        if (!qmi_message_tlv_write_guint16 (self, QMI_ENDIAN_LITTLE, input->arg_transaction_id, error)) {
            g_prefix_error (error, "Cannot write integer in TLV 'Transaction ID': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Transaction ID': ");
            goto error_out;
        }
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

/**
 * QmiMessageWdsAbortOutput:
 *
 * The #QmiMessageWdsAbortOutput structure contains private data and should only be accessed
 * using the provided API.
 */
typedef struct _QmiMessageWdsAbortOutput QmiMessageWdsAbortOutput;
static GType qmi_message_wds_abort_output_get_type (void) G_GNUC_CONST;
#define QMI_TYPE_MESSAGE_WDS_ABORT_OUTPUT (qmi_message_wds_abort_output_get_type ())

struct _QmiMessageWdsAbortOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;
};

#define QMI_MESSAGE_WDS_ABORT_OUTPUT_TLV_RESULT 0x02

gboolean qmi_message_wds_abort_output_get_result (
    QmiMessageWdsAbortOutput *self,
    GError **error);

/**
 * qmi_message_wds_abort_output_get_result:
 * @self: a QmiMessageWdsAbortOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_abort_output_get_result (
    QmiMessageWdsAbortOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

static QmiMessageWdsAbortOutput *qmi_message_wds_abort_output_ref (QmiMessageWdsAbortOutput *self);
static void qmi_message_wds_abort_output_unref (QmiMessageWdsAbortOutput *self);

static GType
qmi_message_wds_abort_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsAbortOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_abort_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_abort_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_abort_output_ref:
 * @self: a #QmiMessageWdsAbortOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
static QmiMessageWdsAbortOutput *
qmi_message_wds_abort_output_ref (QmiMessageWdsAbortOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_abort_output_unref:
 * @self: a #QmiMessageWdsAbortOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
static void
qmi_message_wds_abort_output_unref (QmiMessageWdsAbortOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsAbortOutput, self);
    }
}

static gchar *
qmi_message_wds_abort_input_transaction_id_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_ABORT_INPUT_TLV_TRANSACTION_ID, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT16_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_abort_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_abort_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_abort_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_WDS_ABORT_INPUT_TLV_TRANSACTION_ID:
            tlv_type_str = "Transaction ID";
            translated_value = qmi_message_wds_abort_input_transaction_id_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_ABORT_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_abort_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Abort\" (0x0002)\n",
                            line_prefix);

    {
        struct message_abort_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_abort_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsAbortOutput *
__qmi_message_wds_abort_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsAbortOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_ABORT, NULL);

    self = g_slice_new0 (QmiMessageWdsAbortOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_ABORT_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_abort_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_abort_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Get Supported Messages */


/* --- Input -- */

/* Note: no fields in the Input container */

static QmiMessage *
__qmi_message_wds_get_supported_messages_request_create (
    guint16 transaction_id,
    guint8 cid,
    gpointer unused,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_GET_SUPPORTED_MESSAGES);

    return self;
}

/* --- Output -- */

struct _QmiMessageWdsGetSupportedMessagesOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;

    /* List */
    gboolean arg_list_set;
    GArray *arg_list;
};

#define QMI_MESSAGE_WDS_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_RESULT 0x02
#define QMI_MESSAGE_WDS_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_LIST 0x10

/**
 * qmi_message_wds_get_supported_messages_output_get_result:
 * @self: a QmiMessageWdsGetSupportedMessagesOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_get_supported_messages_output_get_result (
    QmiMessageWdsGetSupportedMessagesOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

/**
 * qmi_message_wds_get_supported_messages_output_get_list:
 * @self: a #QmiMessageWdsGetSupportedMessagesOutput.
 * @value_list: a placeholder for the output #GArray of #guint8 elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'List' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_supported_messages_output_get_list (
    QmiMessageWdsGetSupportedMessagesOutput *self,
    GArray **value_list,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_list_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'List' was not found in the message");
        return FALSE;
    }

    if (value_list)
        *value_list = self->arg_list;

    return TRUE;
}

GType
qmi_message_wds_get_supported_messages_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetSupportedMessagesOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_supported_messages_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_supported_messages_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_supported_messages_output_ref:
 * @self: a #QmiMessageWdsGetSupportedMessagesOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetSupportedMessagesOutput *
qmi_message_wds_get_supported_messages_output_ref (QmiMessageWdsGetSupportedMessagesOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_supported_messages_output_unref:
 * @self: a #QmiMessageWdsGetSupportedMessagesOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_supported_messages_output_unref (QmiMessageWdsGetSupportedMessagesOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        if (self->arg_list)
            g_array_unref (self->arg_list);
        g_slice_free (QmiMessageWdsGetSupportedMessagesOutput, self);
    }
}

static gchar *
qmi_message_wds_get_supported_messages_output_list_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_LIST, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint list_i;
        guint16 list_n_items;

        /* Read number of items in the array */
        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(list_n_items), &error))
            goto out;

        g_string_append (printable, "{");

        for (list_i = 0; list_i < list_n_items; list_i++) {
            g_string_append_printf (printable, " [%u] = '", list_i);

            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%u", (guint)tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_get_supported_messages_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_get_supported_messages_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_get_supported_messages_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_LIST:
            tlv_type_str = "List";
            translated_value = qmi_message_wds_get_supported_messages_output_list_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_get_supported_messages_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Get Supported Messages\" (0x001E)\n",
                            line_prefix);

    {
        struct message_get_supported_messages_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_get_supported_messages_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsGetSupportedMessagesOutput *
__qmi_message_wds_get_supported_messages_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsGetSupportedMessagesOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_GET_SUPPORTED_MESSAGES, NULL);

    self = g_slice_new0 (QmiMessageWdsGetSupportedMessagesOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_get_supported_messages_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_get_supported_messages_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_SUPPORTED_MESSAGES_OUTPUT_TLV_LIST, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_supported_messages_output_list_out;
            }
            {
                guint list_i;
                guint16 list_n_items;

                /* Read number of items in the array */
                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(list_n_items), NULL))
                    goto qmi_message_wds_get_supported_messages_output_list_out;

                self->arg_list = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (guint8),
                    (guint)list_n_items);

                for (list_i = 0; list_i < list_n_items; list_i++) {
                    guint8 list_aux;

                    if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(list_aux), NULL))
                        goto qmi_message_wds_get_supported_messages_output_list_out;
                    g_array_insert_val (self->arg_list, list_i, list_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'List' TLV", offset);
            }

            self->arg_list_set = TRUE;

qmi_message_wds_get_supported_messages_output_list_out:
            ;

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Start Network */


/* --- Input -- */

struct _QmiMessageWdsStartNetworkInput {
    volatile gint ref_count;

    /* Call Type */
    gboolean arg_call_type_set;
    guint8 arg_call_type;

    /* Extended Technology Preference */
    gboolean arg_extended_technology_preference_set;
    guint16 arg_extended_technology_preference;

    /* Enable Autoconnect */
    gboolean arg_enable_autoconnect_set;
    guint8 arg_enable_autoconnect;

    /* Profile Index 3GPP2 */
    gboolean arg_profile_index_3gpp2_set;
    guint8 arg_profile_index_3gpp2;

    /* Profile Index 3GPP */
    gboolean arg_profile_index_3gpp_set;
    guint8 arg_profile_index_3gpp;

    /* Technology Preference */
    gboolean arg_technology_preference_set;
    guint8 arg_technology_preference;

    /* IP Family Preference */
    gboolean arg_ip_family_preference_set;
    guint8 arg_ip_family_preference;

    /* Password */
    gboolean arg_password_set;
    gchar *arg_password;

    /* Username */
    gboolean arg_username_set;
    gchar *arg_username;

    /* Authentication Preference */
    gboolean arg_authentication_preference_set;
    guint8 arg_authentication_preference;

    /* IPv4 Address Preference */
    gboolean arg_ipv4_address_preference_set;
    guint32 arg_ipv4_address_preference;

    /* APN */
    gboolean arg_apn_set;
    gchar *arg_apn;

    /* Secondary NBNS Address Preference */
    gboolean arg_secondary_nbns_address_preference_set;
    guint32 arg_secondary_nbns_address_preference;

    /* Primary NBNS Address Preference */
    gboolean arg_primary_nbns_address_preference_set;
    guint32 arg_primary_nbns_address_preference;

    /* Secondary DNS Address Preference */
    gboolean arg_secondary_dns_address_preference_set;
    guint32 arg_secondary_dns_address_preference;

    /* Primary DNS Address Preference */
    gboolean arg_primary_dns_address_preference_set;
    guint32 arg_primary_dns_address_preference;
};

#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_CALL_TYPE 0x35
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_EXTENDED_TECHNOLOGY_PREFERENCE 0x34
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_ENABLE_AUTOCONNECT 0x33
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PROFILE_INDEX_3GPP2 0x32
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PROFILE_INDEX_3GPP 0x31
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_TECHNOLOGY_PREFERENCE 0x30
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_IP_FAMILY_PREFERENCE 0x19
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PASSWORD 0x18
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_USERNAME 0x17
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_AUTHENTICATION_PREFERENCE 0x16
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_IPV4_ADDRESS_PREFERENCE 0x15
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_APN 0x14
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_SECONDARY_NBNS_ADDRESS_PREFERENCE 0x13
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PRIMARY_NBNS_ADDRESS_PREFERENCE 0x12
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_SECONDARY_DNS_ADDRESS_PREFERENCE 0x11
#define QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PRIMARY_DNS_ADDRESS_PREFERENCE 0x10

/**
 * qmi_message_wds_start_network_input_get_call_type:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_call_type: a placeholder for the output #QmiWdsCallType, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Call Type' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_call_type (
    QmiMessageWdsStartNetworkInput *self,
    QmiWdsCallType *value_call_type,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_call_type_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Call Type' was not found in the message");
        return FALSE;
    }

    if (value_call_type)
        *value_call_type = (QmiWdsCallType)(self->arg_call_type);

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_call_type:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_call_type: a #QmiWdsCallType.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Call Type' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_call_type (
    QmiMessageWdsStartNetworkInput *self,
    QmiWdsCallType value_call_type,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_call_type = (guint8)(value_call_type);
    self->arg_call_type_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_extended_technology_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_extended_technology_preference: a placeholder for the output #QmiWdsExtendedTechnologyPreference, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Extended Technology Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_extended_technology_preference (
    QmiMessageWdsStartNetworkInput *self,
    QmiWdsExtendedTechnologyPreference *value_extended_technology_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_extended_technology_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Extended Technology Preference' was not found in the message");
        return FALSE;
    }

    if (value_extended_technology_preference)
        *value_extended_technology_preference = (QmiWdsExtendedTechnologyPreference)(self->arg_extended_technology_preference);

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_extended_technology_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_extended_technology_preference: a #QmiWdsExtendedTechnologyPreference.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Extended Technology Preference' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_extended_technology_preference (
    QmiMessageWdsStartNetworkInput *self,
    QmiWdsExtendedTechnologyPreference value_extended_technology_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_extended_technology_preference = (guint16)(value_extended_technology_preference);
    self->arg_extended_technology_preference_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_enable_autoconnect:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_enable_autoconnect: a placeholder for the output #gboolean, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Enable Autoconnect' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_enable_autoconnect (
    QmiMessageWdsStartNetworkInput *self,
    gboolean *value_enable_autoconnect,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_enable_autoconnect_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Enable Autoconnect' was not found in the message");
        return FALSE;
    }

    if (value_enable_autoconnect)
        *value_enable_autoconnect = (gboolean)(self->arg_enable_autoconnect);

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_enable_autoconnect:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_enable_autoconnect: a #gboolean.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Enable Autoconnect' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_enable_autoconnect (
    QmiMessageWdsStartNetworkInput *self,
    gboolean value_enable_autoconnect,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_enable_autoconnect = (guint8)(value_enable_autoconnect);
    self->arg_enable_autoconnect_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_profile_index_3gpp2:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_profile_index_3gpp2: a placeholder for the output #guint8, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Profile Index 3GPP2' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_profile_index_3gpp2 (
    QmiMessageWdsStartNetworkInput *self,
    guint8 *value_profile_index_3gpp2,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_profile_index_3gpp2_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Profile Index 3GPP2' was not found in the message");
        return FALSE;
    }

    if (value_profile_index_3gpp2)
        *value_profile_index_3gpp2 = self->arg_profile_index_3gpp2;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_profile_index_3gpp2:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_profile_index_3gpp2: a #guint8.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Profile Index 3GPP2' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_profile_index_3gpp2 (
    QmiMessageWdsStartNetworkInput *self,
    guint8 value_profile_index_3gpp2,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_profile_index_3gpp2 = value_profile_index_3gpp2;
    self->arg_profile_index_3gpp2_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_profile_index_3gpp:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_profile_index_3gpp: a placeholder for the output #guint8, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Profile Index 3GPP' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_profile_index_3gpp (
    QmiMessageWdsStartNetworkInput *self,
    guint8 *value_profile_index_3gpp,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_profile_index_3gpp_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Profile Index 3GPP' was not found in the message");
        return FALSE;
    }

    if (value_profile_index_3gpp)
        *value_profile_index_3gpp = self->arg_profile_index_3gpp;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_profile_index_3gpp:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_profile_index_3gpp: a #guint8.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Profile Index 3GPP' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_profile_index_3gpp (
    QmiMessageWdsStartNetworkInput *self,
    guint8 value_profile_index_3gpp,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_profile_index_3gpp = value_profile_index_3gpp;
    self->arg_profile_index_3gpp_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_technology_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_technology_preference: a placeholder for the output #QmiWdsTechnologyPreference, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Technology Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_technology_preference (
    QmiMessageWdsStartNetworkInput *self,
    QmiWdsTechnologyPreference *value_technology_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_technology_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Technology Preference' was not found in the message");
        return FALSE;
    }

    if (value_technology_preference)
        *value_technology_preference = (QmiWdsTechnologyPreference)(self->arg_technology_preference);

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_technology_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_technology_preference: a #QmiWdsTechnologyPreference.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Technology Preference' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_technology_preference (
    QmiMessageWdsStartNetworkInput *self,
    QmiWdsTechnologyPreference value_technology_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_technology_preference = (guint8)(value_technology_preference);
    self->arg_technology_preference_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_ip_family_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_ip_family_preference: a placeholder for the output #QmiWdsIpFamily, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IP Family Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_ip_family_preference (
    QmiMessageWdsStartNetworkInput *self,
    QmiWdsIpFamily *value_ip_family_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ip_family_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IP Family Preference' was not found in the message");
        return FALSE;
    }

    if (value_ip_family_preference)
        *value_ip_family_preference = (QmiWdsIpFamily)(self->arg_ip_family_preference);

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_ip_family_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_ip_family_preference: a #QmiWdsIpFamily.
 * @error: Return location for error or %NULL.
 *
 * Set the 'IP Family Preference' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_ip_family_preference (
    QmiMessageWdsStartNetworkInput *self,
    QmiWdsIpFamily value_ip_family_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_ip_family_preference = (guint8)(value_ip_family_preference);
    self->arg_ip_family_preference_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_password:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_password: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Password' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_password (
    QmiMessageWdsStartNetworkInput *self,
    const gchar **value_password,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_password_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Password' was not found in the message");
        return FALSE;
    }

    if (value_password)
        *value_password = self->arg_password;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_password:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_password: a constant string.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Password' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_password (
    QmiMessageWdsStartNetworkInput *self,
    const gchar *value_password,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    g_free (self->arg_password);
    self->arg_password = g_strdup (value_password ? value_password : "");
    self->arg_password_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_username:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_username: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Username' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_username (
    QmiMessageWdsStartNetworkInput *self,
    const gchar **value_username,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_username_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Username' was not found in the message");
        return FALSE;
    }

    if (value_username)
        *value_username = self->arg_username;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_username:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_username: a constant string.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Username' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_username (
    QmiMessageWdsStartNetworkInput *self,
    const gchar *value_username,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    g_free (self->arg_username);
    self->arg_username = g_strdup (value_username ? value_username : "");
    self->arg_username_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_authentication_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_authentication_preference: a placeholder for the output #QmiWdsAuthentication, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Authentication Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_authentication_preference (
    QmiMessageWdsStartNetworkInput *self,
    QmiWdsAuthentication *value_authentication_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_authentication_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Authentication Preference' was not found in the message");
        return FALSE;
    }

    if (value_authentication_preference)
        *value_authentication_preference = (QmiWdsAuthentication)(self->arg_authentication_preference);

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_authentication_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_authentication_preference: a #QmiWdsAuthentication.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Authentication Preference' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_authentication_preference (
    QmiMessageWdsStartNetworkInput *self,
    QmiWdsAuthentication value_authentication_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_authentication_preference = (guint8)(value_authentication_preference);
    self->arg_authentication_preference_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_ipv4_address_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_ipv4_address_preference: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv4 Address Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_ipv4_address_preference (
    QmiMessageWdsStartNetworkInput *self,
    guint32 *value_ipv4_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv4_address_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv4 Address Preference' was not found in the message");
        return FALSE;
    }

    if (value_ipv4_address_preference)
        *value_ipv4_address_preference = self->arg_ipv4_address_preference;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_ipv4_address_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_ipv4_address_preference: a #guint32.
 * @error: Return location for error or %NULL.
 *
 * Set the 'IPv4 Address Preference' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_ipv4_address_preference (
    QmiMessageWdsStartNetworkInput *self,
    guint32 value_ipv4_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_ipv4_address_preference = value_ipv4_address_preference;
    self->arg_ipv4_address_preference_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_apn:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_apn: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'APN' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_apn (
    QmiMessageWdsStartNetworkInput *self,
    const gchar **value_apn,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_apn_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'APN' was not found in the message");
        return FALSE;
    }

    if (value_apn)
        *value_apn = self->arg_apn;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_apn:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_apn: a constant string.
 * @error: Return location for error or %NULL.
 *
 * Set the 'APN' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_apn (
    QmiMessageWdsStartNetworkInput *self,
    const gchar *value_apn,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    g_free (self->arg_apn);
    self->arg_apn = g_strdup (value_apn ? value_apn : "");
    self->arg_apn_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_secondary_nbns_address_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_secondary_nbns_address_preference: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Secondary NBNS Address Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_secondary_nbns_address_preference (
    QmiMessageWdsStartNetworkInput *self,
    guint32 *value_secondary_nbns_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_secondary_nbns_address_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Secondary NBNS Address Preference' was not found in the message");
        return FALSE;
    }

    if (value_secondary_nbns_address_preference)
        *value_secondary_nbns_address_preference = self->arg_secondary_nbns_address_preference;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_secondary_nbns_address_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_secondary_nbns_address_preference: a #guint32.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Secondary NBNS Address Preference' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_secondary_nbns_address_preference (
    QmiMessageWdsStartNetworkInput *self,
    guint32 value_secondary_nbns_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_secondary_nbns_address_preference = value_secondary_nbns_address_preference;
    self->arg_secondary_nbns_address_preference_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_primary_nbns_address_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_primary_nbns_address_preference: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Primary NBNS Address Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_primary_nbns_address_preference (
    QmiMessageWdsStartNetworkInput *self,
    guint32 *value_primary_nbns_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_primary_nbns_address_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Primary NBNS Address Preference' was not found in the message");
        return FALSE;
    }

    if (value_primary_nbns_address_preference)
        *value_primary_nbns_address_preference = self->arg_primary_nbns_address_preference;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_primary_nbns_address_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_primary_nbns_address_preference: a #guint32.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Primary NBNS Address Preference' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_primary_nbns_address_preference (
    QmiMessageWdsStartNetworkInput *self,
    guint32 value_primary_nbns_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_primary_nbns_address_preference = value_primary_nbns_address_preference;
    self->arg_primary_nbns_address_preference_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_secondary_dns_address_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_secondary_dns_address_preference: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Secondary DNS Address Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_secondary_dns_address_preference (
    QmiMessageWdsStartNetworkInput *self,
    guint32 *value_secondary_dns_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_secondary_dns_address_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Secondary DNS Address Preference' was not found in the message");
        return FALSE;
    }

    if (value_secondary_dns_address_preference)
        *value_secondary_dns_address_preference = self->arg_secondary_dns_address_preference;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_secondary_dns_address_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_secondary_dns_address_preference: a #guint32.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Secondary DNS Address Preference' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_secondary_dns_address_preference (
    QmiMessageWdsStartNetworkInput *self,
    guint32 value_secondary_dns_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_secondary_dns_address_preference = value_secondary_dns_address_preference;
    self->arg_secondary_dns_address_preference_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_get_primary_dns_address_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_primary_dns_address_preference: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Primary DNS Address Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_get_primary_dns_address_preference (
    QmiMessageWdsStartNetworkInput *self,
    guint32 *value_primary_dns_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_primary_dns_address_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Primary DNS Address Preference' was not found in the message");
        return FALSE;
    }

    if (value_primary_dns_address_preference)
        *value_primary_dns_address_preference = self->arg_primary_dns_address_preference;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_input_set_primary_dns_address_preference:
 * @self: a #QmiMessageWdsStartNetworkInput.
 * @value_primary_dns_address_preference: a #guint32.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Primary DNS Address Preference' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_input_set_primary_dns_address_preference (
    QmiMessageWdsStartNetworkInput *self,
    guint32 value_primary_dns_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_primary_dns_address_preference = value_primary_dns_address_preference;
    self->arg_primary_dns_address_preference_set = TRUE;

    return TRUE;
}

GType
qmi_message_wds_start_network_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsStartNetworkInput"),
                                          (GBoxedCopyFunc) qmi_message_wds_start_network_input_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_start_network_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_start_network_input_ref:
 * @self: a #QmiMessageWdsStartNetworkInput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsStartNetworkInput *
qmi_message_wds_start_network_input_ref (QmiMessageWdsStartNetworkInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_start_network_input_unref:
 * @self: a #QmiMessageWdsStartNetworkInput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_start_network_input_unref (QmiMessageWdsStartNetworkInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_free (self->arg_password);
        g_free (self->arg_username);
        g_free (self->arg_apn);
        g_slice_free (QmiMessageWdsStartNetworkInput, self);
    }
}

/**
 * qmi_message_wds_start_network_input_new:
 *
 * Allocates a new #QmiMessageWdsStartNetworkInput.
 *
 * Returns: the newly created #QmiMessageWdsStartNetworkInput. The returned value should be freed with qmi_message_wds_start_network_input_unref().
 */
QmiMessageWdsStartNetworkInput *
qmi_message_wds_start_network_input_new (void)
{
    QmiMessageWdsStartNetworkInput *self;

    self = g_slice_new0 (QmiMessageWdsStartNetworkInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_wds_start_network_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageWdsStartNetworkInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_START_NETWORK);

    /* All TLVs are optional, we allow NULL input */
    if (!input)
        return self;

    /* Try to add the 'Call Type' TLV */
    if (input->arg_call_type_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_CALL_TYPE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Call Type': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_call_type;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Call Type': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Call Type': ");
            goto error_out;
        }
    }

    /* Try to add the 'Extended Technology Preference' TLV */
    if (input->arg_extended_technology_preference_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_EXTENDED_TECHNOLOGY_PREFERENCE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Extended Technology Preference': ");
            goto error_out;
        }

        {
            guint16 tmp;

            tmp = (guint16) input->arg_extended_technology_preference;
            /* Write the guint16 variable to the buffer */
            if (!qmi_message_tlv_write_guint16 (self, QMI_ENDIAN_LITTLE, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Extended Technology Preference': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Extended Technology Preference': ");
            goto error_out;
        }
    }

    /* Try to add the 'Enable Autoconnect' TLV */
    if (input->arg_enable_autoconnect_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_ENABLE_AUTOCONNECT, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Enable Autoconnect': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_enable_autoconnect;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Enable Autoconnect': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Enable Autoconnect': ");
            goto error_out;
        }
    }

    /* Try to add the 'Profile Index 3GPP2' TLV */
    if (input->arg_profile_index_3gpp2_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PROFILE_INDEX_3GPP2, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Profile Index 3GPP2': ");
            goto error_out;
        }

        /* Write the guint8 variable to the buffer */
        if (!qmi_message_tlv_write_guint8 (self, input->arg_profile_index_3gpp2, error)) {
            g_prefix_error (error, "Cannot write integer in TLV 'Profile Index 3GPP2': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Profile Index 3GPP2': ");
            goto error_out;
        }
    }

    /* Try to add the 'Profile Index 3GPP' TLV */
    if (input->arg_profile_index_3gpp_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PROFILE_INDEX_3GPP, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Profile Index 3GPP': ");
            goto error_out;
        }

        /* Write the guint8 variable to the buffer */
        if (!qmi_message_tlv_write_guint8 (self, input->arg_profile_index_3gpp, error)) {
            g_prefix_error (error, "Cannot write integer in TLV 'Profile Index 3GPP': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Profile Index 3GPP': ");
            goto error_out;
        }
    }

    /* Try to add the 'Technology Preference' TLV */
    if (input->arg_technology_preference_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_TECHNOLOGY_PREFERENCE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Technology Preference': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_technology_preference;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Technology Preference': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Technology Preference': ");
            goto error_out;
        }
    }

    /* Try to add the 'IP Family Preference' TLV */
    if (input->arg_ip_family_preference_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_IP_FAMILY_PREFERENCE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'IP Family Preference': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_ip_family_preference;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'IP Family Preference': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'IP Family Preference': ");
            goto error_out;
        }
    }

    /* Try to add the 'Password' TLV */
    if (input->arg_password_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PASSWORD, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Password': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_string (self, 0, input->arg_password, -1, error)) {
            g_prefix_error (error, "Cannot write string in TLV 'Password': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Password': ");
            goto error_out;
        }
    }

    /* Try to add the 'Username' TLV */
    if (input->arg_username_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_USERNAME, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Username': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_string (self, 0, input->arg_username, -1, error)) {
            g_prefix_error (error, "Cannot write string in TLV 'Username': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Username': ");
            goto error_out;
        }
    }

    /* Try to add the 'Authentication Preference' TLV */
    if (input->arg_authentication_preference_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_AUTHENTICATION_PREFERENCE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Authentication Preference': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_authentication_preference;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Authentication Preference': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Authentication Preference': ");
            goto error_out;
        }
    }

    /* Try to add the 'IPv4 Address Preference' TLV */
    if (input->arg_ipv4_address_preference_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_IPV4_ADDRESS_PREFERENCE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'IPv4 Address Preference': ");
            goto error_out;
        }

        /* Write the guint32 variable to the buffer */
        if (!qmi_message_tlv_write_guint32 (self, QMI_ENDIAN_LITTLE, input->arg_ipv4_address_preference, error)) {
            g_prefix_error (error, "Cannot write integer in TLV 'IPv4 Address Preference': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'IPv4 Address Preference': ");
            goto error_out;
        }
    }

    /* Try to add the 'APN' TLV */
    if (input->arg_apn_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_APN, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'APN': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_string (self, 0, input->arg_apn, -1, error)) {
            g_prefix_error (error, "Cannot write string in TLV 'APN': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'APN': ");
            goto error_out;
        }
    }

    /* Try to add the 'Secondary NBNS Address Preference' TLV */
    if (input->arg_secondary_nbns_address_preference_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_SECONDARY_NBNS_ADDRESS_PREFERENCE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Secondary NBNS Address Preference': ");
            goto error_out;
        }

        /* Write the guint32 variable to the buffer */
        if (!qmi_message_tlv_write_guint32 (self, QMI_ENDIAN_LITTLE, input->arg_secondary_nbns_address_preference, error)) {
            g_prefix_error (error, "Cannot write integer in TLV 'Secondary NBNS Address Preference': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Secondary NBNS Address Preference': ");
            goto error_out;
        }
    }

    /* Try to add the 'Primary NBNS Address Preference' TLV */
    if (input->arg_primary_nbns_address_preference_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PRIMARY_NBNS_ADDRESS_PREFERENCE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Primary NBNS Address Preference': ");
            goto error_out;
        }

        /* Write the guint32 variable to the buffer */
        if (!qmi_message_tlv_write_guint32 (self, QMI_ENDIAN_LITTLE, input->arg_primary_nbns_address_preference, error)) {
            g_prefix_error (error, "Cannot write integer in TLV 'Primary NBNS Address Preference': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Primary NBNS Address Preference': ");
            goto error_out;
        }
    }

    /* Try to add the 'Secondary DNS Address Preference' TLV */
    if (input->arg_secondary_dns_address_preference_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_SECONDARY_DNS_ADDRESS_PREFERENCE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Secondary DNS Address Preference': ");
            goto error_out;
        }

        /* Write the guint32 variable to the buffer */
        if (!qmi_message_tlv_write_guint32 (self, QMI_ENDIAN_LITTLE, input->arg_secondary_dns_address_preference, error)) {
            g_prefix_error (error, "Cannot write integer in TLV 'Secondary DNS Address Preference': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Secondary DNS Address Preference': ");
            goto error_out;
        }
    }

    /* Try to add the 'Primary DNS Address Preference' TLV */
    if (input->arg_primary_dns_address_preference_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PRIMARY_DNS_ADDRESS_PREFERENCE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Primary DNS Address Preference': ");
            goto error_out;
        }

        /* Write the guint32 variable to the buffer */
        if (!qmi_message_tlv_write_guint32 (self, QMI_ENDIAN_LITTLE, input->arg_primary_dns_address_preference, error)) {
            g_prefix_error (error, "Cannot write integer in TLV 'Primary DNS Address Preference': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Primary DNS Address Preference': ");
            goto error_out;
        }
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

struct _QmiMessageWdsStartNetworkOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;

    /* Packet Data Handle */
    gboolean arg_packet_data_handle_set;
    guint32 arg_packet_data_handle;

    /* Call End Reason */
    gboolean arg_call_end_reason_set;
    guint16 arg_call_end_reason;

    /* Verbose Call End Reason */
    gboolean arg_verbose_call_end_reason_set;
    guint16 arg_verbose_call_end_reason_type;
    gint16 arg_verbose_call_end_reason_reason;
};

#define QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_RESULT 0x02
#define QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_PACKET_DATA_HANDLE 0x01
#define QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_CALL_END_REASON 0x10
#define QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_VERBOSE_CALL_END_REASON 0x11

/**
 * qmi_message_wds_start_network_output_get_result:
 * @self: a QmiMessageWdsStartNetworkOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_start_network_output_get_result (
    QmiMessageWdsStartNetworkOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

/**
 * qmi_message_wds_start_network_output_get_packet_data_handle:
 * @self: a #QmiMessageWdsStartNetworkOutput.
 * @value_packet_data_handle: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Packet Data Handle' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_output_get_packet_data_handle (
    QmiMessageWdsStartNetworkOutput *self,
    guint32 *value_packet_data_handle,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_packet_data_handle_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Packet Data Handle' was not found in the message");
        return FALSE;
    }

    if (value_packet_data_handle)
        *value_packet_data_handle = self->arg_packet_data_handle;

    return TRUE;
}

/**
 * qmi_message_wds_start_network_output_get_call_end_reason:
 * @self: a #QmiMessageWdsStartNetworkOutput.
 * @value_call_end_reason: a placeholder for the output #QmiWdsCallEndReason, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Call End Reason' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_output_get_call_end_reason (
    QmiMessageWdsStartNetworkOutput *self,
    QmiWdsCallEndReason *value_call_end_reason,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_call_end_reason_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Call End Reason' was not found in the message");
        return FALSE;
    }

    if (value_call_end_reason)
        *value_call_end_reason = (QmiWdsCallEndReason)(self->arg_call_end_reason);

    return TRUE;
}

/**
 * qmi_message_wds_start_network_output_get_verbose_call_end_reason:
 * @self: a #QmiMessageWdsStartNetworkOutput.
 * @value_verbose_call_end_reason_type: a placeholder for the output #QmiWdsVerboseCallEndReasonType, or %NULL if not required.
 * @value_verbose_call_end_reason_reason: a placeholder for the output #gint16, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Verbose Call End Reason' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_start_network_output_get_verbose_call_end_reason (
    QmiMessageWdsStartNetworkOutput *self,
    QmiWdsVerboseCallEndReasonType *value_verbose_call_end_reason_type,
    gint16 *value_verbose_call_end_reason_reason,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_verbose_call_end_reason_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Verbose Call End Reason' was not found in the message");
        return FALSE;
    }

    if (value_verbose_call_end_reason_type)
        *value_verbose_call_end_reason_type = (QmiWdsVerboseCallEndReasonType)(self->arg_verbose_call_end_reason_type);
    if (value_verbose_call_end_reason_reason)
        *value_verbose_call_end_reason_reason = self->arg_verbose_call_end_reason_reason;

    return TRUE;
}

GType
qmi_message_wds_start_network_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsStartNetworkOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_start_network_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_start_network_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_start_network_output_ref:
 * @self: a #QmiMessageWdsStartNetworkOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsStartNetworkOutput *
qmi_message_wds_start_network_output_ref (QmiMessageWdsStartNetworkOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_start_network_output_unref:
 * @self: a #QmiMessageWdsStartNetworkOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_start_network_output_unref (QmiMessageWdsStartNetworkOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsStartNetworkOutput, self);
    }
}

static gchar *
qmi_message_wds_start_network_input_call_type_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_CALL_TYPE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_CALL_TYPE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_call_type_get_string ((QmiWdsCallType)tmp));
#elif defined  __QMI_WDS_CALL_TYPE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_call_type_build_string_from_mask ((QmiWdsCallType)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsCallType
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_extended_technology_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_EXTENDED_TECHNOLOGY_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_EXTENDED_TECHNOLOGY_PREFERENCE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_extended_technology_preference_get_string ((QmiWdsExtendedTechnologyPreference)tmp));
#elif defined  __QMI_WDS_EXTENDED_TECHNOLOGY_PREFERENCE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_extended_technology_preference_build_string_from_mask ((QmiWdsExtendedTechnologyPreference)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsExtendedTechnologyPreference
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_enable_autoconnect_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_ENABLE_AUTOCONNECT, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_profile_index_3gpp2_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PROFILE_INDEX_3GPP2, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_profile_index_3gpp_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PROFILE_INDEX_3GPP, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_technology_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_TECHNOLOGY_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_TECHNOLOGY_PREFERENCE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_technology_preference_get_string ((QmiWdsTechnologyPreference)tmp));
#elif defined  __QMI_WDS_TECHNOLOGY_PREFERENCE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_technology_preference_build_string_from_mask ((QmiWdsTechnologyPreference)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsTechnologyPreference
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_ip_family_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_IP_FAMILY_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_IP_FAMILY_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_ip_family_get_string ((QmiWdsIpFamily)tmp));
#elif defined  __QMI_WDS_IP_FAMILY_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_ip_family_build_string_from_mask ((QmiWdsIpFamily)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsIpFamily
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_password_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PASSWORD, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_username_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_USERNAME, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_authentication_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_AUTHENTICATION_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_AUTHENTICATION_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_authentication_get_string ((QmiWdsAuthentication)tmp));
#elif defined  __QMI_WDS_AUTHENTICATION_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_authentication_build_string_from_mask ((QmiWdsAuthentication)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsAuthentication
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_ipv4_address_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_IPV4_ADDRESS_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_apn_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_APN, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_secondary_nbns_address_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_SECONDARY_NBNS_ADDRESS_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_primary_nbns_address_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PRIMARY_NBNS_ADDRESS_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_secondary_dns_address_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_SECONDARY_DNS_ADDRESS_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_input_primary_dns_address_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PRIMARY_DNS_ADDRESS_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_output_packet_data_handle_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_PACKET_DATA_HANDLE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_output_call_end_reason_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_CALL_END_REASON, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_CALL_END_REASON_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_call_end_reason_get_string ((QmiWdsCallEndReason)tmp));
#elif defined  __QMI_WDS_CALL_END_REASON_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_call_end_reason_build_string_from_mask ((QmiWdsCallEndReason)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsCallEndReason
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_start_network_output_verbose_call_end_reason_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_VERBOSE_CALL_END_REASON, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " type = '");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_VERBOSE_CALL_END_REASON_TYPE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_verbose_call_end_reason_type_get_string ((QmiWdsVerboseCallEndReasonType)tmp));
#elif defined  __QMI_WDS_VERBOSE_CALL_END_REASON_TYPE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_verbose_call_end_reason_type_build_string_from_mask ((QmiWdsVerboseCallEndReasonType)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsVerboseCallEndReasonType
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " reason = '");

    {
        gint16 tmp;

        if (!qmi_message_tlv_read_gint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GINT16_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_start_network_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_start_network_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_start_network_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_CALL_TYPE:
            tlv_type_str = "Call Type";
            translated_value = qmi_message_wds_start_network_input_call_type_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_EXTENDED_TECHNOLOGY_PREFERENCE:
            tlv_type_str = "Extended Technology Preference";
            translated_value = qmi_message_wds_start_network_input_extended_technology_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_ENABLE_AUTOCONNECT:
            tlv_type_str = "Enable Autoconnect";
            translated_value = qmi_message_wds_start_network_input_enable_autoconnect_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PROFILE_INDEX_3GPP2:
            tlv_type_str = "Profile Index 3GPP2";
            translated_value = qmi_message_wds_start_network_input_profile_index_3gpp2_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PROFILE_INDEX_3GPP:
            tlv_type_str = "Profile Index 3GPP";
            translated_value = qmi_message_wds_start_network_input_profile_index_3gpp_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_TECHNOLOGY_PREFERENCE:
            tlv_type_str = "Technology Preference";
            translated_value = qmi_message_wds_start_network_input_technology_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_IP_FAMILY_PREFERENCE:
            tlv_type_str = "IP Family Preference";
            translated_value = qmi_message_wds_start_network_input_ip_family_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PASSWORD:
            tlv_type_str = "Password";
            translated_value = qmi_message_wds_start_network_input_password_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_USERNAME:
            tlv_type_str = "Username";
            translated_value = qmi_message_wds_start_network_input_username_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_AUTHENTICATION_PREFERENCE:
            tlv_type_str = "Authentication Preference";
            translated_value = qmi_message_wds_start_network_input_authentication_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_IPV4_ADDRESS_PREFERENCE:
            tlv_type_str = "IPv4 Address Preference";
            translated_value = qmi_message_wds_start_network_input_ipv4_address_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_APN:
            tlv_type_str = "APN";
            translated_value = qmi_message_wds_start_network_input_apn_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_SECONDARY_NBNS_ADDRESS_PREFERENCE:
            tlv_type_str = "Secondary NBNS Address Preference";
            translated_value = qmi_message_wds_start_network_input_secondary_nbns_address_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PRIMARY_NBNS_ADDRESS_PREFERENCE:
            tlv_type_str = "Primary NBNS Address Preference";
            translated_value = qmi_message_wds_start_network_input_primary_nbns_address_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_SECONDARY_DNS_ADDRESS_PREFERENCE:
            tlv_type_str = "Secondary DNS Address Preference";
            translated_value = qmi_message_wds_start_network_input_secondary_dns_address_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_INPUT_TLV_PRIMARY_DNS_ADDRESS_PREFERENCE:
            tlv_type_str = "Primary DNS Address Preference";
            translated_value = qmi_message_wds_start_network_input_primary_dns_address_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_PACKET_DATA_HANDLE:
            tlv_type_str = "Packet Data Handle";
            translated_value = qmi_message_wds_start_network_output_packet_data_handle_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_CALL_END_REASON:
            tlv_type_str = "Call End Reason";
            translated_value = qmi_message_wds_start_network_output_call_end_reason_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_VERBOSE_CALL_END_REASON:
            tlv_type_str = "Verbose Call End Reason";
            translated_value = qmi_message_wds_start_network_output_verbose_call_end_reason_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_start_network_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Start Network\" (0x0020)\n",
                            line_prefix);

    {
        struct message_start_network_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_start_network_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsStartNetworkOutput *
__qmi_message_wds_start_network_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsStartNetworkOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_START_NETWORK, NULL);

    self = g_slice_new0 (QmiMessageWdsStartNetworkOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_start_network_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_start_network_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_PACKET_DATA_HANDLE, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Packet Data Handle TLV: ");
                qmi_message_wds_start_network_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_packet_data_handle), error))
                goto qmi_message_wds_start_network_output_packet_data_handle_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Packet Data Handle' TLV", offset);
            }

            self->arg_packet_data_handle_set = TRUE;

qmi_message_wds_start_network_output_packet_data_handle_out:
            if (!self->arg_packet_data_handle_set) {
                qmi_message_wds_start_network_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status != QMI_STATUS_SUCCESS))
            break;
        /* Prerequisite.... */
        if (!(self->arg_result.error_code == QMI_PROTOCOL_ERROR_CALL_FAILED))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_CALL_END_REASON, NULL, NULL)) == 0) {
                goto qmi_message_wds_start_network_output_call_end_reason_out;
            }
            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, NULL))
                    goto qmi_message_wds_start_network_output_call_end_reason_out;
                self->arg_call_end_reason = (QmiWdsCallEndReason)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Call End Reason' TLV", offset);
            }

            self->arg_call_end_reason_set = TRUE;

qmi_message_wds_start_network_output_call_end_reason_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status != QMI_STATUS_SUCCESS))
            break;
        /* Prerequisite.... */
        if (!(self->arg_result.error_code == QMI_PROTOCOL_ERROR_CALL_FAILED))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_START_NETWORK_OUTPUT_TLV_VERBOSE_CALL_END_REASON, NULL, NULL)) == 0) {
                goto qmi_message_wds_start_network_output_verbose_call_end_reason_out;
            }
            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, NULL))
                    goto qmi_message_wds_start_network_output_verbose_call_end_reason_out;
                self->arg_verbose_call_end_reason_type = (QmiWdsVerboseCallEndReasonType)tmp;
            }
            if (!qmi_message_tlv_read_gint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_verbose_call_end_reason_reason), NULL))
                goto qmi_message_wds_start_network_output_verbose_call_end_reason_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Verbose Call End Reason' TLV", offset);
            }

            self->arg_verbose_call_end_reason_set = TRUE;

qmi_message_wds_start_network_output_verbose_call_end_reason_out:
            ;

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Stop Network */


/* --- Input -- */

struct _QmiMessageWdsStopNetworkInput {
    volatile gint ref_count;

    /* Disable Autoconnect */
    gboolean arg_disable_autoconnect_set;
    guint8 arg_disable_autoconnect;

    /* Packet Data Handle */
    gboolean arg_packet_data_handle_set;
    guint32 arg_packet_data_handle;
};

#define QMI_MESSAGE_WDS_STOP_NETWORK_INPUT_TLV_DISABLE_AUTOCONNECT 0x10
#define QMI_MESSAGE_WDS_STOP_NETWORK_INPUT_TLV_PACKET_DATA_HANDLE 0x01

/**
 * qmi_message_wds_stop_network_input_get_disable_autoconnect:
 * @self: a #QmiMessageWdsStopNetworkInput.
 * @value_disable_autoconnect: a placeholder for the output #gboolean, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Disable Autoconnect' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_stop_network_input_get_disable_autoconnect (
    QmiMessageWdsStopNetworkInput *self,
    gboolean *value_disable_autoconnect,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_disable_autoconnect_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Disable Autoconnect' was not found in the message");
        return FALSE;
    }

    if (value_disable_autoconnect)
        *value_disable_autoconnect = (gboolean)(self->arg_disable_autoconnect);

    return TRUE;
}

/**
 * qmi_message_wds_stop_network_input_set_disable_autoconnect:
 * @self: a #QmiMessageWdsStopNetworkInput.
 * @value_disable_autoconnect: a #gboolean.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Disable Autoconnect' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_stop_network_input_set_disable_autoconnect (
    QmiMessageWdsStopNetworkInput *self,
    gboolean value_disable_autoconnect,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_disable_autoconnect = (guint8)(value_disable_autoconnect);
    self->arg_disable_autoconnect_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_stop_network_input_get_packet_data_handle:
 * @self: a #QmiMessageWdsStopNetworkInput.
 * @value_packet_data_handle: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Packet Data Handle' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_stop_network_input_get_packet_data_handle (
    QmiMessageWdsStopNetworkInput *self,
    guint32 *value_packet_data_handle,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_packet_data_handle_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Packet Data Handle' was not found in the message");
        return FALSE;
    }

    if (value_packet_data_handle)
        *value_packet_data_handle = self->arg_packet_data_handle;

    return TRUE;
}

/**
 * qmi_message_wds_stop_network_input_set_packet_data_handle:
 * @self: a #QmiMessageWdsStopNetworkInput.
 * @value_packet_data_handle: a #guint32.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Packet Data Handle' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_stop_network_input_set_packet_data_handle (
    QmiMessageWdsStopNetworkInput *self,
    guint32 value_packet_data_handle,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_packet_data_handle = value_packet_data_handle;
    self->arg_packet_data_handle_set = TRUE;

    return TRUE;
}

GType
qmi_message_wds_stop_network_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsStopNetworkInput"),
                                          (GBoxedCopyFunc) qmi_message_wds_stop_network_input_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_stop_network_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_stop_network_input_ref:
 * @self: a #QmiMessageWdsStopNetworkInput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsStopNetworkInput *
qmi_message_wds_stop_network_input_ref (QmiMessageWdsStopNetworkInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_stop_network_input_unref:
 * @self: a #QmiMessageWdsStopNetworkInput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_stop_network_input_unref (QmiMessageWdsStopNetworkInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsStopNetworkInput, self);
    }
}

/**
 * qmi_message_wds_stop_network_input_new:
 *
 * Allocates a new #QmiMessageWdsStopNetworkInput.
 *
 * Returns: the newly created #QmiMessageWdsStopNetworkInput. The returned value should be freed with qmi_message_wds_stop_network_input_unref().
 */
QmiMessageWdsStopNetworkInput *
qmi_message_wds_stop_network_input_new (void)
{
    QmiMessageWdsStopNetworkInput *self;

    self = g_slice_new0 (QmiMessageWdsStopNetworkInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_wds_stop_network_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageWdsStopNetworkInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_STOP_NETWORK);

    /* All TLVs are optional, we allow NULL input */
    if (!input)
        return self;

    /* Try to add the 'Disable Autoconnect' TLV */
    if (input->arg_disable_autoconnect_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_STOP_NETWORK_INPUT_TLV_DISABLE_AUTOCONNECT, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Disable Autoconnect': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_disable_autoconnect;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Disable Autoconnect': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Disable Autoconnect': ");
            goto error_out;
        }
    }

    /* Try to add the 'Packet Data Handle' TLV */
    if (input->arg_packet_data_handle_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_STOP_NETWORK_INPUT_TLV_PACKET_DATA_HANDLE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Packet Data Handle': ");
            goto error_out;
        }

        /* Write the guint32 variable to the buffer */
        if (!qmi_message_tlv_write_guint32 (self, QMI_ENDIAN_LITTLE, input->arg_packet_data_handle, error)) {
            g_prefix_error (error, "Cannot write integer in TLV 'Packet Data Handle': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Packet Data Handle': ");
            goto error_out;
        }
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

struct _QmiMessageWdsStopNetworkOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;
};

#define QMI_MESSAGE_WDS_STOP_NETWORK_OUTPUT_TLV_RESULT 0x02

/**
 * qmi_message_wds_stop_network_output_get_result:
 * @self: a QmiMessageWdsStopNetworkOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_stop_network_output_get_result (
    QmiMessageWdsStopNetworkOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

GType
qmi_message_wds_stop_network_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsStopNetworkOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_stop_network_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_stop_network_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_stop_network_output_ref:
 * @self: a #QmiMessageWdsStopNetworkOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsStopNetworkOutput *
qmi_message_wds_stop_network_output_ref (QmiMessageWdsStopNetworkOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_stop_network_output_unref:
 * @self: a #QmiMessageWdsStopNetworkOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_stop_network_output_unref (QmiMessageWdsStopNetworkOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsStopNetworkOutput, self);
    }
}

static gchar *
qmi_message_wds_stop_network_input_disable_autoconnect_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_STOP_NETWORK_INPUT_TLV_DISABLE_AUTOCONNECT, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_stop_network_input_packet_data_handle_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_STOP_NETWORK_INPUT_TLV_PACKET_DATA_HANDLE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_stop_network_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_stop_network_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_stop_network_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_WDS_STOP_NETWORK_INPUT_TLV_DISABLE_AUTOCONNECT:
            tlv_type_str = "Disable Autoconnect";
            translated_value = qmi_message_wds_stop_network_input_disable_autoconnect_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_STOP_NETWORK_INPUT_TLV_PACKET_DATA_HANDLE:
            tlv_type_str = "Packet Data Handle";
            translated_value = qmi_message_wds_stop_network_input_packet_data_handle_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_STOP_NETWORK_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_stop_network_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Stop Network\" (0x0021)\n",
                            line_prefix);

    {
        struct message_stop_network_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_stop_network_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsStopNetworkOutput *
__qmi_message_wds_stop_network_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsStopNetworkOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_STOP_NETWORK, NULL);

    self = g_slice_new0 (QmiMessageWdsStopNetworkOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_STOP_NETWORK_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_stop_network_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_stop_network_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Get Packet Service Status */


/* --- Input -- */

/* Note: no fields in the Input container */

static QmiMessage *
__qmi_message_wds_get_packet_service_status_request_create (
    guint16 transaction_id,
    guint8 cid,
    gpointer unused,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_GET_PACKET_SERVICE_STATUS);

    return self;
}

/* --- Output -- */

struct _QmiMessageWdsGetPacketServiceStatusOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;

    /* Connection Status */
    gboolean arg_connection_status_set;
    guint8 arg_connection_status;
};

#define QMI_MESSAGE_WDS_GET_PACKET_SERVICE_STATUS_OUTPUT_TLV_RESULT 0x02
#define QMI_MESSAGE_WDS_GET_PACKET_SERVICE_STATUS_OUTPUT_TLV_CONNECTION_STATUS 0x01

/**
 * qmi_message_wds_get_packet_service_status_output_get_result:
 * @self: a QmiMessageWdsGetPacketServiceStatusOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_get_packet_service_status_output_get_result (
    QmiMessageWdsGetPacketServiceStatusOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

/**
 * qmi_message_wds_get_packet_service_status_output_get_connection_status:
 * @self: a #QmiMessageWdsGetPacketServiceStatusOutput.
 * @value_connection_status: a placeholder for the output #QmiWdsConnectionStatus, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Connection Status' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_service_status_output_get_connection_status (
    QmiMessageWdsGetPacketServiceStatusOutput *self,
    QmiWdsConnectionStatus *value_connection_status,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_connection_status_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Connection Status' was not found in the message");
        return FALSE;
    }

    if (value_connection_status)
        *value_connection_status = (QmiWdsConnectionStatus)(self->arg_connection_status);

    return TRUE;
}

GType
qmi_message_wds_get_packet_service_status_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetPacketServiceStatusOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_packet_service_status_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_packet_service_status_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_packet_service_status_output_ref:
 * @self: a #QmiMessageWdsGetPacketServiceStatusOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetPacketServiceStatusOutput *
qmi_message_wds_get_packet_service_status_output_ref (QmiMessageWdsGetPacketServiceStatusOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_packet_service_status_output_unref:
 * @self: a #QmiMessageWdsGetPacketServiceStatusOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_packet_service_status_output_unref (QmiMessageWdsGetPacketServiceStatusOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsGetPacketServiceStatusOutput, self);
    }
}

static gchar *
qmi_message_wds_get_packet_service_status_output_connection_status_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_SERVICE_STATUS_OUTPUT_TLV_CONNECTION_STATUS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_CONNECTION_STATUS_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_connection_status_get_string ((QmiWdsConnectionStatus)tmp));
#elif defined  __QMI_WDS_CONNECTION_STATUS_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_connection_status_build_string_from_mask ((QmiWdsConnectionStatus)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsConnectionStatus
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_get_packet_service_status_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_get_packet_service_status_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_get_packet_service_status_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_PACKET_SERVICE_STATUS_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PACKET_SERVICE_STATUS_OUTPUT_TLV_CONNECTION_STATUS:
            tlv_type_str = "Connection Status";
            translated_value = qmi_message_wds_get_packet_service_status_output_connection_status_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_get_packet_service_status_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Get Packet Service Status\" (0x0022)\n",
                            line_prefix);

    {
        struct message_get_packet_service_status_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_get_packet_service_status_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsGetPacketServiceStatusOutput *
__qmi_message_wds_get_packet_service_status_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsGetPacketServiceStatusOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_GET_PACKET_SERVICE_STATUS, NULL);

    self = g_slice_new0 (QmiMessageWdsGetPacketServiceStatusOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_SERVICE_STATUS_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_get_packet_service_status_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_get_packet_service_status_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_SERVICE_STATUS_OUTPUT_TLV_CONNECTION_STATUS, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Connection Status TLV: ");
                qmi_message_wds_get_packet_service_status_output_unref (self);
                return NULL;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, error))
                    goto qmi_message_wds_get_packet_service_status_output_connection_status_out;
                self->arg_connection_status = (QmiWdsConnectionStatus)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Connection Status' TLV", offset);
            }

            self->arg_connection_status_set = TRUE;

qmi_message_wds_get_packet_service_status_output_connection_status_out:
            if (!self->arg_connection_status_set) {
                qmi_message_wds_get_packet_service_status_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* INDICATION: Qmi Indication WDS Packet Service Status */


/* --- Output -- */

struct _QmiIndicationWdsPacketServiceStatusOutput {
    volatile gint ref_count;

    /* Extended Technology Preference */
    gboolean arg_extended_technology_preference_set;
    guint16 arg_extended_technology_preference;

    /* IP Family */
    gboolean arg_ip_family_set;
    guint8 arg_ip_family;

    /* Verbose Call End Reason */
    gboolean arg_verbose_call_end_reason_set;
    guint16 arg_verbose_call_end_reason_type;
    gint16 arg_verbose_call_end_reason_reason;

    /* Call End Reason */
    gboolean arg_call_end_reason_set;
    guint16 arg_call_end_reason;

    /* Connection Status */
    gboolean arg_connection_status_set;
    guint8 arg_connection_status_status;
    guint8 arg_connection_status_reconfiguration_required;
};

#define QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_EXTENDED_TECHNOLOGY_PREFERENCE 0x34
#define QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_IP_FAMILY 0x12
#define QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_VERBOSE_CALL_END_REASON 0x11
#define QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_CALL_END_REASON 0x10
#define QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_CONNECTION_STATUS 0x01

/**
 * qmi_indication_wds_packet_service_status_output_get_extended_technology_preference:
 * @self: a #QmiIndicationWdsPacketServiceStatusOutput.
 * @value_extended_technology_preference: a placeholder for the output #QmiWdsExtendedTechnologyPreference, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Extended Technology Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_indication_wds_packet_service_status_output_get_extended_technology_preference (
    QmiIndicationWdsPacketServiceStatusOutput *self,
    QmiWdsExtendedTechnologyPreference *value_extended_technology_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_extended_technology_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Extended Technology Preference' was not found in the message");
        return FALSE;
    }

    if (value_extended_technology_preference)
        *value_extended_technology_preference = (QmiWdsExtendedTechnologyPreference)(self->arg_extended_technology_preference);

    return TRUE;
}

/**
 * qmi_indication_wds_packet_service_status_output_get_ip_family:
 * @self: a #QmiIndicationWdsPacketServiceStatusOutput.
 * @value_ip_family: a placeholder for the output #QmiWdsIpFamily, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IP Family' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_indication_wds_packet_service_status_output_get_ip_family (
    QmiIndicationWdsPacketServiceStatusOutput *self,
    QmiWdsIpFamily *value_ip_family,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ip_family_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IP Family' was not found in the message");
        return FALSE;
    }

    if (value_ip_family)
        *value_ip_family = (QmiWdsIpFamily)(self->arg_ip_family);

    return TRUE;
}

/**
 * qmi_indication_wds_packet_service_status_output_get_verbose_call_end_reason:
 * @self: a #QmiIndicationWdsPacketServiceStatusOutput.
 * @value_verbose_call_end_reason_type: a placeholder for the output #QmiWdsVerboseCallEndReasonType, or %NULL if not required.
 * @value_verbose_call_end_reason_reason: a placeholder for the output #gint16, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Verbose Call End Reason' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_indication_wds_packet_service_status_output_get_verbose_call_end_reason (
    QmiIndicationWdsPacketServiceStatusOutput *self,
    QmiWdsVerboseCallEndReasonType *value_verbose_call_end_reason_type,
    gint16 *value_verbose_call_end_reason_reason,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_verbose_call_end_reason_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Verbose Call End Reason' was not found in the message");
        return FALSE;
    }

    if (value_verbose_call_end_reason_type)
        *value_verbose_call_end_reason_type = (QmiWdsVerboseCallEndReasonType)(self->arg_verbose_call_end_reason_type);
    if (value_verbose_call_end_reason_reason)
        *value_verbose_call_end_reason_reason = self->arg_verbose_call_end_reason_reason;

    return TRUE;
}

/**
 * qmi_indication_wds_packet_service_status_output_get_call_end_reason:
 * @self: a #QmiIndicationWdsPacketServiceStatusOutput.
 * @value_call_end_reason: a placeholder for the output #QmiWdsCallEndReason, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Call End Reason' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_indication_wds_packet_service_status_output_get_call_end_reason (
    QmiIndicationWdsPacketServiceStatusOutput *self,
    QmiWdsCallEndReason *value_call_end_reason,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_call_end_reason_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Call End Reason' was not found in the message");
        return FALSE;
    }

    if (value_call_end_reason)
        *value_call_end_reason = (QmiWdsCallEndReason)(self->arg_call_end_reason);

    return TRUE;
}

/**
 * qmi_indication_wds_packet_service_status_output_get_connection_status:
 * @self: a #QmiIndicationWdsPacketServiceStatusOutput.
 * @value_connection_status_status: a placeholder for the output #QmiWdsConnectionStatus, or %NULL if not required.
 * @value_connection_status_reconfiguration_required: a placeholder for the output #gboolean, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Connection Status' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_indication_wds_packet_service_status_output_get_connection_status (
    QmiIndicationWdsPacketServiceStatusOutput *self,
    QmiWdsConnectionStatus *value_connection_status_status,
    gboolean *value_connection_status_reconfiguration_required,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_connection_status_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Connection Status' was not found in the message");
        return FALSE;
    }

    if (value_connection_status_status)
        *value_connection_status_status = (QmiWdsConnectionStatus)(self->arg_connection_status_status);
    if (value_connection_status_reconfiguration_required)
        *value_connection_status_reconfiguration_required = (gboolean)(self->arg_connection_status_reconfiguration_required);

    return TRUE;
}

GType
qmi_indication_wds_packet_service_status_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiIndicationWdsPacketServiceStatusOutput"),
                                          (GBoxedCopyFunc) qmi_indication_wds_packet_service_status_output_ref,
                                          (GBoxedFreeFunc) qmi_indication_wds_packet_service_status_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_indication_wds_packet_service_status_output_ref:
 * @self: a #QmiIndicationWdsPacketServiceStatusOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiIndicationWdsPacketServiceStatusOutput *
qmi_indication_wds_packet_service_status_output_ref (QmiIndicationWdsPacketServiceStatusOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_indication_wds_packet_service_status_output_unref:
 * @self: a #QmiIndicationWdsPacketServiceStatusOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_indication_wds_packet_service_status_output_unref (QmiIndicationWdsPacketServiceStatusOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiIndicationWdsPacketServiceStatusOutput, self);
    }
}

static gchar *
qmi_indication_wds_packet_service_status_output_extended_technology_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_EXTENDED_TECHNOLOGY_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_EXTENDED_TECHNOLOGY_PREFERENCE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_extended_technology_preference_get_string ((QmiWdsExtendedTechnologyPreference)tmp));
#elif defined  __QMI_WDS_EXTENDED_TECHNOLOGY_PREFERENCE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_extended_technology_preference_build_string_from_mask ((QmiWdsExtendedTechnologyPreference)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsExtendedTechnologyPreference
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_indication_wds_packet_service_status_output_ip_family_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_IP_FAMILY, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_IP_FAMILY_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_ip_family_get_string ((QmiWdsIpFamily)tmp));
#elif defined  __QMI_WDS_IP_FAMILY_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_ip_family_build_string_from_mask ((QmiWdsIpFamily)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsIpFamily
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_indication_wds_packet_service_status_output_verbose_call_end_reason_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_VERBOSE_CALL_END_REASON, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " type = '");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_VERBOSE_CALL_END_REASON_TYPE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_verbose_call_end_reason_type_get_string ((QmiWdsVerboseCallEndReasonType)tmp));
#elif defined  __QMI_WDS_VERBOSE_CALL_END_REASON_TYPE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_verbose_call_end_reason_type_build_string_from_mask ((QmiWdsVerboseCallEndReasonType)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsVerboseCallEndReasonType
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " reason = '");

    {
        gint16 tmp;

        if (!qmi_message_tlv_read_gint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GINT16_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_indication_wds_packet_service_status_output_call_end_reason_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_CALL_END_REASON, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_CALL_END_REASON_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_call_end_reason_get_string ((QmiWdsCallEndReason)tmp));
#elif defined  __QMI_WDS_CALL_END_REASON_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_call_end_reason_build_string_from_mask ((QmiWdsCallEndReason)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsCallEndReason
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_indication_wds_packet_service_status_output_connection_status_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_CONNECTION_STATUS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " status = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_CONNECTION_STATUS_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_connection_status_get_string ((QmiWdsConnectionStatus)tmp));
#elif defined  __QMI_WDS_CONNECTION_STATUS_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_connection_status_build_string_from_mask ((QmiWdsConnectionStatus)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsConnectionStatus
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " reconfiguration_required = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct indication_packet_service_status_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
indication_packet_service_status_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct indication_packet_service_status_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    {
        switch (type) {
        case QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_EXTENDED_TECHNOLOGY_PREFERENCE:
            tlv_type_str = "Extended Technology Preference";
            translated_value = qmi_indication_wds_packet_service_status_output_extended_technology_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_IP_FAMILY:
            tlv_type_str = "IP Family";
            translated_value = qmi_indication_wds_packet_service_status_output_ip_family_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_VERBOSE_CALL_END_REASON:
            tlv_type_str = "Verbose Call End Reason";
            translated_value = qmi_indication_wds_packet_service_status_output_verbose_call_end_reason_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_CALL_END_REASON:
            tlv_type_str = "Call End Reason";
            translated_value = qmi_indication_wds_packet_service_status_output_call_end_reason_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_CONNECTION_STATUS:
            tlv_type_str = "Connection Status";
            translated_value = qmi_indication_wds_packet_service_status_output_connection_status_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
indication_packet_service_status_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Packet Service Status\" (0x0022)\n",
                            line_prefix);

    {
        struct indication_packet_service_status_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)indication_packet_service_status_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiIndicationWdsPacketServiceStatusOutput *
__qmi_indication_wds_packet_service_status_indication_parse (
    QmiMessage *message,
    GError **error)
{
    QmiIndicationWdsPacketServiceStatusOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_INDICATION_WDS_PACKET_SERVICE_STATUS, NULL);

    self = g_slice_new0 (QmiIndicationWdsPacketServiceStatusOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_EXTENDED_TECHNOLOGY_PREFERENCE, NULL, NULL)) == 0) {
                goto qmi_indication_wds_packet_service_status_output_extended_technology_preference_out;
            }
            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, NULL))
                    goto qmi_indication_wds_packet_service_status_output_extended_technology_preference_out;
                self->arg_extended_technology_preference = (QmiWdsExtendedTechnologyPreference)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Extended Technology Preference' TLV", offset);
            }

            self->arg_extended_technology_preference_set = TRUE;

qmi_indication_wds_packet_service_status_output_extended_technology_preference_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_IP_FAMILY, NULL, NULL)) == 0) {
                goto qmi_indication_wds_packet_service_status_output_ip_family_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_indication_wds_packet_service_status_output_ip_family_out;
                self->arg_ip_family = (QmiWdsIpFamily)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IP Family' TLV", offset);
            }

            self->arg_ip_family_set = TRUE;

qmi_indication_wds_packet_service_status_output_ip_family_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_VERBOSE_CALL_END_REASON, NULL, NULL)) == 0) {
                goto qmi_indication_wds_packet_service_status_output_verbose_call_end_reason_out;
            }
            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, NULL))
                    goto qmi_indication_wds_packet_service_status_output_verbose_call_end_reason_out;
                self->arg_verbose_call_end_reason_type = (QmiWdsVerboseCallEndReasonType)tmp;
            }
            if (!qmi_message_tlv_read_gint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_verbose_call_end_reason_reason), NULL))
                goto qmi_indication_wds_packet_service_status_output_verbose_call_end_reason_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Verbose Call End Reason' TLV", offset);
            }

            self->arg_verbose_call_end_reason_set = TRUE;

qmi_indication_wds_packet_service_status_output_verbose_call_end_reason_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_CALL_END_REASON, NULL, NULL)) == 0) {
                goto qmi_indication_wds_packet_service_status_output_call_end_reason_out;
            }
            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, NULL))
                    goto qmi_indication_wds_packet_service_status_output_call_end_reason_out;
                self->arg_call_end_reason = (QmiWdsCallEndReason)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Call End Reason' TLV", offset);
            }

            self->arg_call_end_reason_set = TRUE;

qmi_indication_wds_packet_service_status_output_call_end_reason_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT_TLV_CONNECTION_STATUS, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Connection Status TLV: ");
                qmi_indication_wds_packet_service_status_output_unref (self);
                return NULL;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, error))
                    goto qmi_indication_wds_packet_service_status_output_connection_status_out;
                self->arg_connection_status_status = (QmiWdsConnectionStatus)tmp;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, error))
                    goto qmi_indication_wds_packet_service_status_output_connection_status_out;
                self->arg_connection_status_reconfiguration_required = (gboolean)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Connection Status' TLV", offset);
            }

            self->arg_connection_status_set = TRUE;

qmi_indication_wds_packet_service_status_output_connection_status_out:
            if (!self->arg_connection_status_set) {
                qmi_indication_wds_packet_service_status_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Get Packet Statistics */


/* --- Input -- */

struct _QmiMessageWdsGetPacketStatisticsInput {
    volatile gint ref_count;

    /* Mask */
    gboolean arg_mask_set;
    guint32 arg_mask;
};

#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_INPUT_TLV_MASK 0x01

/**
 * qmi_message_wds_get_packet_statistics_input_get_mask:
 * @self: a #QmiMessageWdsGetPacketStatisticsInput.
 * @value_mask: a placeholder for the output #QmiWdsPacketStatisticsMaskFlag, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Mask' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_input_get_mask (
    QmiMessageWdsGetPacketStatisticsInput *self,
    QmiWdsPacketStatisticsMaskFlag *value_mask,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_mask_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Mask' was not found in the message");
        return FALSE;
    }

    if (value_mask)
        *value_mask = (QmiWdsPacketStatisticsMaskFlag)(self->arg_mask);

    return TRUE;
}

/**
 * qmi_message_wds_get_packet_statistics_input_set_mask:
 * @self: a #QmiMessageWdsGetPacketStatisticsInput.
 * @value_mask: a #QmiWdsPacketStatisticsMaskFlag.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Mask' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_input_set_mask (
    QmiMessageWdsGetPacketStatisticsInput *self,
    QmiWdsPacketStatisticsMaskFlag value_mask,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_mask = (guint32)(value_mask);
    self->arg_mask_set = TRUE;

    return TRUE;
}

GType
qmi_message_wds_get_packet_statistics_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetPacketStatisticsInput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_packet_statistics_input_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_packet_statistics_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_packet_statistics_input_ref:
 * @self: a #QmiMessageWdsGetPacketStatisticsInput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetPacketStatisticsInput *
qmi_message_wds_get_packet_statistics_input_ref (QmiMessageWdsGetPacketStatisticsInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_packet_statistics_input_unref:
 * @self: a #QmiMessageWdsGetPacketStatisticsInput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_packet_statistics_input_unref (QmiMessageWdsGetPacketStatisticsInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsGetPacketStatisticsInput, self);
    }
}

/**
 * qmi_message_wds_get_packet_statistics_input_new:
 *
 * Allocates a new #QmiMessageWdsGetPacketStatisticsInput.
 *
 * Returns: the newly created #QmiMessageWdsGetPacketStatisticsInput. The returned value should be freed with qmi_message_wds_get_packet_statistics_input_unref().
 */
QmiMessageWdsGetPacketStatisticsInput *
qmi_message_wds_get_packet_statistics_input_new (void)
{
    QmiMessageWdsGetPacketStatisticsInput *self;

    self = g_slice_new0 (QmiMessageWdsGetPacketStatisticsInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_wds_get_packet_statistics_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageWdsGetPacketStatisticsInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_GET_PACKET_STATISTICS);

    /* All TLVs are optional, we allow NULL input */
    if (!input)
        return self;

    /* Try to add the 'Mask' TLV */
    if (input->arg_mask_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_INPUT_TLV_MASK, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Mask': ");
            goto error_out;
        }

        {
            guint32 tmp;

            tmp = (guint32) input->arg_mask;
            /* Write the guint32 variable to the buffer */
            if (!qmi_message_tlv_write_guint32 (self, QMI_ENDIAN_LITTLE, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Mask': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Mask': ");
            goto error_out;
        }
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

struct _QmiMessageWdsGetPacketStatisticsOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;

    /* Last Call Rx Bytes Ok */
    gboolean arg_last_call_rx_bytes_ok_set;
    guint64 arg_last_call_rx_bytes_ok;

    /* Last Call Tx Bytes Ok */
    gboolean arg_last_call_tx_bytes_ok_set;
    guint64 arg_last_call_tx_bytes_ok;

    /* Tx Packets Ok */
    gboolean arg_tx_packets_ok_set;
    guint32 arg_tx_packets_ok;

    /* Rx Packets Ok */
    gboolean arg_rx_packets_ok_set;
    guint32 arg_rx_packets_ok;

    /* Tx Packets Error */
    gboolean arg_tx_packets_error_set;
    guint32 arg_tx_packets_error;

    /* Rx Packets Error */
    gboolean arg_rx_packets_error_set;
    guint32 arg_rx_packets_error;

    /* Tx Overflows */
    gboolean arg_tx_overflows_set;
    guint32 arg_tx_overflows;

    /* Rx Overflows */
    gboolean arg_rx_overflows_set;
    guint32 arg_rx_overflows;

    /* Tx Bytes Ok */
    gboolean arg_tx_bytes_ok_set;
    guint64 arg_tx_bytes_ok;

    /* Rx Bytes Ok */
    gboolean arg_rx_bytes_ok_set;
    guint64 arg_rx_bytes_ok;

    /* Tx Packets Dropped */
    gboolean arg_tx_packets_dropped_set;
    guint32 arg_tx_packets_dropped;

    /* Rx Packets Dropped */
    gboolean arg_rx_packets_dropped_set;
    guint32 arg_rx_packets_dropped;
};

#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RESULT 0x02
#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_LAST_CALL_RX_BYTES_OK 0x1C
#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_LAST_CALL_TX_BYTES_OK 0x1B
#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_PACKETS_OK 0x10
#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_PACKETS_OK 0x11
#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_PACKETS_ERROR 0x12
#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_PACKETS_ERROR 0x13
#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_OVERFLOWS 0x14
#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_OVERFLOWS 0x15
#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_BYTES_OK 0x19
#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_BYTES_OK 0x1A
#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_PACKETS_DROPPED 0x1D
#define QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_PACKETS_DROPPED 0x1E

/**
 * qmi_message_wds_get_packet_statistics_output_get_result:
 * @self: a QmiMessageWdsGetPacketStatisticsOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_get_packet_statistics_output_get_result (
    QmiMessageWdsGetPacketStatisticsOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

/**
 * qmi_message_wds_get_packet_statistics_output_get_last_call_rx_bytes_ok:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 * @value_last_call_rx_bytes_ok: a placeholder for the output #guint64, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Last Call Rx Bytes Ok' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_output_get_last_call_rx_bytes_ok (
    QmiMessageWdsGetPacketStatisticsOutput *self,
    guint64 *value_last_call_rx_bytes_ok,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_last_call_rx_bytes_ok_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Last Call Rx Bytes Ok' was not found in the message");
        return FALSE;
    }

    if (value_last_call_rx_bytes_ok)
        *value_last_call_rx_bytes_ok = self->arg_last_call_rx_bytes_ok;

    return TRUE;
}

/**
 * qmi_message_wds_get_packet_statistics_output_get_last_call_tx_bytes_ok:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 * @value_last_call_tx_bytes_ok: a placeholder for the output #guint64, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Last Call Tx Bytes Ok' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_output_get_last_call_tx_bytes_ok (
    QmiMessageWdsGetPacketStatisticsOutput *self,
    guint64 *value_last_call_tx_bytes_ok,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_last_call_tx_bytes_ok_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Last Call Tx Bytes Ok' was not found in the message");
        return FALSE;
    }

    if (value_last_call_tx_bytes_ok)
        *value_last_call_tx_bytes_ok = self->arg_last_call_tx_bytes_ok;

    return TRUE;
}

/**
 * qmi_message_wds_get_packet_statistics_output_get_tx_packets_ok:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 * @value_tx_packets_ok: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Tx Packets Ok' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_output_get_tx_packets_ok (
    QmiMessageWdsGetPacketStatisticsOutput *self,
    guint32 *value_tx_packets_ok,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_tx_packets_ok_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Tx Packets Ok' was not found in the message");
        return FALSE;
    }

    if (value_tx_packets_ok)
        *value_tx_packets_ok = self->arg_tx_packets_ok;

    return TRUE;
}

/**
 * qmi_message_wds_get_packet_statistics_output_get_rx_packets_ok:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 * @value_rx_packets_ok: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Rx Packets Ok' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_output_get_rx_packets_ok (
    QmiMessageWdsGetPacketStatisticsOutput *self,
    guint32 *value_rx_packets_ok,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_rx_packets_ok_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Rx Packets Ok' was not found in the message");
        return FALSE;
    }

    if (value_rx_packets_ok)
        *value_rx_packets_ok = self->arg_rx_packets_ok;

    return TRUE;
}

/**
 * qmi_message_wds_get_packet_statistics_output_get_tx_packets_error:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 * @value_tx_packets_error: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Tx Packets Error' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_output_get_tx_packets_error (
    QmiMessageWdsGetPacketStatisticsOutput *self,
    guint32 *value_tx_packets_error,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_tx_packets_error_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Tx Packets Error' was not found in the message");
        return FALSE;
    }

    if (value_tx_packets_error)
        *value_tx_packets_error = self->arg_tx_packets_error;

    return TRUE;
}

/**
 * qmi_message_wds_get_packet_statistics_output_get_rx_packets_error:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 * @value_rx_packets_error: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Rx Packets Error' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_output_get_rx_packets_error (
    QmiMessageWdsGetPacketStatisticsOutput *self,
    guint32 *value_rx_packets_error,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_rx_packets_error_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Rx Packets Error' was not found in the message");
        return FALSE;
    }

    if (value_rx_packets_error)
        *value_rx_packets_error = self->arg_rx_packets_error;

    return TRUE;
}

/**
 * qmi_message_wds_get_packet_statistics_output_get_tx_overflows:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 * @value_tx_overflows: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Tx Overflows' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_output_get_tx_overflows (
    QmiMessageWdsGetPacketStatisticsOutput *self,
    guint32 *value_tx_overflows,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_tx_overflows_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Tx Overflows' was not found in the message");
        return FALSE;
    }

    if (value_tx_overflows)
        *value_tx_overflows = self->arg_tx_overflows;

    return TRUE;
}

/**
 * qmi_message_wds_get_packet_statistics_output_get_rx_overflows:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 * @value_rx_overflows: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Rx Overflows' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_output_get_rx_overflows (
    QmiMessageWdsGetPacketStatisticsOutput *self,
    guint32 *value_rx_overflows,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_rx_overflows_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Rx Overflows' was not found in the message");
        return FALSE;
    }

    if (value_rx_overflows)
        *value_rx_overflows = self->arg_rx_overflows;

    return TRUE;
}

/**
 * qmi_message_wds_get_packet_statistics_output_get_tx_bytes_ok:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 * @value_tx_bytes_ok: a placeholder for the output #guint64, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Tx Bytes Ok' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_output_get_tx_bytes_ok (
    QmiMessageWdsGetPacketStatisticsOutput *self,
    guint64 *value_tx_bytes_ok,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_tx_bytes_ok_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Tx Bytes Ok' was not found in the message");
        return FALSE;
    }

    if (value_tx_bytes_ok)
        *value_tx_bytes_ok = self->arg_tx_bytes_ok;

    return TRUE;
}

/**
 * qmi_message_wds_get_packet_statistics_output_get_rx_bytes_ok:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 * @value_rx_bytes_ok: a placeholder for the output #guint64, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Rx Bytes Ok' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_output_get_rx_bytes_ok (
    QmiMessageWdsGetPacketStatisticsOutput *self,
    guint64 *value_rx_bytes_ok,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_rx_bytes_ok_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Rx Bytes Ok' was not found in the message");
        return FALSE;
    }

    if (value_rx_bytes_ok)
        *value_rx_bytes_ok = self->arg_rx_bytes_ok;

    return TRUE;
}

/**
 * qmi_message_wds_get_packet_statistics_output_get_tx_packets_dropped:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 * @value_tx_packets_dropped: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Tx Packets Dropped' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_output_get_tx_packets_dropped (
    QmiMessageWdsGetPacketStatisticsOutput *self,
    guint32 *value_tx_packets_dropped,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_tx_packets_dropped_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Tx Packets Dropped' was not found in the message");
        return FALSE;
    }

    if (value_tx_packets_dropped)
        *value_tx_packets_dropped = self->arg_tx_packets_dropped;

    return TRUE;
}

/**
 * qmi_message_wds_get_packet_statistics_output_get_rx_packets_dropped:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 * @value_rx_packets_dropped: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Rx Packets Dropped' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_packet_statistics_output_get_rx_packets_dropped (
    QmiMessageWdsGetPacketStatisticsOutput *self,
    guint32 *value_rx_packets_dropped,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_rx_packets_dropped_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Rx Packets Dropped' was not found in the message");
        return FALSE;
    }

    if (value_rx_packets_dropped)
        *value_rx_packets_dropped = self->arg_rx_packets_dropped;

    return TRUE;
}

GType
qmi_message_wds_get_packet_statistics_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetPacketStatisticsOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_packet_statistics_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_packet_statistics_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_packet_statistics_output_ref:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetPacketStatisticsOutput *
qmi_message_wds_get_packet_statistics_output_ref (QmiMessageWdsGetPacketStatisticsOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_packet_statistics_output_unref:
 * @self: a #QmiMessageWdsGetPacketStatisticsOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_packet_statistics_output_unref (QmiMessageWdsGetPacketStatisticsOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsGetPacketStatisticsOutput, self);
    }
}

static gchar *
qmi_message_wds_get_packet_statistics_input_mask_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_INPUT_TLV_MASK, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_PACKET_STATISTICS_MASK_FLAG_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_packet_statistics_mask_flag_get_string ((QmiWdsPacketStatisticsMaskFlag)tmp));
#elif defined  __QMI_WDS_PACKET_STATISTICS_MASK_FLAG_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_packet_statistics_mask_flag_build_string_from_mask ((QmiWdsPacketStatisticsMaskFlag)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsPacketStatisticsMaskFlag
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_packet_statistics_output_last_call_rx_bytes_ok_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_LAST_CALL_RX_BYTES_OK, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint64 tmp;

        if (!qmi_message_tlv_read_guint64 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT64_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_packet_statistics_output_last_call_tx_bytes_ok_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_LAST_CALL_TX_BYTES_OK, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint64 tmp;

        if (!qmi_message_tlv_read_guint64 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT64_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_packet_statistics_output_tx_packets_ok_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_PACKETS_OK, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_packet_statistics_output_rx_packets_ok_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_PACKETS_OK, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_packet_statistics_output_tx_packets_error_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_PACKETS_ERROR, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_packet_statistics_output_rx_packets_error_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_PACKETS_ERROR, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_packet_statistics_output_tx_overflows_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_OVERFLOWS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_packet_statistics_output_rx_overflows_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_OVERFLOWS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_packet_statistics_output_tx_bytes_ok_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_BYTES_OK, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint64 tmp;

        if (!qmi_message_tlv_read_guint64 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT64_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_packet_statistics_output_rx_bytes_ok_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_BYTES_OK, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint64 tmp;

        if (!qmi_message_tlv_read_guint64 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT64_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_packet_statistics_output_tx_packets_dropped_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_PACKETS_DROPPED, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_packet_statistics_output_rx_packets_dropped_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_PACKETS_DROPPED, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_get_packet_statistics_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_get_packet_statistics_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_get_packet_statistics_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_INPUT_TLV_MASK:
            tlv_type_str = "Mask";
            translated_value = qmi_message_wds_get_packet_statistics_input_mask_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_LAST_CALL_RX_BYTES_OK:
            tlv_type_str = "Last Call Rx Bytes Ok";
            translated_value = qmi_message_wds_get_packet_statistics_output_last_call_rx_bytes_ok_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_LAST_CALL_TX_BYTES_OK:
            tlv_type_str = "Last Call Tx Bytes Ok";
            translated_value = qmi_message_wds_get_packet_statistics_output_last_call_tx_bytes_ok_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_PACKETS_OK:
            tlv_type_str = "Tx Packets Ok";
            translated_value = qmi_message_wds_get_packet_statistics_output_tx_packets_ok_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_PACKETS_OK:
            tlv_type_str = "Rx Packets Ok";
            translated_value = qmi_message_wds_get_packet_statistics_output_rx_packets_ok_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_PACKETS_ERROR:
            tlv_type_str = "Tx Packets Error";
            translated_value = qmi_message_wds_get_packet_statistics_output_tx_packets_error_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_PACKETS_ERROR:
            tlv_type_str = "Rx Packets Error";
            translated_value = qmi_message_wds_get_packet_statistics_output_rx_packets_error_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_OVERFLOWS:
            tlv_type_str = "Tx Overflows";
            translated_value = qmi_message_wds_get_packet_statistics_output_tx_overflows_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_OVERFLOWS:
            tlv_type_str = "Rx Overflows";
            translated_value = qmi_message_wds_get_packet_statistics_output_rx_overflows_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_BYTES_OK:
            tlv_type_str = "Tx Bytes Ok";
            translated_value = qmi_message_wds_get_packet_statistics_output_tx_bytes_ok_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_BYTES_OK:
            tlv_type_str = "Rx Bytes Ok";
            translated_value = qmi_message_wds_get_packet_statistics_output_rx_bytes_ok_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_PACKETS_DROPPED:
            tlv_type_str = "Tx Packets Dropped";
            translated_value = qmi_message_wds_get_packet_statistics_output_tx_packets_dropped_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_PACKETS_DROPPED:
            tlv_type_str = "Rx Packets Dropped";
            translated_value = qmi_message_wds_get_packet_statistics_output_rx_packets_dropped_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_get_packet_statistics_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Get Packet Statistics\" (0x0024)\n",
                            line_prefix);

    {
        struct message_get_packet_statistics_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_get_packet_statistics_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsGetPacketStatisticsOutput *
__qmi_message_wds_get_packet_statistics_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsGetPacketStatisticsOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_GET_PACKET_STATISTICS, NULL);

    self = g_slice_new0 (QmiMessageWdsGetPacketStatisticsOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_get_packet_statistics_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_get_packet_statistics_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_LAST_CALL_RX_BYTES_OK, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_packet_statistics_output_last_call_rx_bytes_ok_out;
            }
            if (!qmi_message_tlv_read_guint64 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_last_call_rx_bytes_ok), NULL))
                goto qmi_message_wds_get_packet_statistics_output_last_call_rx_bytes_ok_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Last Call Rx Bytes Ok' TLV", offset);
            }

            self->arg_last_call_rx_bytes_ok_set = TRUE;

qmi_message_wds_get_packet_statistics_output_last_call_rx_bytes_ok_out:
            ;

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_LAST_CALL_TX_BYTES_OK, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_packet_statistics_output_last_call_tx_bytes_ok_out;
            }
            if (!qmi_message_tlv_read_guint64 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_last_call_tx_bytes_ok), NULL))
                goto qmi_message_wds_get_packet_statistics_output_last_call_tx_bytes_ok_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Last Call Tx Bytes Ok' TLV", offset);
            }

            self->arg_last_call_tx_bytes_ok_set = TRUE;

qmi_message_wds_get_packet_statistics_output_last_call_tx_bytes_ok_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_PACKETS_OK, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_packet_statistics_output_tx_packets_ok_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_tx_packets_ok), NULL))
                goto qmi_message_wds_get_packet_statistics_output_tx_packets_ok_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Tx Packets Ok' TLV", offset);
            }

            self->arg_tx_packets_ok_set = TRUE;

qmi_message_wds_get_packet_statistics_output_tx_packets_ok_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_PACKETS_OK, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_packet_statistics_output_rx_packets_ok_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_rx_packets_ok), NULL))
                goto qmi_message_wds_get_packet_statistics_output_rx_packets_ok_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Rx Packets Ok' TLV", offset);
            }

            self->arg_rx_packets_ok_set = TRUE;

qmi_message_wds_get_packet_statistics_output_rx_packets_ok_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_PACKETS_ERROR, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_packet_statistics_output_tx_packets_error_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_tx_packets_error), NULL))
                goto qmi_message_wds_get_packet_statistics_output_tx_packets_error_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Tx Packets Error' TLV", offset);
            }

            self->arg_tx_packets_error_set = TRUE;

qmi_message_wds_get_packet_statistics_output_tx_packets_error_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_PACKETS_ERROR, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_packet_statistics_output_rx_packets_error_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_rx_packets_error), NULL))
                goto qmi_message_wds_get_packet_statistics_output_rx_packets_error_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Rx Packets Error' TLV", offset);
            }

            self->arg_rx_packets_error_set = TRUE;

qmi_message_wds_get_packet_statistics_output_rx_packets_error_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_OVERFLOWS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_packet_statistics_output_tx_overflows_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_tx_overflows), NULL))
                goto qmi_message_wds_get_packet_statistics_output_tx_overflows_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Tx Overflows' TLV", offset);
            }

            self->arg_tx_overflows_set = TRUE;

qmi_message_wds_get_packet_statistics_output_tx_overflows_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_OVERFLOWS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_packet_statistics_output_rx_overflows_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_rx_overflows), NULL))
                goto qmi_message_wds_get_packet_statistics_output_rx_overflows_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Rx Overflows' TLV", offset);
            }

            self->arg_rx_overflows_set = TRUE;

qmi_message_wds_get_packet_statistics_output_rx_overflows_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_BYTES_OK, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_packet_statistics_output_tx_bytes_ok_out;
            }
            if (!qmi_message_tlv_read_guint64 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_tx_bytes_ok), NULL))
                goto qmi_message_wds_get_packet_statistics_output_tx_bytes_ok_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Tx Bytes Ok' TLV", offset);
            }

            self->arg_tx_bytes_ok_set = TRUE;

qmi_message_wds_get_packet_statistics_output_tx_bytes_ok_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_BYTES_OK, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_packet_statistics_output_rx_bytes_ok_out;
            }
            if (!qmi_message_tlv_read_guint64 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_rx_bytes_ok), NULL))
                goto qmi_message_wds_get_packet_statistics_output_rx_bytes_ok_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Rx Bytes Ok' TLV", offset);
            }

            self->arg_rx_bytes_ok_set = TRUE;

qmi_message_wds_get_packet_statistics_output_rx_bytes_ok_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_TX_PACKETS_DROPPED, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_packet_statistics_output_tx_packets_dropped_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_tx_packets_dropped), NULL))
                goto qmi_message_wds_get_packet_statistics_output_tx_packets_dropped_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Tx Packets Dropped' TLV", offset);
            }

            self->arg_tx_packets_dropped_set = TRUE;

qmi_message_wds_get_packet_statistics_output_tx_packets_dropped_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PACKET_STATISTICS_OUTPUT_TLV_RX_PACKETS_DROPPED, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_packet_statistics_output_rx_packets_dropped_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_rx_packets_dropped), NULL))
                goto qmi_message_wds_get_packet_statistics_output_rx_packets_dropped_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Rx Packets Dropped' TLV", offset);
            }

            self->arg_rx_packets_dropped_set = TRUE;

qmi_message_wds_get_packet_statistics_output_rx_packets_dropped_out:
            ;

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Get Profile List */


/* --- Input -- */

struct _QmiMessageWdsGetProfileListInput {
    volatile gint ref_count;

    /* Profile Type */
    gboolean arg_profile_type_set;
    guint8 arg_profile_type;
};

#define QMI_MESSAGE_WDS_GET_PROFILE_LIST_INPUT_TLV_PROFILE_TYPE 0x10

/**
 * qmi_message_wds_get_profile_list_input_get_profile_type:
 * @self: a #QmiMessageWdsGetProfileListInput.
 * @value_profile_type: a placeholder for the output #QmiWdsProfileType, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Profile Type' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_list_input_get_profile_type (
    QmiMessageWdsGetProfileListInput *self,
    QmiWdsProfileType *value_profile_type,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_profile_type_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Profile Type' was not found in the message");
        return FALSE;
    }

    if (value_profile_type)
        *value_profile_type = (QmiWdsProfileType)(self->arg_profile_type);

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_list_input_set_profile_type:
 * @self: a #QmiMessageWdsGetProfileListInput.
 * @value_profile_type: a #QmiWdsProfileType.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Profile Type' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_list_input_set_profile_type (
    QmiMessageWdsGetProfileListInput *self,
    QmiWdsProfileType value_profile_type,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_profile_type = (guint8)(value_profile_type);
    self->arg_profile_type_set = TRUE;

    return TRUE;
}

GType
qmi_message_wds_get_profile_list_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetProfileListInput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_profile_list_input_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_profile_list_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_profile_list_input_ref:
 * @self: a #QmiMessageWdsGetProfileListInput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetProfileListInput *
qmi_message_wds_get_profile_list_input_ref (QmiMessageWdsGetProfileListInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_profile_list_input_unref:
 * @self: a #QmiMessageWdsGetProfileListInput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_profile_list_input_unref (QmiMessageWdsGetProfileListInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsGetProfileListInput, self);
    }
}

/**
 * qmi_message_wds_get_profile_list_input_new:
 *
 * Allocates a new #QmiMessageWdsGetProfileListInput.
 *
 * Returns: the newly created #QmiMessageWdsGetProfileListInput. The returned value should be freed with qmi_message_wds_get_profile_list_input_unref().
 */
QmiMessageWdsGetProfileListInput *
qmi_message_wds_get_profile_list_input_new (void)
{
    QmiMessageWdsGetProfileListInput *self;

    self = g_slice_new0 (QmiMessageWdsGetProfileListInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_wds_get_profile_list_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageWdsGetProfileListInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_GET_PROFILE_LIST);

    /* All TLVs are optional, we allow NULL input */
    if (!input)
        return self;

    /* Try to add the 'Profile Type' TLV */
    if (input->arg_profile_type_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_GET_PROFILE_LIST_INPUT_TLV_PROFILE_TYPE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Profile Type': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_profile_type;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Profile Type': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Profile Type': ");
            goto error_out;
        }
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

static void
profile_list_qmimessagewdsgetprofilelistoutputprofilelistprofile_clear (QmiMessageWdsGetProfileListOutputProfileListProfile *p)
{
    g_free ((*p).profile_name);
}

struct _QmiMessageWdsGetProfileListOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;

    /* Profile List */
    gboolean arg_profile_list_set;
    GArray *arg_profile_list;

    /* Extended Error Code */
    gboolean arg_extended_error_code_set;
    guint16 arg_extended_error_code;
};

#define QMI_MESSAGE_WDS_GET_PROFILE_LIST_OUTPUT_TLV_RESULT 0x02
#define QMI_MESSAGE_WDS_GET_PROFILE_LIST_OUTPUT_TLV_PROFILE_LIST 0x01
#define QMI_MESSAGE_WDS_GET_PROFILE_LIST_OUTPUT_TLV_EXTENDED_ERROR_CODE 0xE0

/**
 * qmi_message_wds_get_profile_list_output_get_result:
 * @self: a QmiMessageWdsGetProfileListOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_get_profile_list_output_get_result (
    QmiMessageWdsGetProfileListOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

/**
 * qmi_message_wds_get_profile_list_output_get_profile_list:
 * @self: a #QmiMessageWdsGetProfileListOutput.
 * @value_profile_list: a placeholder for the output #GArray of #QmiMessageWdsGetProfileListOutputProfileListProfile elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Profile List' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_list_output_get_profile_list (
    QmiMessageWdsGetProfileListOutput *self,
    GArray **value_profile_list,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_profile_list_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Profile List' was not found in the message");
        return FALSE;
    }

    if (value_profile_list)
        *value_profile_list = self->arg_profile_list;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_list_output_get_extended_error_code:
 * @self: a #QmiMessageWdsGetProfileListOutput.
 * @value_extended_error_code: a placeholder for the output #QmiWdsDsProfileError, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Extended Error Code' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_list_output_get_extended_error_code (
    QmiMessageWdsGetProfileListOutput *self,
    QmiWdsDsProfileError *value_extended_error_code,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_extended_error_code_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Extended Error Code' was not found in the message");
        return FALSE;
    }

    if (value_extended_error_code)
        *value_extended_error_code = (QmiWdsDsProfileError)(self->arg_extended_error_code);

    return TRUE;
}

GType
qmi_message_wds_get_profile_list_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetProfileListOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_profile_list_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_profile_list_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_profile_list_output_ref:
 * @self: a #QmiMessageWdsGetProfileListOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetProfileListOutput *
qmi_message_wds_get_profile_list_output_ref (QmiMessageWdsGetProfileListOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_profile_list_output_unref:
 * @self: a #QmiMessageWdsGetProfileListOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_profile_list_output_unref (QmiMessageWdsGetProfileListOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        if (self->arg_profile_list)
            g_array_unref (self->arg_profile_list);
        g_slice_free (QmiMessageWdsGetProfileListOutput, self);
    }
}

static gchar *
qmi_message_wds_get_profile_list_input_profile_type_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_LIST_INPUT_TLV_PROFILE_TYPE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_PROFILE_TYPE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_profile_type_get_string ((QmiWdsProfileType)tmp));
#elif defined  __QMI_WDS_PROFILE_TYPE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_profile_type_build_string_from_mask ((QmiWdsProfileType)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsProfileType
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_list_output_profile_list_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_LIST_OUTPUT_TLV_PROFILE_LIST, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint profile_list_i;
        guint8 profile_list_n_items;

        /* Read number of items in the array */
        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(profile_list_n_items), &error))
            goto out;

        g_string_append (printable, "{");

        for (profile_list_i = 0; profile_list_i < profile_list_n_items; profile_list_i++) {
            g_string_append_printf (printable, " [%u] = '", profile_list_i);
            g_string_append (printable, "[");
            g_string_append (printable, " profile_type = '");

            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
#if defined  __QMI_WDS_PROFILE_TYPE_IS_ENUM__
                g_string_append_printf (printable, "%s", qmi_wds_profile_type_get_string ((QmiWdsProfileType)tmp));
#elif defined  __QMI_WDS_PROFILE_TYPE_IS_FLAGS__
                {
                    gchar *flags_str;

                    flags_str = qmi_wds_profile_type_build_string_from_mask ((QmiWdsProfileType)tmp);
                    g_string_append_printf (printable, "%s", flags_str);
                    g_free (flags_str);
                }
#else
# error unexpected public format: QmiWdsProfileType
#endif
            }
            g_string_append (printable, "'");
            g_string_append (printable, " profile_index = '");

            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%u", (guint)tmp);
            }
            g_string_append (printable, "'");
            g_string_append (printable, " profile_name = '");

            {
                gchar *tmp;

                if (!qmi_message_tlv_read_string (message, init_offset, &offset, 1, 0, &tmp, &error))
                    goto out;
                g_string_append (printable, tmp);
                g_free (tmp);
            }
            g_string_append (printable, "'");
            g_string_append (printable, " ]");
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_list_output_extended_error_code_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_LIST_OUTPUT_TLV_EXTENDED_ERROR_CODE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_DS_PROFILE_ERROR_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_ds_profile_error_get_string ((QmiWdsDsProfileError)tmp));
#elif defined  __QMI_WDS_DS_PROFILE_ERROR_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_ds_profile_error_build_string_from_mask ((QmiWdsDsProfileError)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsDsProfileError
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_get_profile_list_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_get_profile_list_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_get_profile_list_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_PROFILE_LIST_INPUT_TLV_PROFILE_TYPE:
            tlv_type_str = "Profile Type";
            translated_value = qmi_message_wds_get_profile_list_input_profile_type_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_PROFILE_LIST_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_LIST_OUTPUT_TLV_PROFILE_LIST:
            tlv_type_str = "Profile List";
            translated_value = qmi_message_wds_get_profile_list_output_profile_list_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_LIST_OUTPUT_TLV_EXTENDED_ERROR_CODE:
            tlv_type_str = "Extended Error Code";
            translated_value = qmi_message_wds_get_profile_list_output_extended_error_code_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_get_profile_list_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Get Profile List\" (0x002A)\n",
                            line_prefix);

    {
        struct message_get_profile_list_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_get_profile_list_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsGetProfileListOutput *
__qmi_message_wds_get_profile_list_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsGetProfileListOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_GET_PROFILE_LIST, NULL);

    self = g_slice_new0 (QmiMessageWdsGetProfileListOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_LIST_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_get_profile_list_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_get_profile_list_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_LIST_OUTPUT_TLV_PROFILE_LIST, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Profile List TLV: ");
                qmi_message_wds_get_profile_list_output_unref (self);
                return NULL;
            }
            {
                guint profile_list_i;
                guint8 profile_list_n_items;

                /* Read number of items in the array */
                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(profile_list_n_items), error))
                    goto qmi_message_wds_get_profile_list_output_profile_list_out;

                self->arg_profile_list = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (QmiMessageWdsGetProfileListOutputProfileListProfile),
                    (guint)profile_list_n_items);

                g_array_set_clear_func (self->arg_profile_list,
                                        (GDestroyNotify)profile_list_qmimessagewdsgetprofilelistoutputprofilelistprofile_clear);

                for (profile_list_i = 0; profile_list_i < profile_list_n_items; profile_list_i++) {
                    QmiMessageWdsGetProfileListOutputProfileListProfile profile_list_aux;

                    {
                        guint8 tmp;

                        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, error))
                            goto qmi_message_wds_get_profile_list_output_profile_list_out;
                        profile_list_aux.profile_type = (QmiWdsProfileType)tmp;
                    }
                    if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(profile_list_aux.profile_index), error))
                        goto qmi_message_wds_get_profile_list_output_profile_list_out;
                    if (!qmi_message_tlv_read_string (message, init_offset, &offset, 1, 0, &(profile_list_aux.profile_name), error))
                        goto qmi_message_wds_get_profile_list_output_profile_list_out;
                    g_array_insert_val (self->arg_profile_list, profile_list_i, profile_list_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Profile List' TLV", offset);
            }

            self->arg_profile_list_set = TRUE;

qmi_message_wds_get_profile_list_output_profile_list_out:
            if (!self->arg_profile_list_set) {
                qmi_message_wds_get_profile_list_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status != QMI_STATUS_SUCCESS))
            break;
        /* Prerequisite.... */
        if (!(self->arg_result.error_code == QMI_PROTOCOL_ERROR_EXTENDED_INTERNAL))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_LIST_OUTPUT_TLV_EXTENDED_ERROR_CODE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_list_output_extended_error_code_out;
            }
            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, NULL))
                    goto qmi_message_wds_get_profile_list_output_extended_error_code_out;
                self->arg_extended_error_code = (QmiWdsDsProfileError)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Extended Error Code' TLV", offset);
            }

            self->arg_extended_error_code_set = TRUE;

qmi_message_wds_get_profile_list_output_extended_error_code_out:
            ;

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Get Profile Settings */


/* --- Input -- */

struct _QmiMessageWdsGetProfileSettingsInput {
    volatile gint ref_count;

    /* Profile ID */
    gboolean arg_profile_id_set;
    guint8 arg_profile_id_profile_type;
    guint8 arg_profile_id_profile_index;
};

#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_INPUT_TLV_PROFILE_ID 0x01

/**
 * qmi_message_wds_get_profile_settings_input_get_profile_id:
 * @self: a #QmiMessageWdsGetProfileSettingsInput.
 * @value_profile_id_profile_type: a placeholder for the output #QmiWdsProfileType, or %NULL if not required.
 * @value_profile_id_profile_index: a placeholder for the output #guint8, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Profile ID' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_input_get_profile_id (
    QmiMessageWdsGetProfileSettingsInput *self,
    QmiWdsProfileType *value_profile_id_profile_type,
    guint8 *value_profile_id_profile_index,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_profile_id_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Profile ID' was not found in the message");
        return FALSE;
    }

    if (value_profile_id_profile_type)
        *value_profile_id_profile_type = (QmiWdsProfileType)(self->arg_profile_id_profile_type);
    if (value_profile_id_profile_index)
        *value_profile_id_profile_index = self->arg_profile_id_profile_index;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_input_set_profile_id:
 * @self: a #QmiMessageWdsGetProfileSettingsInput.
 * @value_profile_id_profile_type: a #QmiWdsProfileType.
 * @value_profile_id_profile_index: a #guint8.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Profile ID' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_input_set_profile_id (
    QmiMessageWdsGetProfileSettingsInput *self,
    QmiWdsProfileType value_profile_id_profile_type,
    guint8 value_profile_id_profile_index,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_profile_id_profile_type = (guint8)(value_profile_id_profile_type);
    self->arg_profile_id_profile_index = value_profile_id_profile_index;
    self->arg_profile_id_set = TRUE;

    return TRUE;
}

GType
qmi_message_wds_get_profile_settings_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetProfileSettingsInput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_profile_settings_input_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_profile_settings_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_profile_settings_input_ref:
 * @self: a #QmiMessageWdsGetProfileSettingsInput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetProfileSettingsInput *
qmi_message_wds_get_profile_settings_input_ref (QmiMessageWdsGetProfileSettingsInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_profile_settings_input_unref:
 * @self: a #QmiMessageWdsGetProfileSettingsInput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_profile_settings_input_unref (QmiMessageWdsGetProfileSettingsInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsGetProfileSettingsInput, self);
    }
}

/**
 * qmi_message_wds_get_profile_settings_input_new:
 *
 * Allocates a new #QmiMessageWdsGetProfileSettingsInput.
 *
 * Returns: the newly created #QmiMessageWdsGetProfileSettingsInput. The returned value should be freed with qmi_message_wds_get_profile_settings_input_unref().
 */
QmiMessageWdsGetProfileSettingsInput *
qmi_message_wds_get_profile_settings_input_new (void)
{
    QmiMessageWdsGetProfileSettingsInput *self;

    self = g_slice_new0 (QmiMessageWdsGetProfileSettingsInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_wds_get_profile_settings_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageWdsGetProfileSettingsInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS);

    /* All TLVs are optional, we allow NULL input */
    if (!input)
        return self;

    /* Try to add the 'Profile ID' TLV */
    if (input->arg_profile_id_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_INPUT_TLV_PROFILE_ID, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Profile ID': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_profile_id_profile_type;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Profile ID': ");
                goto error_out;
            }
        }
        /* Write the guint8 variable to the buffer */
        if (!qmi_message_tlv_write_guint8 (self, input->arg_profile_id_profile_index, error)) {
            g_prefix_error (error, "Cannot write integer in TLV 'Profile ID': ");
            goto error_out;
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Profile ID': ");
            goto error_out;
        }
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

struct _QmiMessageWdsGetProfileSettingsOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;

    /* IPv6 Address Preference */
    gboolean arg_ipv6_address_preference_set;
    GArray *arg_ipv6_address_preference_address;

    /* Profile Name */
    gboolean arg_profile_name_set;
    gchar *arg_profile_name;

    /* PDP Type */
    gboolean arg_pdp_type_set;
    guint8 arg_pdp_type;

    /* APN Name */
    gboolean arg_apn_name_set;
    gchar *arg_apn_name;

    /* Primary IPv4 DNS Address */
    gboolean arg_primary_ipv4_dns_address_set;
    guint32 arg_primary_ipv4_dns_address;

    /* Secondary IPv4 DNS Address */
    gboolean arg_secondary_ipv4_dns_address_set;
    guint32 arg_secondary_ipv4_dns_address;

    /* GPRS Requested QoS */
    gboolean arg_gprs_requested_qos_set;
    guint32 arg_gprs_requested_qos_precedence_class;
    guint32 arg_gprs_requested_qos_delay_class;
    guint32 arg_gprs_requested_qos_reliability_class;
    guint32 arg_gprs_requested_qos_peak_throughput_class;
    guint32 arg_gprs_requested_qos_mean_throughput_class;

    /* GPRS Minimum QoS */
    gboolean arg_gprs_minimum_qos_set;
    guint32 arg_gprs_minimum_qos_precedence_class;
    guint32 arg_gprs_minimum_qos_delay_class;
    guint32 arg_gprs_minimum_qos_reliability_class;
    guint32 arg_gprs_minimum_qos_peak_throughput_class;
    guint32 arg_gprs_minimum_qos_mean_throughput_class;

    /* Username */
    gboolean arg_username_set;
    gchar *arg_username;

    /* Password */
    gboolean arg_password_set;
    gchar *arg_password;

    /* Authentication */
    gboolean arg_authentication_set;
    guint8 arg_authentication;

    /* IPv4 Address Preference */
    gboolean arg_ipv4_address_preference_set;
    guint32 arg_ipv4_address_preference;

    /* PCSCF Address Using PCO */
    gboolean arg_pcscf_address_using_pco_set;
    guint8 arg_pcscf_address_using_pco;

    /* PCSCF Address Using DHCP */
    gboolean arg_pcscf_address_using_dhcp_set;
    guint8 arg_pcscf_address_using_dhcp;

    /* IMCN Flag */
    gboolean arg_imcn_flag_set;
    guint8 arg_imcn_flag;

    /* IPv6 Primary DNS Address Preference */
    gboolean arg_ipv6_primary_dns_address_preference_set;
    GArray *arg_ipv6_primary_dns_address_preference;

    /* IPv6 Secondary DNS Address Preference */
    gboolean arg_ipv6_secondary_dns_address_preference_set;
    GArray *arg_ipv6_secondary_dns_address_preference;

    /* Extended Error Code */
    gboolean arg_extended_error_code_set;
    guint16 arg_extended_error_code;
};

#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_RESULT 0x02
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV6_ADDRESS_PREFERENCE 0x28
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PROFILE_NAME 0x10
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PDP_TYPE 0x11
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_APN_NAME 0x14
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PRIMARY_IPV4_DNS_ADDRESS 0x15
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_SECONDARY_IPV4_DNS_ADDRESS 0x16
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_GPRS_REQUESTED_QOS 0x19
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_GPRS_MINIMUM_QOS 0x1A
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_USERNAME 0x1B
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PASSWORD 0x1C
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_AUTHENTICATION 0x1D
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV4_ADDRESS_PREFERENCE 0x1E
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_PCO 0x1F
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_DHCP 0x21
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IMCN_FLAG 0x22
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV6_PRIMARY_DNS_ADDRESS_PREFERENCE 0x2B
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV6_SECONDARY_DNS_ADDRESS_PREFERENCE 0x2C
#define QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_EXTENDED_ERROR_CODE 0xE0

/**
 * qmi_message_wds_get_profile_settings_output_get_result:
 * @self: a QmiMessageWdsGetProfileSettingsOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_result (
    QmiMessageWdsGetProfileSettingsOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_ipv6_address_preference:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_ipv6_address_preference_address: a placeholder for the output #GArray of #guint16 elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv6 Address Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_ipv6_address_preference (
    QmiMessageWdsGetProfileSettingsOutput *self,
    GArray **value_ipv6_address_preference_address,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv6_address_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv6 Address Preference' was not found in the message");
        return FALSE;
    }

    if (value_ipv6_address_preference_address)
        *value_ipv6_address_preference_address = self->arg_ipv6_address_preference_address;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_profile_name:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_profile_name: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Profile Name' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_profile_name (
    QmiMessageWdsGetProfileSettingsOutput *self,
    const gchar **value_profile_name,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_profile_name_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Profile Name' was not found in the message");
        return FALSE;
    }

    if (value_profile_name)
        *value_profile_name = self->arg_profile_name;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_pdp_type:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_pdp_type: a placeholder for the output #QmiWdsPdpType, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'PDP Type' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_pdp_type (
    QmiMessageWdsGetProfileSettingsOutput *self,
    QmiWdsPdpType *value_pdp_type,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_pdp_type_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'PDP Type' was not found in the message");
        return FALSE;
    }

    if (value_pdp_type)
        *value_pdp_type = (QmiWdsPdpType)(self->arg_pdp_type);

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_apn_name:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_apn_name: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'APN Name' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_apn_name (
    QmiMessageWdsGetProfileSettingsOutput *self,
    const gchar **value_apn_name,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_apn_name_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'APN Name' was not found in the message");
        return FALSE;
    }

    if (value_apn_name)
        *value_apn_name = self->arg_apn_name;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_primary_ipv4_dns_address:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_primary_ipv4_dns_address: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Primary IPv4 DNS Address' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_primary_ipv4_dns_address (
    QmiMessageWdsGetProfileSettingsOutput *self,
    guint32 *value_primary_ipv4_dns_address,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_primary_ipv4_dns_address_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Primary IPv4 DNS Address' was not found in the message");
        return FALSE;
    }

    if (value_primary_ipv4_dns_address)
        *value_primary_ipv4_dns_address = self->arg_primary_ipv4_dns_address;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_secondary_ipv4_dns_address:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_secondary_ipv4_dns_address: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Secondary IPv4 DNS Address' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_secondary_ipv4_dns_address (
    QmiMessageWdsGetProfileSettingsOutput *self,
    guint32 *value_secondary_ipv4_dns_address,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_secondary_ipv4_dns_address_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Secondary IPv4 DNS Address' was not found in the message");
        return FALSE;
    }

    if (value_secondary_ipv4_dns_address)
        *value_secondary_ipv4_dns_address = self->arg_secondary_ipv4_dns_address;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_gprs_requested_qos:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_gprs_requested_qos_precedence_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_requested_qos_delay_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_requested_qos_reliability_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_requested_qos_peak_throughput_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_requested_qos_mean_throughput_class: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'GPRS Requested QoS' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_gprs_requested_qos (
    QmiMessageWdsGetProfileSettingsOutput *self,
    guint32 *value_gprs_requested_qos_precedence_class,
    guint32 *value_gprs_requested_qos_delay_class,
    guint32 *value_gprs_requested_qos_reliability_class,
    guint32 *value_gprs_requested_qos_peak_throughput_class,
    guint32 *value_gprs_requested_qos_mean_throughput_class,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_gprs_requested_qos_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'GPRS Requested QoS' was not found in the message");
        return FALSE;
    }

    if (value_gprs_requested_qos_precedence_class)
        *value_gprs_requested_qos_precedence_class = self->arg_gprs_requested_qos_precedence_class;
    if (value_gprs_requested_qos_delay_class)
        *value_gprs_requested_qos_delay_class = self->arg_gprs_requested_qos_delay_class;
    if (value_gprs_requested_qos_reliability_class)
        *value_gprs_requested_qos_reliability_class = self->arg_gprs_requested_qos_reliability_class;
    if (value_gprs_requested_qos_peak_throughput_class)
        *value_gprs_requested_qos_peak_throughput_class = self->arg_gprs_requested_qos_peak_throughput_class;
    if (value_gprs_requested_qos_mean_throughput_class)
        *value_gprs_requested_qos_mean_throughput_class = self->arg_gprs_requested_qos_mean_throughput_class;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_gprs_minimum_qos:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_gprs_minimum_qos_precedence_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_minimum_qos_delay_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_minimum_qos_reliability_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_minimum_qos_peak_throughput_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_minimum_qos_mean_throughput_class: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'GPRS Minimum QoS' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_gprs_minimum_qos (
    QmiMessageWdsGetProfileSettingsOutput *self,
    guint32 *value_gprs_minimum_qos_precedence_class,
    guint32 *value_gprs_minimum_qos_delay_class,
    guint32 *value_gprs_minimum_qos_reliability_class,
    guint32 *value_gprs_minimum_qos_peak_throughput_class,
    guint32 *value_gprs_minimum_qos_mean_throughput_class,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_gprs_minimum_qos_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'GPRS Minimum QoS' was not found in the message");
        return FALSE;
    }

    if (value_gprs_minimum_qos_precedence_class)
        *value_gprs_minimum_qos_precedence_class = self->arg_gprs_minimum_qos_precedence_class;
    if (value_gprs_minimum_qos_delay_class)
        *value_gprs_minimum_qos_delay_class = self->arg_gprs_minimum_qos_delay_class;
    if (value_gprs_minimum_qos_reliability_class)
        *value_gprs_minimum_qos_reliability_class = self->arg_gprs_minimum_qos_reliability_class;
    if (value_gprs_minimum_qos_peak_throughput_class)
        *value_gprs_minimum_qos_peak_throughput_class = self->arg_gprs_minimum_qos_peak_throughput_class;
    if (value_gprs_minimum_qos_mean_throughput_class)
        *value_gprs_minimum_qos_mean_throughput_class = self->arg_gprs_minimum_qos_mean_throughput_class;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_username:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_username: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Username' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_username (
    QmiMessageWdsGetProfileSettingsOutput *self,
    const gchar **value_username,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_username_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Username' was not found in the message");
        return FALSE;
    }

    if (value_username)
        *value_username = self->arg_username;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_password:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_password: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Password' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_password (
    QmiMessageWdsGetProfileSettingsOutput *self,
    const gchar **value_password,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_password_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Password' was not found in the message");
        return FALSE;
    }

    if (value_password)
        *value_password = self->arg_password;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_authentication:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_authentication: a placeholder for the output #QmiWdsAuthentication, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Authentication' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_authentication (
    QmiMessageWdsGetProfileSettingsOutput *self,
    QmiWdsAuthentication *value_authentication,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_authentication_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Authentication' was not found in the message");
        return FALSE;
    }

    if (value_authentication)
        *value_authentication = (QmiWdsAuthentication)(self->arg_authentication);

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_ipv4_address_preference:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_ipv4_address_preference: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv4 Address Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_ipv4_address_preference (
    QmiMessageWdsGetProfileSettingsOutput *self,
    guint32 *value_ipv4_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv4_address_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv4 Address Preference' was not found in the message");
        return FALSE;
    }

    if (value_ipv4_address_preference)
        *value_ipv4_address_preference = self->arg_ipv4_address_preference;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_pcscf_address_using_pco:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_pcscf_address_using_pco: a placeholder for the output #gboolean, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'PCSCF Address Using PCO' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_pcscf_address_using_pco (
    QmiMessageWdsGetProfileSettingsOutput *self,
    gboolean *value_pcscf_address_using_pco,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_pcscf_address_using_pco_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'PCSCF Address Using PCO' was not found in the message");
        return FALSE;
    }

    if (value_pcscf_address_using_pco)
        *value_pcscf_address_using_pco = (gboolean)(self->arg_pcscf_address_using_pco);

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_pcscf_address_using_dhcp:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_pcscf_address_using_dhcp: a placeholder for the output #gboolean, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'PCSCF Address Using DHCP' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_pcscf_address_using_dhcp (
    QmiMessageWdsGetProfileSettingsOutput *self,
    gboolean *value_pcscf_address_using_dhcp,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_pcscf_address_using_dhcp_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'PCSCF Address Using DHCP' was not found in the message");
        return FALSE;
    }

    if (value_pcscf_address_using_dhcp)
        *value_pcscf_address_using_dhcp = (gboolean)(self->arg_pcscf_address_using_dhcp);

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_imcn_flag:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_imcn_flag: a placeholder for the output #gboolean, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IMCN Flag' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_imcn_flag (
    QmiMessageWdsGetProfileSettingsOutput *self,
    gboolean *value_imcn_flag,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_imcn_flag_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IMCN Flag' was not found in the message");
        return FALSE;
    }

    if (value_imcn_flag)
        *value_imcn_flag = (gboolean)(self->arg_imcn_flag);

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_ipv6_primary_dns_address_preference:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_ipv6_primary_dns_address_preference: a placeholder for the output #GArray of #guint16 elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv6 Primary DNS Address Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_ipv6_primary_dns_address_preference (
    QmiMessageWdsGetProfileSettingsOutput *self,
    GArray **value_ipv6_primary_dns_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv6_primary_dns_address_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv6 Primary DNS Address Preference' was not found in the message");
        return FALSE;
    }

    if (value_ipv6_primary_dns_address_preference)
        *value_ipv6_primary_dns_address_preference = self->arg_ipv6_primary_dns_address_preference;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_ipv6_secondary_dns_address_preference:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_ipv6_secondary_dns_address_preference: a placeholder for the output #GArray of #guint16 elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv6 Secondary DNS Address Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_ipv6_secondary_dns_address_preference (
    QmiMessageWdsGetProfileSettingsOutput *self,
    GArray **value_ipv6_secondary_dns_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv6_secondary_dns_address_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv6 Secondary DNS Address Preference' was not found in the message");
        return FALSE;
    }

    if (value_ipv6_secondary_dns_address_preference)
        *value_ipv6_secondary_dns_address_preference = self->arg_ipv6_secondary_dns_address_preference;

    return TRUE;
}

/**
 * qmi_message_wds_get_profile_settings_output_get_extended_error_code:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 * @value_extended_error_code: a placeholder for the output #QmiWdsDsProfileError, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Extended Error Code' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_profile_settings_output_get_extended_error_code (
    QmiMessageWdsGetProfileSettingsOutput *self,
    QmiWdsDsProfileError *value_extended_error_code,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_extended_error_code_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Extended Error Code' was not found in the message");
        return FALSE;
    }

    if (value_extended_error_code)
        *value_extended_error_code = (QmiWdsDsProfileError)(self->arg_extended_error_code);

    return TRUE;
}

GType
qmi_message_wds_get_profile_settings_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetProfileSettingsOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_profile_settings_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_profile_settings_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_profile_settings_output_ref:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetProfileSettingsOutput *
qmi_message_wds_get_profile_settings_output_ref (QmiMessageWdsGetProfileSettingsOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_profile_settings_output_unref:
 * @self: a #QmiMessageWdsGetProfileSettingsOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_profile_settings_output_unref (QmiMessageWdsGetProfileSettingsOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        if (self->arg_ipv6_address_preference_address)
            g_array_unref (self->arg_ipv6_address_preference_address);
        g_free (self->arg_profile_name);
        g_free (self->arg_apn_name);
        g_free (self->arg_username);
        g_free (self->arg_password);
        if (self->arg_ipv6_primary_dns_address_preference)
            g_array_unref (self->arg_ipv6_primary_dns_address_preference);
        if (self->arg_ipv6_secondary_dns_address_preference)
            g_array_unref (self->arg_ipv6_secondary_dns_address_preference);
        g_slice_free (QmiMessageWdsGetProfileSettingsOutput, self);
    }
}

static gchar *
qmi_message_wds_get_profile_settings_input_profile_id_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_INPUT_TLV_PROFILE_ID, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " profile_type = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_PROFILE_TYPE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_profile_type_get_string ((QmiWdsProfileType)tmp));
#elif defined  __QMI_WDS_PROFILE_TYPE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_profile_type_build_string_from_mask ((QmiWdsProfileType)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsProfileType
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " profile_index = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_ipv6_address_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV6_ADDRESS_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " address = '");
    {
        guint address_i;
        guint16 address_n_items = 8;


        g_string_append (printable, "{");

        for (address_i = 0; address_i < address_n_items; address_i++) {
            g_string_append_printf (printable, " [%u] = '", address_i);

            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%" G_GUINT16_FORMAT "", tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_profile_name_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PROFILE_NAME, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_pdp_type_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PDP_TYPE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_PDP_TYPE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_pdp_type_get_string ((QmiWdsPdpType)tmp));
#elif defined  __QMI_WDS_PDP_TYPE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_pdp_type_build_string_from_mask ((QmiWdsPdpType)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsPdpType
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_apn_name_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_APN_NAME, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_primary_ipv4_dns_address_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PRIMARY_IPV4_DNS_ADDRESS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_secondary_ipv4_dns_address_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_SECONDARY_IPV4_DNS_ADDRESS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_gprs_requested_qos_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_GPRS_REQUESTED_QOS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " precedence_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " delay_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " reliability_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " peak_throughput_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " mean_throughput_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_gprs_minimum_qos_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_GPRS_MINIMUM_QOS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " precedence_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " delay_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " reliability_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " peak_throughput_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " mean_throughput_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_username_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_USERNAME, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_password_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PASSWORD, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_authentication_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_AUTHENTICATION, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_AUTHENTICATION_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_authentication_get_string ((QmiWdsAuthentication)tmp));
#elif defined  __QMI_WDS_AUTHENTICATION_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_authentication_build_string_from_mask ((QmiWdsAuthentication)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsAuthentication
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_ipv4_address_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV4_ADDRESS_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_pcscf_address_using_pco_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_PCO, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_pcscf_address_using_dhcp_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_DHCP, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_imcn_flag_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IMCN_FLAG, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_ipv6_primary_dns_address_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV6_PRIMARY_DNS_ADDRESS_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint ipv6_primary_dns_address_preference_i;
        guint16 ipv6_primary_dns_address_preference_n_items = 8;


        g_string_append (printable, "{");

        for (ipv6_primary_dns_address_preference_i = 0; ipv6_primary_dns_address_preference_i < ipv6_primary_dns_address_preference_n_items; ipv6_primary_dns_address_preference_i++) {
            g_string_append_printf (printable, " [%u] = '", ipv6_primary_dns_address_preference_i);

            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%" G_GUINT16_FORMAT "", tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_ipv6_secondary_dns_address_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV6_SECONDARY_DNS_ADDRESS_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint ipv6_secondary_dns_address_preference_i;
        guint16 ipv6_secondary_dns_address_preference_n_items = 8;


        g_string_append (printable, "{");

        for (ipv6_secondary_dns_address_preference_i = 0; ipv6_secondary_dns_address_preference_i < ipv6_secondary_dns_address_preference_n_items; ipv6_secondary_dns_address_preference_i++) {
            g_string_append_printf (printable, " [%u] = '", ipv6_secondary_dns_address_preference_i);

            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%" G_GUINT16_FORMAT "", tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_profile_settings_output_extended_error_code_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_EXTENDED_ERROR_CODE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_DS_PROFILE_ERROR_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_ds_profile_error_get_string ((QmiWdsDsProfileError)tmp));
#elif defined  __QMI_WDS_DS_PROFILE_ERROR_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_ds_profile_error_build_string_from_mask ((QmiWdsDsProfileError)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsDsProfileError
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_get_profile_settings_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_get_profile_settings_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_get_profile_settings_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_INPUT_TLV_PROFILE_ID:
            tlv_type_str = "Profile ID";
            translated_value = qmi_message_wds_get_profile_settings_input_profile_id_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV6_ADDRESS_PREFERENCE:
            tlv_type_str = "IPv6 Address Preference";
            translated_value = qmi_message_wds_get_profile_settings_output_ipv6_address_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PROFILE_NAME:
            tlv_type_str = "Profile Name";
            translated_value = qmi_message_wds_get_profile_settings_output_profile_name_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PDP_TYPE:
            tlv_type_str = "PDP Type";
            translated_value = qmi_message_wds_get_profile_settings_output_pdp_type_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_APN_NAME:
            tlv_type_str = "APN Name";
            translated_value = qmi_message_wds_get_profile_settings_output_apn_name_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PRIMARY_IPV4_DNS_ADDRESS:
            tlv_type_str = "Primary IPv4 DNS Address";
            translated_value = qmi_message_wds_get_profile_settings_output_primary_ipv4_dns_address_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_SECONDARY_IPV4_DNS_ADDRESS:
            tlv_type_str = "Secondary IPv4 DNS Address";
            translated_value = qmi_message_wds_get_profile_settings_output_secondary_ipv4_dns_address_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_GPRS_REQUESTED_QOS:
            tlv_type_str = "GPRS Requested QoS";
            translated_value = qmi_message_wds_get_profile_settings_output_gprs_requested_qos_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_GPRS_MINIMUM_QOS:
            tlv_type_str = "GPRS Minimum QoS";
            translated_value = qmi_message_wds_get_profile_settings_output_gprs_minimum_qos_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_USERNAME:
            tlv_type_str = "Username";
            translated_value = qmi_message_wds_get_profile_settings_output_username_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PASSWORD:
            tlv_type_str = "Password";
            translated_value = qmi_message_wds_get_profile_settings_output_password_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_AUTHENTICATION:
            tlv_type_str = "Authentication";
            translated_value = qmi_message_wds_get_profile_settings_output_authentication_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV4_ADDRESS_PREFERENCE:
            tlv_type_str = "IPv4 Address Preference";
            translated_value = qmi_message_wds_get_profile_settings_output_ipv4_address_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_PCO:
            tlv_type_str = "PCSCF Address Using PCO";
            translated_value = qmi_message_wds_get_profile_settings_output_pcscf_address_using_pco_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_DHCP:
            tlv_type_str = "PCSCF Address Using DHCP";
            translated_value = qmi_message_wds_get_profile_settings_output_pcscf_address_using_dhcp_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IMCN_FLAG:
            tlv_type_str = "IMCN Flag";
            translated_value = qmi_message_wds_get_profile_settings_output_imcn_flag_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV6_PRIMARY_DNS_ADDRESS_PREFERENCE:
            tlv_type_str = "IPv6 Primary DNS Address Preference";
            translated_value = qmi_message_wds_get_profile_settings_output_ipv6_primary_dns_address_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV6_SECONDARY_DNS_ADDRESS_PREFERENCE:
            tlv_type_str = "IPv6 Secondary DNS Address Preference";
            translated_value = qmi_message_wds_get_profile_settings_output_ipv6_secondary_dns_address_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_EXTENDED_ERROR_CODE:
            tlv_type_str = "Extended Error Code";
            translated_value = qmi_message_wds_get_profile_settings_output_extended_error_code_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_get_profile_settings_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Get Profile Settings\" (0x002B)\n",
                            line_prefix);

    {
        struct message_get_profile_settings_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_get_profile_settings_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsGetProfileSettingsOutput *
__qmi_message_wds_get_profile_settings_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsGetProfileSettingsOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS, NULL);

    self = g_slice_new0 (QmiMessageWdsGetProfileSettingsOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_get_profile_settings_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_get_profile_settings_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV6_ADDRESS_PREFERENCE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_ipv6_address_preference_out;
            }
            {
                guint address_i;
                guint16 address_n_items = 8;


                self->arg_ipv6_address_preference_address = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (guint16),
                    (guint)address_n_items);

                for (address_i = 0; address_i < address_n_items; address_i++) {
                    guint16 address_aux;

                    if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &(address_aux), NULL))
                        goto qmi_message_wds_get_profile_settings_output_ipv6_address_preference_out;
                    g_array_insert_val (self->arg_ipv6_address_preference_address, address_i, address_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv6 Address Preference' TLV", offset);
            }

            self->arg_ipv6_address_preference_set = TRUE;

qmi_message_wds_get_profile_settings_output_ipv6_address_preference_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PROFILE_NAME, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_profile_name_out;
            }
            if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &(self->arg_profile_name), NULL))
                goto qmi_message_wds_get_profile_settings_output_profile_name_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Profile Name' TLV", offset);
            }

            self->arg_profile_name_set = TRUE;

qmi_message_wds_get_profile_settings_output_profile_name_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PDP_TYPE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_pdp_type_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_profile_settings_output_pdp_type_out;
                self->arg_pdp_type = (QmiWdsPdpType)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'PDP Type' TLV", offset);
            }

            self->arg_pdp_type_set = TRUE;

qmi_message_wds_get_profile_settings_output_pdp_type_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_APN_NAME, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_apn_name_out;
            }
            if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &(self->arg_apn_name), NULL))
                goto qmi_message_wds_get_profile_settings_output_apn_name_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'APN Name' TLV", offset);
            }

            self->arg_apn_name_set = TRUE;

qmi_message_wds_get_profile_settings_output_apn_name_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PRIMARY_IPV4_DNS_ADDRESS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_primary_ipv4_dns_address_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_primary_ipv4_dns_address), NULL))
                goto qmi_message_wds_get_profile_settings_output_primary_ipv4_dns_address_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Primary IPv4 DNS Address' TLV", offset);
            }

            self->arg_primary_ipv4_dns_address_set = TRUE;

qmi_message_wds_get_profile_settings_output_primary_ipv4_dns_address_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_SECONDARY_IPV4_DNS_ADDRESS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_secondary_ipv4_dns_address_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_secondary_ipv4_dns_address), NULL))
                goto qmi_message_wds_get_profile_settings_output_secondary_ipv4_dns_address_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Secondary IPv4 DNS Address' TLV", offset);
            }

            self->arg_secondary_ipv4_dns_address_set = TRUE;

qmi_message_wds_get_profile_settings_output_secondary_ipv4_dns_address_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_GPRS_REQUESTED_QOS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_gprs_requested_qos_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_requested_qos_precedence_class), NULL))
                goto qmi_message_wds_get_profile_settings_output_gprs_requested_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_requested_qos_delay_class), NULL))
                goto qmi_message_wds_get_profile_settings_output_gprs_requested_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_requested_qos_reliability_class), NULL))
                goto qmi_message_wds_get_profile_settings_output_gprs_requested_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_requested_qos_peak_throughput_class), NULL))
                goto qmi_message_wds_get_profile_settings_output_gprs_requested_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_requested_qos_mean_throughput_class), NULL))
                goto qmi_message_wds_get_profile_settings_output_gprs_requested_qos_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'GPRS Requested QoS' TLV", offset);
            }

            self->arg_gprs_requested_qos_set = TRUE;

qmi_message_wds_get_profile_settings_output_gprs_requested_qos_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_GPRS_MINIMUM_QOS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_gprs_minimum_qos_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_minimum_qos_precedence_class), NULL))
                goto qmi_message_wds_get_profile_settings_output_gprs_minimum_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_minimum_qos_delay_class), NULL))
                goto qmi_message_wds_get_profile_settings_output_gprs_minimum_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_minimum_qos_reliability_class), NULL))
                goto qmi_message_wds_get_profile_settings_output_gprs_minimum_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_minimum_qos_peak_throughput_class), NULL))
                goto qmi_message_wds_get_profile_settings_output_gprs_minimum_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_minimum_qos_mean_throughput_class), NULL))
                goto qmi_message_wds_get_profile_settings_output_gprs_minimum_qos_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'GPRS Minimum QoS' TLV", offset);
            }

            self->arg_gprs_minimum_qos_set = TRUE;

qmi_message_wds_get_profile_settings_output_gprs_minimum_qos_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_USERNAME, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_username_out;
            }
            if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &(self->arg_username), NULL))
                goto qmi_message_wds_get_profile_settings_output_username_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Username' TLV", offset);
            }

            self->arg_username_set = TRUE;

qmi_message_wds_get_profile_settings_output_username_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PASSWORD, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_password_out;
            }
            if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &(self->arg_password), NULL))
                goto qmi_message_wds_get_profile_settings_output_password_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Password' TLV", offset);
            }

            self->arg_password_set = TRUE;

qmi_message_wds_get_profile_settings_output_password_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_AUTHENTICATION, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_authentication_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_profile_settings_output_authentication_out;
                self->arg_authentication = (QmiWdsAuthentication)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Authentication' TLV", offset);
            }

            self->arg_authentication_set = TRUE;

qmi_message_wds_get_profile_settings_output_authentication_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV4_ADDRESS_PREFERENCE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_ipv4_address_preference_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_ipv4_address_preference), NULL))
                goto qmi_message_wds_get_profile_settings_output_ipv4_address_preference_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv4 Address Preference' TLV", offset);
            }

            self->arg_ipv4_address_preference_set = TRUE;

qmi_message_wds_get_profile_settings_output_ipv4_address_preference_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_PCO, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_pcscf_address_using_pco_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_profile_settings_output_pcscf_address_using_pco_out;
                self->arg_pcscf_address_using_pco = (gboolean)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'PCSCF Address Using PCO' TLV", offset);
            }

            self->arg_pcscf_address_using_pco_set = TRUE;

qmi_message_wds_get_profile_settings_output_pcscf_address_using_pco_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_DHCP, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_pcscf_address_using_dhcp_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_profile_settings_output_pcscf_address_using_dhcp_out;
                self->arg_pcscf_address_using_dhcp = (gboolean)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'PCSCF Address Using DHCP' TLV", offset);
            }

            self->arg_pcscf_address_using_dhcp_set = TRUE;

qmi_message_wds_get_profile_settings_output_pcscf_address_using_dhcp_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IMCN_FLAG, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_imcn_flag_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_profile_settings_output_imcn_flag_out;
                self->arg_imcn_flag = (gboolean)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IMCN Flag' TLV", offset);
            }

            self->arg_imcn_flag_set = TRUE;

qmi_message_wds_get_profile_settings_output_imcn_flag_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV6_PRIMARY_DNS_ADDRESS_PREFERENCE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_ipv6_primary_dns_address_preference_out;
            }
            {
                guint ipv6_primary_dns_address_preference_i;
                guint16 ipv6_primary_dns_address_preference_n_items = 8;


                self->arg_ipv6_primary_dns_address_preference = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (guint16),
                    (guint)ipv6_primary_dns_address_preference_n_items);

                for (ipv6_primary_dns_address_preference_i = 0; ipv6_primary_dns_address_preference_i < ipv6_primary_dns_address_preference_n_items; ipv6_primary_dns_address_preference_i++) {
                    guint16 ipv6_primary_dns_address_preference_aux;

                    if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &(ipv6_primary_dns_address_preference_aux), NULL))
                        goto qmi_message_wds_get_profile_settings_output_ipv6_primary_dns_address_preference_out;
                    g_array_insert_val (self->arg_ipv6_primary_dns_address_preference, ipv6_primary_dns_address_preference_i, ipv6_primary_dns_address_preference_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv6 Primary DNS Address Preference' TLV", offset);
            }

            self->arg_ipv6_primary_dns_address_preference_set = TRUE;

qmi_message_wds_get_profile_settings_output_ipv6_primary_dns_address_preference_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_IPV6_SECONDARY_DNS_ADDRESS_PREFERENCE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_ipv6_secondary_dns_address_preference_out;
            }
            {
                guint ipv6_secondary_dns_address_preference_i;
                guint16 ipv6_secondary_dns_address_preference_n_items = 8;


                self->arg_ipv6_secondary_dns_address_preference = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (guint16),
                    (guint)ipv6_secondary_dns_address_preference_n_items);

                for (ipv6_secondary_dns_address_preference_i = 0; ipv6_secondary_dns_address_preference_i < ipv6_secondary_dns_address_preference_n_items; ipv6_secondary_dns_address_preference_i++) {
                    guint16 ipv6_secondary_dns_address_preference_aux;

                    if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &(ipv6_secondary_dns_address_preference_aux), NULL))
                        goto qmi_message_wds_get_profile_settings_output_ipv6_secondary_dns_address_preference_out;
                    g_array_insert_val (self->arg_ipv6_secondary_dns_address_preference, ipv6_secondary_dns_address_preference_i, ipv6_secondary_dns_address_preference_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv6 Secondary DNS Address Preference' TLV", offset);
            }

            self->arg_ipv6_secondary_dns_address_preference_set = TRUE;

qmi_message_wds_get_profile_settings_output_ipv6_secondary_dns_address_preference_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status != QMI_STATUS_SUCCESS))
            break;
        /* Prerequisite.... */
        if (!(self->arg_result.error_code == QMI_PROTOCOL_ERROR_EXTENDED_INTERNAL))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS_OUTPUT_TLV_EXTENDED_ERROR_CODE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_profile_settings_output_extended_error_code_out;
            }
            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, NULL))
                    goto qmi_message_wds_get_profile_settings_output_extended_error_code_out;
                self->arg_extended_error_code = (QmiWdsDsProfileError)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Extended Error Code' TLV", offset);
            }

            self->arg_extended_error_code_set = TRUE;

qmi_message_wds_get_profile_settings_output_extended_error_code_out:
            ;

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Get Default Settings */


/* --- Input -- */

struct _QmiMessageWdsGetDefaultSettingsInput {
    volatile gint ref_count;

    /* Profile Type */
    gboolean arg_profile_type_set;
    guint8 arg_profile_type;
};

#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_INPUT_TLV_PROFILE_TYPE 0x01

/**
 * qmi_message_wds_get_default_settings_input_get_profile_type:
 * @self: a #QmiMessageWdsGetDefaultSettingsInput.
 * @value_profile_type: a placeholder for the output #QmiWdsProfileType, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Profile Type' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_input_get_profile_type (
    QmiMessageWdsGetDefaultSettingsInput *self,
    QmiWdsProfileType *value_profile_type,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_profile_type_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Profile Type' was not found in the message");
        return FALSE;
    }

    if (value_profile_type)
        *value_profile_type = (QmiWdsProfileType)(self->arg_profile_type);

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_input_set_profile_type:
 * @self: a #QmiMessageWdsGetDefaultSettingsInput.
 * @value_profile_type: a #QmiWdsProfileType.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Profile Type' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_input_set_profile_type (
    QmiMessageWdsGetDefaultSettingsInput *self,
    QmiWdsProfileType value_profile_type,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_profile_type = (guint8)(value_profile_type);
    self->arg_profile_type_set = TRUE;

    return TRUE;
}

GType
qmi_message_wds_get_default_settings_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetDefaultSettingsInput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_default_settings_input_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_default_settings_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_default_settings_input_ref:
 * @self: a #QmiMessageWdsGetDefaultSettingsInput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetDefaultSettingsInput *
qmi_message_wds_get_default_settings_input_ref (QmiMessageWdsGetDefaultSettingsInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_default_settings_input_unref:
 * @self: a #QmiMessageWdsGetDefaultSettingsInput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_default_settings_input_unref (QmiMessageWdsGetDefaultSettingsInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsGetDefaultSettingsInput, self);
    }
}

/**
 * qmi_message_wds_get_default_settings_input_new:
 *
 * Allocates a new #QmiMessageWdsGetDefaultSettingsInput.
 *
 * Returns: the newly created #QmiMessageWdsGetDefaultSettingsInput. The returned value should be freed with qmi_message_wds_get_default_settings_input_unref().
 */
QmiMessageWdsGetDefaultSettingsInput *
qmi_message_wds_get_default_settings_input_new (void)
{
    QmiMessageWdsGetDefaultSettingsInput *self;

    self = g_slice_new0 (QmiMessageWdsGetDefaultSettingsInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_wds_get_default_settings_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageWdsGetDefaultSettingsInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS);

    /* All TLVs are optional, we allow NULL input */
    if (!input)
        return self;

    /* Try to add the 'Profile Type' TLV */
    if (input->arg_profile_type_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_INPUT_TLV_PROFILE_TYPE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Profile Type': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_profile_type;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Profile Type': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Profile Type': ");
            goto error_out;
        }
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

struct _QmiMessageWdsGetDefaultSettingsOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;

    /* IPv6 Address Preference */
    gboolean arg_ipv6_address_preference_set;
    GArray *arg_ipv6_address_preference_address;

    /* Profile Name */
    gboolean arg_profile_name_set;
    gchar *arg_profile_name;

    /* PDP Type */
    gboolean arg_pdp_type_set;
    guint8 arg_pdp_type;

    /* APN Name */
    gboolean arg_apn_name_set;
    gchar *arg_apn_name;

    /* Primary IPv4 DNS Address */
    gboolean arg_primary_ipv4_dns_address_set;
    guint32 arg_primary_ipv4_dns_address;

    /* Secondary IPv4 DNS Address */
    gboolean arg_secondary_ipv4_dns_address_set;
    guint32 arg_secondary_ipv4_dns_address;

    /* GPRS Requested QoS */
    gboolean arg_gprs_requested_qos_set;
    guint32 arg_gprs_requested_qos_precedence_class;
    guint32 arg_gprs_requested_qos_delay_class;
    guint32 arg_gprs_requested_qos_reliability_class;
    guint32 arg_gprs_requested_qos_peak_throughput_class;
    guint32 arg_gprs_requested_qos_mean_throughput_class;

    /* GPRS Minimum QoS */
    gboolean arg_gprs_minimum_qos_set;
    guint32 arg_gprs_minimum_qos_precedence_class;
    guint32 arg_gprs_minimum_qos_delay_class;
    guint32 arg_gprs_minimum_qos_reliability_class;
    guint32 arg_gprs_minimum_qos_peak_throughput_class;
    guint32 arg_gprs_minimum_qos_mean_throughput_class;

    /* Username */
    gboolean arg_username_set;
    gchar *arg_username;

    /* Password */
    gboolean arg_password_set;
    gchar *arg_password;

    /* Authentication */
    gboolean arg_authentication_set;
    guint8 arg_authentication;

    /* IPv4 Address Preference */
    gboolean arg_ipv4_address_preference_set;
    guint32 arg_ipv4_address_preference;

    /* PCSCF Address Using PCO */
    gboolean arg_pcscf_address_using_pco_set;
    guint8 arg_pcscf_address_using_pco;

    /* PCSCF Address Using DHCP */
    gboolean arg_pcscf_address_using_dhcp_set;
    guint8 arg_pcscf_address_using_dhcp;

    /* IMCN Flag */
    gboolean arg_imcn_flag_set;
    guint8 arg_imcn_flag;

    /* IPv6 Primary DNS Address Preference */
    gboolean arg_ipv6_primary_dns_address_preference_set;
    GArray *arg_ipv6_primary_dns_address_preference;

    /* IPv6 Secondary DNS Address Preference */
    gboolean arg_ipv6_secondary_dns_address_preference_set;
    GArray *arg_ipv6_secondary_dns_address_preference;

    /* Extended Error Code */
    gboolean arg_extended_error_code_set;
    guint16 arg_extended_error_code;
};

#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_RESULT 0x02
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV6_ADDRESS_PREFERENCE 0x28
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PROFILE_NAME 0x10
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PDP_TYPE 0x11
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_APN_NAME 0x14
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PRIMARY_IPV4_DNS_ADDRESS 0x15
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_SECONDARY_IPV4_DNS_ADDRESS 0x16
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_GPRS_REQUESTED_QOS 0x19
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_GPRS_MINIMUM_QOS 0x1A
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_USERNAME 0x1B
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PASSWORD 0x1C
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_AUTHENTICATION 0x1D
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV4_ADDRESS_PREFERENCE 0x1E
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_PCO 0x1F
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_DHCP 0x21
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IMCN_FLAG 0x22
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV6_PRIMARY_DNS_ADDRESS_PREFERENCE 0x2B
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV6_SECONDARY_DNS_ADDRESS_PREFERENCE 0x2C
#define QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_EXTENDED_ERROR_CODE 0xE0

/**
 * qmi_message_wds_get_default_settings_output_get_result:
 * @self: a QmiMessageWdsGetDefaultSettingsOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_result (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_ipv6_address_preference:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_ipv6_address_preference_address: a placeholder for the output #GArray of #guint16 elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv6 Address Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_ipv6_address_preference (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    GArray **value_ipv6_address_preference_address,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv6_address_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv6 Address Preference' was not found in the message");
        return FALSE;
    }

    if (value_ipv6_address_preference_address)
        *value_ipv6_address_preference_address = self->arg_ipv6_address_preference_address;

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_profile_name:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_profile_name: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Profile Name' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_profile_name (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    const gchar **value_profile_name,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_profile_name_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Profile Name' was not found in the message");
        return FALSE;
    }

    if (value_profile_name)
        *value_profile_name = self->arg_profile_name;

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_pdp_type:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_pdp_type: a placeholder for the output #QmiWdsPdpType, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'PDP Type' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_pdp_type (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    QmiWdsPdpType *value_pdp_type,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_pdp_type_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'PDP Type' was not found in the message");
        return FALSE;
    }

    if (value_pdp_type)
        *value_pdp_type = (QmiWdsPdpType)(self->arg_pdp_type);

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_apn_name:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_apn_name: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'APN Name' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_apn_name (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    const gchar **value_apn_name,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_apn_name_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'APN Name' was not found in the message");
        return FALSE;
    }

    if (value_apn_name)
        *value_apn_name = self->arg_apn_name;

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_primary_ipv4_dns_address:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_primary_ipv4_dns_address: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Primary IPv4 DNS Address' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_primary_ipv4_dns_address (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    guint32 *value_primary_ipv4_dns_address,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_primary_ipv4_dns_address_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Primary IPv4 DNS Address' was not found in the message");
        return FALSE;
    }

    if (value_primary_ipv4_dns_address)
        *value_primary_ipv4_dns_address = self->arg_primary_ipv4_dns_address;

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_secondary_ipv4_dns_address:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_secondary_ipv4_dns_address: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Secondary IPv4 DNS Address' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_secondary_ipv4_dns_address (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    guint32 *value_secondary_ipv4_dns_address,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_secondary_ipv4_dns_address_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Secondary IPv4 DNS Address' was not found in the message");
        return FALSE;
    }

    if (value_secondary_ipv4_dns_address)
        *value_secondary_ipv4_dns_address = self->arg_secondary_ipv4_dns_address;

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_gprs_requested_qos:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_gprs_requested_qos_precedence_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_requested_qos_delay_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_requested_qos_reliability_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_requested_qos_peak_throughput_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_requested_qos_mean_throughput_class: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'GPRS Requested QoS' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_gprs_requested_qos (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    guint32 *value_gprs_requested_qos_precedence_class,
    guint32 *value_gprs_requested_qos_delay_class,
    guint32 *value_gprs_requested_qos_reliability_class,
    guint32 *value_gprs_requested_qos_peak_throughput_class,
    guint32 *value_gprs_requested_qos_mean_throughput_class,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_gprs_requested_qos_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'GPRS Requested QoS' was not found in the message");
        return FALSE;
    }

    if (value_gprs_requested_qos_precedence_class)
        *value_gprs_requested_qos_precedence_class = self->arg_gprs_requested_qos_precedence_class;
    if (value_gprs_requested_qos_delay_class)
        *value_gprs_requested_qos_delay_class = self->arg_gprs_requested_qos_delay_class;
    if (value_gprs_requested_qos_reliability_class)
        *value_gprs_requested_qos_reliability_class = self->arg_gprs_requested_qos_reliability_class;
    if (value_gprs_requested_qos_peak_throughput_class)
        *value_gprs_requested_qos_peak_throughput_class = self->arg_gprs_requested_qos_peak_throughput_class;
    if (value_gprs_requested_qos_mean_throughput_class)
        *value_gprs_requested_qos_mean_throughput_class = self->arg_gprs_requested_qos_mean_throughput_class;

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_gprs_minimum_qos:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_gprs_minimum_qos_precedence_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_minimum_qos_delay_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_minimum_qos_reliability_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_minimum_qos_peak_throughput_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_minimum_qos_mean_throughput_class: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'GPRS Minimum QoS' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_gprs_minimum_qos (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    guint32 *value_gprs_minimum_qos_precedence_class,
    guint32 *value_gprs_minimum_qos_delay_class,
    guint32 *value_gprs_minimum_qos_reliability_class,
    guint32 *value_gprs_minimum_qos_peak_throughput_class,
    guint32 *value_gprs_minimum_qos_mean_throughput_class,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_gprs_minimum_qos_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'GPRS Minimum QoS' was not found in the message");
        return FALSE;
    }

    if (value_gprs_minimum_qos_precedence_class)
        *value_gprs_minimum_qos_precedence_class = self->arg_gprs_minimum_qos_precedence_class;
    if (value_gprs_minimum_qos_delay_class)
        *value_gprs_minimum_qos_delay_class = self->arg_gprs_minimum_qos_delay_class;
    if (value_gprs_minimum_qos_reliability_class)
        *value_gprs_minimum_qos_reliability_class = self->arg_gprs_minimum_qos_reliability_class;
    if (value_gprs_minimum_qos_peak_throughput_class)
        *value_gprs_minimum_qos_peak_throughput_class = self->arg_gprs_minimum_qos_peak_throughput_class;
    if (value_gprs_minimum_qos_mean_throughput_class)
        *value_gprs_minimum_qos_mean_throughput_class = self->arg_gprs_minimum_qos_mean_throughput_class;

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_username:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_username: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Username' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_username (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    const gchar **value_username,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_username_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Username' was not found in the message");
        return FALSE;
    }

    if (value_username)
        *value_username = self->arg_username;

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_password:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_password: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Password' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_password (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    const gchar **value_password,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_password_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Password' was not found in the message");
        return FALSE;
    }

    if (value_password)
        *value_password = self->arg_password;

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_authentication:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_authentication: a placeholder for the output #QmiWdsAuthentication, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Authentication' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_authentication (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    QmiWdsAuthentication *value_authentication,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_authentication_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Authentication' was not found in the message");
        return FALSE;
    }

    if (value_authentication)
        *value_authentication = (QmiWdsAuthentication)(self->arg_authentication);

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_ipv4_address_preference:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_ipv4_address_preference: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv4 Address Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_ipv4_address_preference (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    guint32 *value_ipv4_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv4_address_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv4 Address Preference' was not found in the message");
        return FALSE;
    }

    if (value_ipv4_address_preference)
        *value_ipv4_address_preference = self->arg_ipv4_address_preference;

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_pcscf_address_using_pco:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_pcscf_address_using_pco: a placeholder for the output #gboolean, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'PCSCF Address Using PCO' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_pcscf_address_using_pco (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    gboolean *value_pcscf_address_using_pco,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_pcscf_address_using_pco_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'PCSCF Address Using PCO' was not found in the message");
        return FALSE;
    }

    if (value_pcscf_address_using_pco)
        *value_pcscf_address_using_pco = (gboolean)(self->arg_pcscf_address_using_pco);

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_pcscf_address_using_dhcp:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_pcscf_address_using_dhcp: a placeholder for the output #gboolean, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'PCSCF Address Using DHCP' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_pcscf_address_using_dhcp (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    gboolean *value_pcscf_address_using_dhcp,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_pcscf_address_using_dhcp_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'PCSCF Address Using DHCP' was not found in the message");
        return FALSE;
    }

    if (value_pcscf_address_using_dhcp)
        *value_pcscf_address_using_dhcp = (gboolean)(self->arg_pcscf_address_using_dhcp);

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_imcn_flag:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_imcn_flag: a placeholder for the output #gboolean, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IMCN Flag' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_imcn_flag (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    gboolean *value_imcn_flag,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_imcn_flag_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IMCN Flag' was not found in the message");
        return FALSE;
    }

    if (value_imcn_flag)
        *value_imcn_flag = (gboolean)(self->arg_imcn_flag);

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_ipv6_primary_dns_address_preference:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_ipv6_primary_dns_address_preference: a placeholder for the output #GArray of #guint16 elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv6 Primary DNS Address Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_ipv6_primary_dns_address_preference (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    GArray **value_ipv6_primary_dns_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv6_primary_dns_address_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv6 Primary DNS Address Preference' was not found in the message");
        return FALSE;
    }

    if (value_ipv6_primary_dns_address_preference)
        *value_ipv6_primary_dns_address_preference = self->arg_ipv6_primary_dns_address_preference;

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_ipv6_secondary_dns_address_preference:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_ipv6_secondary_dns_address_preference: a placeholder for the output #GArray of #guint16 elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv6 Secondary DNS Address Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_ipv6_secondary_dns_address_preference (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    GArray **value_ipv6_secondary_dns_address_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv6_secondary_dns_address_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv6 Secondary DNS Address Preference' was not found in the message");
        return FALSE;
    }

    if (value_ipv6_secondary_dns_address_preference)
        *value_ipv6_secondary_dns_address_preference = self->arg_ipv6_secondary_dns_address_preference;

    return TRUE;
}

/**
 * qmi_message_wds_get_default_settings_output_get_extended_error_code:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 * @value_extended_error_code: a placeholder for the output #QmiWdsDsProfileError, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Extended Error Code' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_default_settings_output_get_extended_error_code (
    QmiMessageWdsGetDefaultSettingsOutput *self,
    QmiWdsDsProfileError *value_extended_error_code,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_extended_error_code_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Extended Error Code' was not found in the message");
        return FALSE;
    }

    if (value_extended_error_code)
        *value_extended_error_code = (QmiWdsDsProfileError)(self->arg_extended_error_code);

    return TRUE;
}

GType
qmi_message_wds_get_default_settings_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetDefaultSettingsOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_default_settings_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_default_settings_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_default_settings_output_ref:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetDefaultSettingsOutput *
qmi_message_wds_get_default_settings_output_ref (QmiMessageWdsGetDefaultSettingsOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_default_settings_output_unref:
 * @self: a #QmiMessageWdsGetDefaultSettingsOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_default_settings_output_unref (QmiMessageWdsGetDefaultSettingsOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        if (self->arg_ipv6_address_preference_address)
            g_array_unref (self->arg_ipv6_address_preference_address);
        g_free (self->arg_profile_name);
        g_free (self->arg_apn_name);
        g_free (self->arg_username);
        g_free (self->arg_password);
        if (self->arg_ipv6_primary_dns_address_preference)
            g_array_unref (self->arg_ipv6_primary_dns_address_preference);
        if (self->arg_ipv6_secondary_dns_address_preference)
            g_array_unref (self->arg_ipv6_secondary_dns_address_preference);
        g_slice_free (QmiMessageWdsGetDefaultSettingsOutput, self);
    }
}

static gchar *
qmi_message_wds_get_default_settings_input_profile_type_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_INPUT_TLV_PROFILE_TYPE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_PROFILE_TYPE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_profile_type_get_string ((QmiWdsProfileType)tmp));
#elif defined  __QMI_WDS_PROFILE_TYPE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_profile_type_build_string_from_mask ((QmiWdsProfileType)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsProfileType
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_ipv6_address_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV6_ADDRESS_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " address = '");
    {
        guint address_i;
        guint16 address_n_items = 8;


        g_string_append (printable, "{");

        for (address_i = 0; address_i < address_n_items; address_i++) {
            g_string_append_printf (printable, " [%u] = '", address_i);

            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%" G_GUINT16_FORMAT "", tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_profile_name_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PROFILE_NAME, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_pdp_type_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PDP_TYPE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_PDP_TYPE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_pdp_type_get_string ((QmiWdsPdpType)tmp));
#elif defined  __QMI_WDS_PDP_TYPE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_pdp_type_build_string_from_mask ((QmiWdsPdpType)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsPdpType
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_apn_name_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_APN_NAME, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_primary_ipv4_dns_address_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PRIMARY_IPV4_DNS_ADDRESS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_secondary_ipv4_dns_address_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_SECONDARY_IPV4_DNS_ADDRESS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_gprs_requested_qos_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_GPRS_REQUESTED_QOS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " precedence_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " delay_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " reliability_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " peak_throughput_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " mean_throughput_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_gprs_minimum_qos_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_GPRS_MINIMUM_QOS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " precedence_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " delay_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " reliability_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " peak_throughput_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " mean_throughput_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_username_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_USERNAME, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_password_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PASSWORD, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_authentication_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_AUTHENTICATION, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_AUTHENTICATION_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_authentication_get_string ((QmiWdsAuthentication)tmp));
#elif defined  __QMI_WDS_AUTHENTICATION_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_authentication_build_string_from_mask ((QmiWdsAuthentication)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsAuthentication
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_ipv4_address_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV4_ADDRESS_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_pcscf_address_using_pco_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_PCO, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_pcscf_address_using_dhcp_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_DHCP, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_imcn_flag_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IMCN_FLAG, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_ipv6_primary_dns_address_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV6_PRIMARY_DNS_ADDRESS_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint ipv6_primary_dns_address_preference_i;
        guint16 ipv6_primary_dns_address_preference_n_items = 8;


        g_string_append (printable, "{");

        for (ipv6_primary_dns_address_preference_i = 0; ipv6_primary_dns_address_preference_i < ipv6_primary_dns_address_preference_n_items; ipv6_primary_dns_address_preference_i++) {
            g_string_append_printf (printable, " [%u] = '", ipv6_primary_dns_address_preference_i);

            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%" G_GUINT16_FORMAT "", tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_ipv6_secondary_dns_address_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV6_SECONDARY_DNS_ADDRESS_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint ipv6_secondary_dns_address_preference_i;
        guint16 ipv6_secondary_dns_address_preference_n_items = 8;


        g_string_append (printable, "{");

        for (ipv6_secondary_dns_address_preference_i = 0; ipv6_secondary_dns_address_preference_i < ipv6_secondary_dns_address_preference_n_items; ipv6_secondary_dns_address_preference_i++) {
            g_string_append_printf (printable, " [%u] = '", ipv6_secondary_dns_address_preference_i);

            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%" G_GUINT16_FORMAT "", tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_default_settings_output_extended_error_code_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_EXTENDED_ERROR_CODE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_DS_PROFILE_ERROR_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_ds_profile_error_get_string ((QmiWdsDsProfileError)tmp));
#elif defined  __QMI_WDS_DS_PROFILE_ERROR_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_ds_profile_error_build_string_from_mask ((QmiWdsDsProfileError)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsDsProfileError
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_get_default_settings_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_get_default_settings_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_get_default_settings_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_INPUT_TLV_PROFILE_TYPE:
            tlv_type_str = "Profile Type";
            translated_value = qmi_message_wds_get_default_settings_input_profile_type_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV6_ADDRESS_PREFERENCE:
            tlv_type_str = "IPv6 Address Preference";
            translated_value = qmi_message_wds_get_default_settings_output_ipv6_address_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PROFILE_NAME:
            tlv_type_str = "Profile Name";
            translated_value = qmi_message_wds_get_default_settings_output_profile_name_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PDP_TYPE:
            tlv_type_str = "PDP Type";
            translated_value = qmi_message_wds_get_default_settings_output_pdp_type_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_APN_NAME:
            tlv_type_str = "APN Name";
            translated_value = qmi_message_wds_get_default_settings_output_apn_name_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PRIMARY_IPV4_DNS_ADDRESS:
            tlv_type_str = "Primary IPv4 DNS Address";
            translated_value = qmi_message_wds_get_default_settings_output_primary_ipv4_dns_address_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_SECONDARY_IPV4_DNS_ADDRESS:
            tlv_type_str = "Secondary IPv4 DNS Address";
            translated_value = qmi_message_wds_get_default_settings_output_secondary_ipv4_dns_address_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_GPRS_REQUESTED_QOS:
            tlv_type_str = "GPRS Requested QoS";
            translated_value = qmi_message_wds_get_default_settings_output_gprs_requested_qos_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_GPRS_MINIMUM_QOS:
            tlv_type_str = "GPRS Minimum QoS";
            translated_value = qmi_message_wds_get_default_settings_output_gprs_minimum_qos_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_USERNAME:
            tlv_type_str = "Username";
            translated_value = qmi_message_wds_get_default_settings_output_username_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PASSWORD:
            tlv_type_str = "Password";
            translated_value = qmi_message_wds_get_default_settings_output_password_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_AUTHENTICATION:
            tlv_type_str = "Authentication";
            translated_value = qmi_message_wds_get_default_settings_output_authentication_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV4_ADDRESS_PREFERENCE:
            tlv_type_str = "IPv4 Address Preference";
            translated_value = qmi_message_wds_get_default_settings_output_ipv4_address_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_PCO:
            tlv_type_str = "PCSCF Address Using PCO";
            translated_value = qmi_message_wds_get_default_settings_output_pcscf_address_using_pco_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_DHCP:
            tlv_type_str = "PCSCF Address Using DHCP";
            translated_value = qmi_message_wds_get_default_settings_output_pcscf_address_using_dhcp_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IMCN_FLAG:
            tlv_type_str = "IMCN Flag";
            translated_value = qmi_message_wds_get_default_settings_output_imcn_flag_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV6_PRIMARY_DNS_ADDRESS_PREFERENCE:
            tlv_type_str = "IPv6 Primary DNS Address Preference";
            translated_value = qmi_message_wds_get_default_settings_output_ipv6_primary_dns_address_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV6_SECONDARY_DNS_ADDRESS_PREFERENCE:
            tlv_type_str = "IPv6 Secondary DNS Address Preference";
            translated_value = qmi_message_wds_get_default_settings_output_ipv6_secondary_dns_address_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_EXTENDED_ERROR_CODE:
            tlv_type_str = "Extended Error Code";
            translated_value = qmi_message_wds_get_default_settings_output_extended_error_code_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_get_default_settings_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Get Default Settings\" (0x002C)\n",
                            line_prefix);

    {
        struct message_get_default_settings_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_get_default_settings_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsGetDefaultSettingsOutput *
__qmi_message_wds_get_default_settings_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsGetDefaultSettingsOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS, NULL);

    self = g_slice_new0 (QmiMessageWdsGetDefaultSettingsOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_get_default_settings_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_get_default_settings_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV6_ADDRESS_PREFERENCE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_ipv6_address_preference_out;
            }
            {
                guint address_i;
                guint16 address_n_items = 8;


                self->arg_ipv6_address_preference_address = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (guint16),
                    (guint)address_n_items);

                for (address_i = 0; address_i < address_n_items; address_i++) {
                    guint16 address_aux;

                    if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &(address_aux), NULL))
                        goto qmi_message_wds_get_default_settings_output_ipv6_address_preference_out;
                    g_array_insert_val (self->arg_ipv6_address_preference_address, address_i, address_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv6 Address Preference' TLV", offset);
            }

            self->arg_ipv6_address_preference_set = TRUE;

qmi_message_wds_get_default_settings_output_ipv6_address_preference_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PROFILE_NAME, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_profile_name_out;
            }
            if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &(self->arg_profile_name), NULL))
                goto qmi_message_wds_get_default_settings_output_profile_name_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Profile Name' TLV", offset);
            }

            self->arg_profile_name_set = TRUE;

qmi_message_wds_get_default_settings_output_profile_name_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PDP_TYPE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_pdp_type_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_default_settings_output_pdp_type_out;
                self->arg_pdp_type = (QmiWdsPdpType)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'PDP Type' TLV", offset);
            }

            self->arg_pdp_type_set = TRUE;

qmi_message_wds_get_default_settings_output_pdp_type_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_APN_NAME, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_apn_name_out;
            }
            if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &(self->arg_apn_name), NULL))
                goto qmi_message_wds_get_default_settings_output_apn_name_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'APN Name' TLV", offset);
            }

            self->arg_apn_name_set = TRUE;

qmi_message_wds_get_default_settings_output_apn_name_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PRIMARY_IPV4_DNS_ADDRESS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_primary_ipv4_dns_address_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_primary_ipv4_dns_address), NULL))
                goto qmi_message_wds_get_default_settings_output_primary_ipv4_dns_address_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Primary IPv4 DNS Address' TLV", offset);
            }

            self->arg_primary_ipv4_dns_address_set = TRUE;

qmi_message_wds_get_default_settings_output_primary_ipv4_dns_address_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_SECONDARY_IPV4_DNS_ADDRESS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_secondary_ipv4_dns_address_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_secondary_ipv4_dns_address), NULL))
                goto qmi_message_wds_get_default_settings_output_secondary_ipv4_dns_address_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Secondary IPv4 DNS Address' TLV", offset);
            }

            self->arg_secondary_ipv4_dns_address_set = TRUE;

qmi_message_wds_get_default_settings_output_secondary_ipv4_dns_address_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_GPRS_REQUESTED_QOS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_gprs_requested_qos_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_requested_qos_precedence_class), NULL))
                goto qmi_message_wds_get_default_settings_output_gprs_requested_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_requested_qos_delay_class), NULL))
                goto qmi_message_wds_get_default_settings_output_gprs_requested_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_requested_qos_reliability_class), NULL))
                goto qmi_message_wds_get_default_settings_output_gprs_requested_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_requested_qos_peak_throughput_class), NULL))
                goto qmi_message_wds_get_default_settings_output_gprs_requested_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_requested_qos_mean_throughput_class), NULL))
                goto qmi_message_wds_get_default_settings_output_gprs_requested_qos_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'GPRS Requested QoS' TLV", offset);
            }

            self->arg_gprs_requested_qos_set = TRUE;

qmi_message_wds_get_default_settings_output_gprs_requested_qos_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_GPRS_MINIMUM_QOS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_gprs_minimum_qos_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_minimum_qos_precedence_class), NULL))
                goto qmi_message_wds_get_default_settings_output_gprs_minimum_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_minimum_qos_delay_class), NULL))
                goto qmi_message_wds_get_default_settings_output_gprs_minimum_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_minimum_qos_reliability_class), NULL))
                goto qmi_message_wds_get_default_settings_output_gprs_minimum_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_minimum_qos_peak_throughput_class), NULL))
                goto qmi_message_wds_get_default_settings_output_gprs_minimum_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_minimum_qos_mean_throughput_class), NULL))
                goto qmi_message_wds_get_default_settings_output_gprs_minimum_qos_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'GPRS Minimum QoS' TLV", offset);
            }

            self->arg_gprs_minimum_qos_set = TRUE;

qmi_message_wds_get_default_settings_output_gprs_minimum_qos_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_USERNAME, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_username_out;
            }
            if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &(self->arg_username), NULL))
                goto qmi_message_wds_get_default_settings_output_username_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Username' TLV", offset);
            }

            self->arg_username_set = TRUE;

qmi_message_wds_get_default_settings_output_username_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PASSWORD, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_password_out;
            }
            if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &(self->arg_password), NULL))
                goto qmi_message_wds_get_default_settings_output_password_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Password' TLV", offset);
            }

            self->arg_password_set = TRUE;

qmi_message_wds_get_default_settings_output_password_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_AUTHENTICATION, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_authentication_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_default_settings_output_authentication_out;
                self->arg_authentication = (QmiWdsAuthentication)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Authentication' TLV", offset);
            }

            self->arg_authentication_set = TRUE;

qmi_message_wds_get_default_settings_output_authentication_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV4_ADDRESS_PREFERENCE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_ipv4_address_preference_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_ipv4_address_preference), NULL))
                goto qmi_message_wds_get_default_settings_output_ipv4_address_preference_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv4 Address Preference' TLV", offset);
            }

            self->arg_ipv4_address_preference_set = TRUE;

qmi_message_wds_get_default_settings_output_ipv4_address_preference_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_PCO, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_pcscf_address_using_pco_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_default_settings_output_pcscf_address_using_pco_out;
                self->arg_pcscf_address_using_pco = (gboolean)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'PCSCF Address Using PCO' TLV", offset);
            }

            self->arg_pcscf_address_using_pco_set = TRUE;

qmi_message_wds_get_default_settings_output_pcscf_address_using_pco_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_DHCP, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_pcscf_address_using_dhcp_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_default_settings_output_pcscf_address_using_dhcp_out;
                self->arg_pcscf_address_using_dhcp = (gboolean)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'PCSCF Address Using DHCP' TLV", offset);
            }

            self->arg_pcscf_address_using_dhcp_set = TRUE;

qmi_message_wds_get_default_settings_output_pcscf_address_using_dhcp_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IMCN_FLAG, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_imcn_flag_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_default_settings_output_imcn_flag_out;
                self->arg_imcn_flag = (gboolean)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IMCN Flag' TLV", offset);
            }

            self->arg_imcn_flag_set = TRUE;

qmi_message_wds_get_default_settings_output_imcn_flag_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV6_PRIMARY_DNS_ADDRESS_PREFERENCE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_ipv6_primary_dns_address_preference_out;
            }
            {
                guint ipv6_primary_dns_address_preference_i;
                guint16 ipv6_primary_dns_address_preference_n_items = 8;


                self->arg_ipv6_primary_dns_address_preference = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (guint16),
                    (guint)ipv6_primary_dns_address_preference_n_items);

                for (ipv6_primary_dns_address_preference_i = 0; ipv6_primary_dns_address_preference_i < ipv6_primary_dns_address_preference_n_items; ipv6_primary_dns_address_preference_i++) {
                    guint16 ipv6_primary_dns_address_preference_aux;

                    if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &(ipv6_primary_dns_address_preference_aux), NULL))
                        goto qmi_message_wds_get_default_settings_output_ipv6_primary_dns_address_preference_out;
                    g_array_insert_val (self->arg_ipv6_primary_dns_address_preference, ipv6_primary_dns_address_preference_i, ipv6_primary_dns_address_preference_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv6 Primary DNS Address Preference' TLV", offset);
            }

            self->arg_ipv6_primary_dns_address_preference_set = TRUE;

qmi_message_wds_get_default_settings_output_ipv6_primary_dns_address_preference_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_IPV6_SECONDARY_DNS_ADDRESS_PREFERENCE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_ipv6_secondary_dns_address_preference_out;
            }
            {
                guint ipv6_secondary_dns_address_preference_i;
                guint16 ipv6_secondary_dns_address_preference_n_items = 8;


                self->arg_ipv6_secondary_dns_address_preference = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (guint16),
                    (guint)ipv6_secondary_dns_address_preference_n_items);

                for (ipv6_secondary_dns_address_preference_i = 0; ipv6_secondary_dns_address_preference_i < ipv6_secondary_dns_address_preference_n_items; ipv6_secondary_dns_address_preference_i++) {
                    guint16 ipv6_secondary_dns_address_preference_aux;

                    if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &(ipv6_secondary_dns_address_preference_aux), NULL))
                        goto qmi_message_wds_get_default_settings_output_ipv6_secondary_dns_address_preference_out;
                    g_array_insert_val (self->arg_ipv6_secondary_dns_address_preference, ipv6_secondary_dns_address_preference_i, ipv6_secondary_dns_address_preference_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv6 Secondary DNS Address Preference' TLV", offset);
            }

            self->arg_ipv6_secondary_dns_address_preference_set = TRUE;

qmi_message_wds_get_default_settings_output_ipv6_secondary_dns_address_preference_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status != QMI_STATUS_SUCCESS))
            break;
        /* Prerequisite.... */
        if (!(self->arg_result.error_code == QMI_PROTOCOL_ERROR_EXTENDED_INTERNAL))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS_OUTPUT_TLV_EXTENDED_ERROR_CODE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_default_settings_output_extended_error_code_out;
            }
            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, NULL))
                    goto qmi_message_wds_get_default_settings_output_extended_error_code_out;
                self->arg_extended_error_code = (QmiWdsDsProfileError)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Extended Error Code' TLV", offset);
            }

            self->arg_extended_error_code_set = TRUE;

qmi_message_wds_get_default_settings_output_extended_error_code_out:
            ;

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Get Current Settings */


/* --- Input -- */

struct _QmiMessageWdsGetCurrentSettingsInput {
    volatile gint ref_count;

    /* Requested Settings */
    gboolean arg_requested_settings_set;
    guint32 arg_requested_settings;
};

#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_INPUT_TLV_REQUESTED_SETTINGS 0x10

/**
 * qmi_message_wds_get_current_settings_input_get_requested_settings:
 * @self: a #QmiMessageWdsGetCurrentSettingsInput.
 * @value_requested_settings: a placeholder for the output #QmiWdsGetCurrentSettingsRequestedSettings, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Requested Settings' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_input_get_requested_settings (
    QmiMessageWdsGetCurrentSettingsInput *self,
    QmiWdsGetCurrentSettingsRequestedSettings *value_requested_settings,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_requested_settings_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Requested Settings' was not found in the message");
        return FALSE;
    }

    if (value_requested_settings)
        *value_requested_settings = (QmiWdsGetCurrentSettingsRequestedSettings)(self->arg_requested_settings);

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_input_set_requested_settings:
 * @self: a #QmiMessageWdsGetCurrentSettingsInput.
 * @value_requested_settings: a #QmiWdsGetCurrentSettingsRequestedSettings.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Requested Settings' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_input_set_requested_settings (
    QmiMessageWdsGetCurrentSettingsInput *self,
    QmiWdsGetCurrentSettingsRequestedSettings value_requested_settings,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_requested_settings = (guint32)(value_requested_settings);
    self->arg_requested_settings_set = TRUE;

    return TRUE;
}

GType
qmi_message_wds_get_current_settings_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetCurrentSettingsInput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_current_settings_input_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_current_settings_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_current_settings_input_ref:
 * @self: a #QmiMessageWdsGetCurrentSettingsInput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetCurrentSettingsInput *
qmi_message_wds_get_current_settings_input_ref (QmiMessageWdsGetCurrentSettingsInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_current_settings_input_unref:
 * @self: a #QmiMessageWdsGetCurrentSettingsInput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_current_settings_input_unref (QmiMessageWdsGetCurrentSettingsInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsGetCurrentSettingsInput, self);
    }
}

/**
 * qmi_message_wds_get_current_settings_input_new:
 *
 * Allocates a new #QmiMessageWdsGetCurrentSettingsInput.
 *
 * Returns: the newly created #QmiMessageWdsGetCurrentSettingsInput. The returned value should be freed with qmi_message_wds_get_current_settings_input_unref().
 */
QmiMessageWdsGetCurrentSettingsInput *
qmi_message_wds_get_current_settings_input_new (void)
{
    QmiMessageWdsGetCurrentSettingsInput *self;

    self = g_slice_new0 (QmiMessageWdsGetCurrentSettingsInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_wds_get_current_settings_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageWdsGetCurrentSettingsInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS);

    /* All TLVs are optional, we allow NULL input */
    if (!input)
        return self;

    /* Try to add the 'Requested Settings' TLV */
    if (input->arg_requested_settings_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_INPUT_TLV_REQUESTED_SETTINGS, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Requested Settings': ");
            goto error_out;
        }

        {
            guint32 tmp;

            tmp = (guint32) input->arg_requested_settings;
            /* Write the guint32 variable to the buffer */
            if (!qmi_message_tlv_write_guint32 (self, QMI_ENDIAN_LITTLE, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Requested Settings': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Requested Settings': ");
            goto error_out;
        }
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

static void
pcscf_domain_name_list_gchar_pointer_clear (gchar * *p)
{
    g_free ((*p));
}

static void
domain_name_list_gchar_pointer_clear (gchar * *p)
{
    g_free ((*p));
}

struct _QmiMessageWdsGetCurrentSettingsOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;

    /* Profile Name */
    gboolean arg_profile_name_set;
    gchar *arg_profile_name;

    /* PDP Type */
    gboolean arg_pdp_type_set;
    guint8 arg_pdp_type;

    /* APN Name */
    gboolean arg_apn_name_set;
    gchar *arg_apn_name;

    /* Primary IPv4 DNS Address */
    gboolean arg_primary_ipv4_dns_address_set;
    guint32 arg_primary_ipv4_dns_address;

    /* Secondary IPv4 DNS Address */
    gboolean arg_secondary_ipv4_dns_address_set;
    guint32 arg_secondary_ipv4_dns_address;

    /* UMTS Granted QoS */
    gboolean arg_umts_granted_qos_set;
    guint8 arg_umts_granted_qos_traffic_class;
    guint32 arg_umts_granted_qos_max_uplink_bitrate;
    guint32 arg_umts_granted_qos_max_downlink_bitrate;
    guint32 arg_umts_granted_qos_guaranteed_uplink_bitrate;
    guint32 arg_umts_granted_qos_guaranteed_downlink_bitrate;
    guint8 arg_umts_granted_qos_qos_delivery_order;
    guint32 arg_umts_granted_qos_maximum_sdu_size;
    guint8 arg_umts_granted_qos_sdu_error_ratio;
    guint8 arg_umts_granted_qos_residual_bit_error_ratio;
    guint8 arg_umts_granted_qos_delivery_erroneous_sdu;
    guint32 arg_umts_granted_qos_transfer_delay;
    guint32 arg_umts_granted_qos_traffic_handling_priority;

    /* GPRS Granted QoS */
    gboolean arg_gprs_granted_qos_set;
    guint32 arg_gprs_granted_qos_precedence_class;
    guint32 arg_gprs_granted_qos_delay_class;
    guint32 arg_gprs_granted_qos_reliability_class;
    guint32 arg_gprs_granted_qos_peak_throughput_class;
    guint32 arg_gprs_granted_qos_mean_throughput_class;

    /* Username */
    gboolean arg_username_set;
    gchar *arg_username;

    /* Authentication */
    gboolean arg_authentication_set;
    guint8 arg_authentication;

    /* IPv4 Address */
    gboolean arg_ipv4_address_set;
    guint32 arg_ipv4_address;

    /* Profile ID */
    gboolean arg_profile_id_set;
    guint8 arg_profile_id_profile_type;
    guint8 arg_profile_id_profile_index;

    /* IPv4 Gateway Address */
    gboolean arg_ipv4_gateway_address_set;
    guint32 arg_ipv4_gateway_address;

    /* IPv4 Gateway Subnet Mask */
    gboolean arg_ipv4_gateway_subnet_mask_set;
    guint32 arg_ipv4_gateway_subnet_mask;

    /* PCSCF Address Using PCO */
    gboolean arg_pcscf_address_using_pco_set;
    guint8 arg_pcscf_address_using_pco;

    /* PCSCF Server Address List */
    gboolean arg_pcscf_server_address_list_set;
    GArray *arg_pcscf_server_address_list;

    /* PCSCF Domain Name List */
    gboolean arg_pcscf_domain_name_list_set;
    GArray *arg_pcscf_domain_name_list;

    /* IPv6 Address */
    gboolean arg_ipv6_address_set;
    GArray *arg_ipv6_address_address;
    guint8 arg_ipv6_address_prefix_length;

    /* IPv6 Gateway Address */
    gboolean arg_ipv6_gateway_address_set;
    GArray *arg_ipv6_gateway_address_address;
    guint8 arg_ipv6_gateway_address_prefix_length;

    /* IPv6 Primary DNS Address */
    gboolean arg_ipv6_primary_dns_address_set;
    GArray *arg_ipv6_primary_dns_address;

    /* IPv6 Secondary DNS Address */
    gboolean arg_ipv6_secondary_dns_address_set;
    GArray *arg_ipv6_secondary_dns_address;

    /* MTU */
    gboolean arg_mtu_set;
    guint32 arg_mtu;

    /* Domain Name List */
    gboolean arg_domain_name_list_set;
    GArray *arg_domain_name_list;

    /* IP Family */
    gboolean arg_ip_family_set;
    guint8 arg_ip_family;

    /* IMCN Flag */
    gboolean arg_imcn_flag_set;
    gint8 arg_imcn_flag;

    /* Extended Technology Preference */
    gboolean arg_extended_technology_preference_set;
    guint16 arg_extended_technology_preference;
};

#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_RESULT 0x02
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PROFILE_NAME 0x10
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PDP_TYPE 0x11
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_APN_NAME 0x14
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PRIMARY_IPV4_DNS_ADDRESS 0x15
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_SECONDARY_IPV4_DNS_ADDRESS 0x16
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_UMTS_GRANTED_QOS 0x17
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_GPRS_GRANTED_QOS 0x19
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_USERNAME 0x1B
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_AUTHENTICATION 0x1D
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV4_ADDRESS 0x1E
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PROFILE_ID 0x1F
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV4_GATEWAY_ADDRESS 0x20
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV4_GATEWAY_SUBNET_MASK 0x21
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_PCO 0x22
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PCSCF_SERVER_ADDRESS_LIST 0x23
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PCSCF_DOMAIN_NAME_LIST 0x24
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_ADDRESS 0x25
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_GATEWAY_ADDRESS 0x26
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_PRIMARY_DNS_ADDRESS 0x27
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_SECONDARY_DNS_ADDRESS 0x28
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_MTU 0x29
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_DOMAIN_NAME_LIST 0x2A
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IP_FAMILY 0x2B
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IMCN_FLAG 0x2C
#define QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_EXTENDED_TECHNOLOGY_PREFERENCE 0x2D

/**
 * qmi_message_wds_get_current_settings_output_get_result:
 * @self: a QmiMessageWdsGetCurrentSettingsOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_result (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_profile_name:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_profile_name: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Profile Name' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_profile_name (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    const gchar **value_profile_name,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_profile_name_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Profile Name' was not found in the message");
        return FALSE;
    }

    if (value_profile_name)
        *value_profile_name = self->arg_profile_name;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_pdp_type:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_pdp_type: a placeholder for the output #QmiWdsPdpType, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'PDP Type' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_pdp_type (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    QmiWdsPdpType *value_pdp_type,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_pdp_type_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'PDP Type' was not found in the message");
        return FALSE;
    }

    if (value_pdp_type)
        *value_pdp_type = (QmiWdsPdpType)(self->arg_pdp_type);

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_apn_name:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_apn_name: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'APN Name' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_apn_name (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    const gchar **value_apn_name,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_apn_name_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'APN Name' was not found in the message");
        return FALSE;
    }

    if (value_apn_name)
        *value_apn_name = self->arg_apn_name;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_primary_ipv4_dns_address:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_primary_ipv4_dns_address: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Primary IPv4 DNS Address' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_primary_ipv4_dns_address (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    guint32 *value_primary_ipv4_dns_address,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_primary_ipv4_dns_address_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Primary IPv4 DNS Address' was not found in the message");
        return FALSE;
    }

    if (value_primary_ipv4_dns_address)
        *value_primary_ipv4_dns_address = self->arg_primary_ipv4_dns_address;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_secondary_ipv4_dns_address:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_secondary_ipv4_dns_address: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Secondary IPv4 DNS Address' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_secondary_ipv4_dns_address (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    guint32 *value_secondary_ipv4_dns_address,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_secondary_ipv4_dns_address_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Secondary IPv4 DNS Address' was not found in the message");
        return FALSE;
    }

    if (value_secondary_ipv4_dns_address)
        *value_secondary_ipv4_dns_address = self->arg_secondary_ipv4_dns_address;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_umts_granted_qos:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_umts_granted_qos_traffic_class: a placeholder for the output #QmiWdsTrafficClass, or %NULL if not required.
 * @value_umts_granted_qos_max_uplink_bitrate: a placeholder for the output #guint32, or %NULL if not required.
 * @value_umts_granted_qos_max_downlink_bitrate: a placeholder for the output #guint32, or %NULL if not required.
 * @value_umts_granted_qos_guaranteed_uplink_bitrate: a placeholder for the output #guint32, or %NULL if not required.
 * @value_umts_granted_qos_guaranteed_downlink_bitrate: a placeholder for the output #guint32, or %NULL if not required.
 * @value_umts_granted_qos_qos_delivery_order: a placeholder for the output #QmiWdsDeliveryOrder, or %NULL if not required.
 * @value_umts_granted_qos_maximum_sdu_size: a placeholder for the output #guint32, or %NULL if not required.
 * @value_umts_granted_qos_sdu_error_ratio: a placeholder for the output #QmiWdsSduErrorRatio, or %NULL if not required.
 * @value_umts_granted_qos_residual_bit_error_ratio: a placeholder for the output #QmiWdsSduResidualBitErrorRatio, or %NULL if not required.
 * @value_umts_granted_qos_delivery_erroneous_sdu: a placeholder for the output #QmiWdsSduErroneousDelivery, or %NULL if not required.
 * @value_umts_granted_qos_transfer_delay: a placeholder for the output #guint32, or %NULL if not required.
 * @value_umts_granted_qos_traffic_handling_priority: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'UMTS Granted QoS' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_umts_granted_qos (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    QmiWdsTrafficClass *value_umts_granted_qos_traffic_class,
    guint32 *value_umts_granted_qos_max_uplink_bitrate,
    guint32 *value_umts_granted_qos_max_downlink_bitrate,
    guint32 *value_umts_granted_qos_guaranteed_uplink_bitrate,
    guint32 *value_umts_granted_qos_guaranteed_downlink_bitrate,
    QmiWdsDeliveryOrder *value_umts_granted_qos_qos_delivery_order,
    guint32 *value_umts_granted_qos_maximum_sdu_size,
    QmiWdsSduErrorRatio *value_umts_granted_qos_sdu_error_ratio,
    QmiWdsSduResidualBitErrorRatio *value_umts_granted_qos_residual_bit_error_ratio,
    QmiWdsSduErroneousDelivery *value_umts_granted_qos_delivery_erroneous_sdu,
    guint32 *value_umts_granted_qos_transfer_delay,
    guint32 *value_umts_granted_qos_traffic_handling_priority,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_umts_granted_qos_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'UMTS Granted QoS' was not found in the message");
        return FALSE;
    }

    if (value_umts_granted_qos_traffic_class)
        *value_umts_granted_qos_traffic_class = (QmiWdsTrafficClass)(self->arg_umts_granted_qos_traffic_class);
    if (value_umts_granted_qos_max_uplink_bitrate)
        *value_umts_granted_qos_max_uplink_bitrate = self->arg_umts_granted_qos_max_uplink_bitrate;
    if (value_umts_granted_qos_max_downlink_bitrate)
        *value_umts_granted_qos_max_downlink_bitrate = self->arg_umts_granted_qos_max_downlink_bitrate;
    if (value_umts_granted_qos_guaranteed_uplink_bitrate)
        *value_umts_granted_qos_guaranteed_uplink_bitrate = self->arg_umts_granted_qos_guaranteed_uplink_bitrate;
    if (value_umts_granted_qos_guaranteed_downlink_bitrate)
        *value_umts_granted_qos_guaranteed_downlink_bitrate = self->arg_umts_granted_qos_guaranteed_downlink_bitrate;
    if (value_umts_granted_qos_qos_delivery_order)
        *value_umts_granted_qos_qos_delivery_order = (QmiWdsDeliveryOrder)(self->arg_umts_granted_qos_qos_delivery_order);
    if (value_umts_granted_qos_maximum_sdu_size)
        *value_umts_granted_qos_maximum_sdu_size = self->arg_umts_granted_qos_maximum_sdu_size;
    if (value_umts_granted_qos_sdu_error_ratio)
        *value_umts_granted_qos_sdu_error_ratio = (QmiWdsSduErrorRatio)(self->arg_umts_granted_qos_sdu_error_ratio);
    if (value_umts_granted_qos_residual_bit_error_ratio)
        *value_umts_granted_qos_residual_bit_error_ratio = (QmiWdsSduResidualBitErrorRatio)(self->arg_umts_granted_qos_residual_bit_error_ratio);
    if (value_umts_granted_qos_delivery_erroneous_sdu)
        *value_umts_granted_qos_delivery_erroneous_sdu = (QmiWdsSduErroneousDelivery)(self->arg_umts_granted_qos_delivery_erroneous_sdu);
    if (value_umts_granted_qos_transfer_delay)
        *value_umts_granted_qos_transfer_delay = self->arg_umts_granted_qos_transfer_delay;
    if (value_umts_granted_qos_traffic_handling_priority)
        *value_umts_granted_qos_traffic_handling_priority = self->arg_umts_granted_qos_traffic_handling_priority;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_gprs_granted_qos:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_gprs_granted_qos_precedence_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_granted_qos_delay_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_granted_qos_reliability_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_granted_qos_peak_throughput_class: a placeholder for the output #guint32, or %NULL if not required.
 * @value_gprs_granted_qos_mean_throughput_class: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'GPRS Granted QoS' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_gprs_granted_qos (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    guint32 *value_gprs_granted_qos_precedence_class,
    guint32 *value_gprs_granted_qos_delay_class,
    guint32 *value_gprs_granted_qos_reliability_class,
    guint32 *value_gprs_granted_qos_peak_throughput_class,
    guint32 *value_gprs_granted_qos_mean_throughput_class,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_gprs_granted_qos_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'GPRS Granted QoS' was not found in the message");
        return FALSE;
    }

    if (value_gprs_granted_qos_precedence_class)
        *value_gprs_granted_qos_precedence_class = self->arg_gprs_granted_qos_precedence_class;
    if (value_gprs_granted_qos_delay_class)
        *value_gprs_granted_qos_delay_class = self->arg_gprs_granted_qos_delay_class;
    if (value_gprs_granted_qos_reliability_class)
        *value_gprs_granted_qos_reliability_class = self->arg_gprs_granted_qos_reliability_class;
    if (value_gprs_granted_qos_peak_throughput_class)
        *value_gprs_granted_qos_peak_throughput_class = self->arg_gprs_granted_qos_peak_throughput_class;
    if (value_gprs_granted_qos_mean_throughput_class)
        *value_gprs_granted_qos_mean_throughput_class = self->arg_gprs_granted_qos_mean_throughput_class;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_username:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_username: a placeholder for the output constant string, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Username' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_username (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    const gchar **value_username,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_username_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Username' was not found in the message");
        return FALSE;
    }

    if (value_username)
        *value_username = self->arg_username;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_authentication:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_authentication: a placeholder for the output #QmiWdsAuthentication, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Authentication' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_authentication (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    QmiWdsAuthentication *value_authentication,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_authentication_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Authentication' was not found in the message");
        return FALSE;
    }

    if (value_authentication)
        *value_authentication = (QmiWdsAuthentication)(self->arg_authentication);

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_ipv4_address:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_ipv4_address: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv4 Address' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_ipv4_address (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    guint32 *value_ipv4_address,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv4_address_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv4 Address' was not found in the message");
        return FALSE;
    }

    if (value_ipv4_address)
        *value_ipv4_address = self->arg_ipv4_address;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_profile_id:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_profile_id_profile_type: a placeholder for the output #QmiWdsProfileType, or %NULL if not required.
 * @value_profile_id_profile_index: a placeholder for the output #guint8, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Profile ID' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_profile_id (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    QmiWdsProfileType *value_profile_id_profile_type,
    guint8 *value_profile_id_profile_index,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_profile_id_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Profile ID' was not found in the message");
        return FALSE;
    }

    if (value_profile_id_profile_type)
        *value_profile_id_profile_type = (QmiWdsProfileType)(self->arg_profile_id_profile_type);
    if (value_profile_id_profile_index)
        *value_profile_id_profile_index = self->arg_profile_id_profile_index;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_ipv4_gateway_address:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_ipv4_gateway_address: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv4 Gateway Address' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_ipv4_gateway_address (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    guint32 *value_ipv4_gateway_address,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv4_gateway_address_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv4 Gateway Address' was not found in the message");
        return FALSE;
    }

    if (value_ipv4_gateway_address)
        *value_ipv4_gateway_address = self->arg_ipv4_gateway_address;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_ipv4_gateway_subnet_mask:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_ipv4_gateway_subnet_mask: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv4 Gateway Subnet Mask' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_ipv4_gateway_subnet_mask (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    guint32 *value_ipv4_gateway_subnet_mask,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv4_gateway_subnet_mask_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv4 Gateway Subnet Mask' was not found in the message");
        return FALSE;
    }

    if (value_ipv4_gateway_subnet_mask)
        *value_ipv4_gateway_subnet_mask = self->arg_ipv4_gateway_subnet_mask;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_pcscf_address_using_pco:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_pcscf_address_using_pco: a placeholder for the output #guint8, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'PCSCF Address Using PCO' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_pcscf_address_using_pco (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    guint8 *value_pcscf_address_using_pco,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_pcscf_address_using_pco_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'PCSCF Address Using PCO' was not found in the message");
        return FALSE;
    }

    if (value_pcscf_address_using_pco)
        *value_pcscf_address_using_pco = self->arg_pcscf_address_using_pco;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_pcscf_server_address_list:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_pcscf_server_address_list: a placeholder for the output #GArray of #guint32 elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'PCSCF Server Address List' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_pcscf_server_address_list (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    GArray **value_pcscf_server_address_list,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_pcscf_server_address_list_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'PCSCF Server Address List' was not found in the message");
        return FALSE;
    }

    if (value_pcscf_server_address_list)
        *value_pcscf_server_address_list = self->arg_pcscf_server_address_list;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_pcscf_domain_name_list:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_pcscf_domain_name_list: a placeholder for the output #GArray of #gchar * elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'PCSCF Domain Name List' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_pcscf_domain_name_list (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    GArray **value_pcscf_domain_name_list,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_pcscf_domain_name_list_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'PCSCF Domain Name List' was not found in the message");
        return FALSE;
    }

    if (value_pcscf_domain_name_list)
        *value_pcscf_domain_name_list = self->arg_pcscf_domain_name_list;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_ipv6_address:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_ipv6_address_address: a placeholder for the output #GArray of #guint16 elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @value_ipv6_address_prefix_length: a placeholder for the output #guint8, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv6 Address' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_ipv6_address (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    GArray **value_ipv6_address_address,
    guint8 *value_ipv6_address_prefix_length,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv6_address_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv6 Address' was not found in the message");
        return FALSE;
    }

    if (value_ipv6_address_address)
        *value_ipv6_address_address = self->arg_ipv6_address_address;
    if (value_ipv6_address_prefix_length)
        *value_ipv6_address_prefix_length = self->arg_ipv6_address_prefix_length;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_ipv6_gateway_address:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_ipv6_gateway_address_address: a placeholder for the output #GArray of #guint16 elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @value_ipv6_gateway_address_prefix_length: a placeholder for the output #guint8, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv6 Gateway Address' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_ipv6_gateway_address (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    GArray **value_ipv6_gateway_address_address,
    guint8 *value_ipv6_gateway_address_prefix_length,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv6_gateway_address_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv6 Gateway Address' was not found in the message");
        return FALSE;
    }

    if (value_ipv6_gateway_address_address)
        *value_ipv6_gateway_address_address = self->arg_ipv6_gateway_address_address;
    if (value_ipv6_gateway_address_prefix_length)
        *value_ipv6_gateway_address_prefix_length = self->arg_ipv6_gateway_address_prefix_length;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_ipv6_primary_dns_address:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_ipv6_primary_dns_address: a placeholder for the output #GArray of #guint16 elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv6 Primary DNS Address' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_ipv6_primary_dns_address (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    GArray **value_ipv6_primary_dns_address,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv6_primary_dns_address_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv6 Primary DNS Address' was not found in the message");
        return FALSE;
    }

    if (value_ipv6_primary_dns_address)
        *value_ipv6_primary_dns_address = self->arg_ipv6_primary_dns_address;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_ipv6_secondary_dns_address:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_ipv6_secondary_dns_address: a placeholder for the output #GArray of #guint16 elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IPv6 Secondary DNS Address' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_ipv6_secondary_dns_address (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    GArray **value_ipv6_secondary_dns_address,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ipv6_secondary_dns_address_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IPv6 Secondary DNS Address' was not found in the message");
        return FALSE;
    }

    if (value_ipv6_secondary_dns_address)
        *value_ipv6_secondary_dns_address = self->arg_ipv6_secondary_dns_address;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_mtu:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_mtu: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'MTU' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_mtu (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    guint32 *value_mtu,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_mtu_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'MTU' was not found in the message");
        return FALSE;
    }

    if (value_mtu)
        *value_mtu = self->arg_mtu;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_domain_name_list:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_domain_name_list: a placeholder for the output #GArray of #gchar * elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Domain Name List' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_domain_name_list (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    GArray **value_domain_name_list,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_domain_name_list_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Domain Name List' was not found in the message");
        return FALSE;
    }

    if (value_domain_name_list)
        *value_domain_name_list = self->arg_domain_name_list;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_ip_family:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_ip_family: a placeholder for the output #QmiWdsIpFamily, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IP Family' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_ip_family (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    QmiWdsIpFamily *value_ip_family,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_ip_family_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IP Family' was not found in the message");
        return FALSE;
    }

    if (value_ip_family)
        *value_ip_family = (QmiWdsIpFamily)(self->arg_ip_family);

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_imcn_flag:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_imcn_flag: a placeholder for the output #gint8, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'IMCN Flag' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_imcn_flag (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    gint8 *value_imcn_flag,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_imcn_flag_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'IMCN Flag' was not found in the message");
        return FALSE;
    }

    if (value_imcn_flag)
        *value_imcn_flag = self->arg_imcn_flag;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_settings_output_get_extended_technology_preference:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 * @value_extended_technology_preference: a placeholder for the output #QmiWdsExtendedTechnologyPreference, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Extended Technology Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_settings_output_get_extended_technology_preference (
    QmiMessageWdsGetCurrentSettingsOutput *self,
    QmiWdsExtendedTechnologyPreference *value_extended_technology_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_extended_technology_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Extended Technology Preference' was not found in the message");
        return FALSE;
    }

    if (value_extended_technology_preference)
        *value_extended_technology_preference = (QmiWdsExtendedTechnologyPreference)(self->arg_extended_technology_preference);

    return TRUE;
}

GType
qmi_message_wds_get_current_settings_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetCurrentSettingsOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_current_settings_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_current_settings_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_current_settings_output_ref:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetCurrentSettingsOutput *
qmi_message_wds_get_current_settings_output_ref (QmiMessageWdsGetCurrentSettingsOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_current_settings_output_unref:
 * @self: a #QmiMessageWdsGetCurrentSettingsOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_current_settings_output_unref (QmiMessageWdsGetCurrentSettingsOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_free (self->arg_profile_name);
        g_free (self->arg_apn_name);
        g_free (self->arg_username);
        if (self->arg_pcscf_server_address_list)
            g_array_unref (self->arg_pcscf_server_address_list);
        if (self->arg_pcscf_domain_name_list)
            g_array_unref (self->arg_pcscf_domain_name_list);
        if (self->arg_ipv6_address_address)
            g_array_unref (self->arg_ipv6_address_address);
        if (self->arg_ipv6_gateway_address_address)
            g_array_unref (self->arg_ipv6_gateway_address_address);
        if (self->arg_ipv6_primary_dns_address)
            g_array_unref (self->arg_ipv6_primary_dns_address);
        if (self->arg_ipv6_secondary_dns_address)
            g_array_unref (self->arg_ipv6_secondary_dns_address);
        if (self->arg_domain_name_list)
            g_array_unref (self->arg_domain_name_list);
        g_slice_free (QmiMessageWdsGetCurrentSettingsOutput, self);
    }
}

static gchar *
qmi_message_wds_get_current_settings_input_requested_settings_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_INPUT_TLV_REQUESTED_SETTINGS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_GET_CURRENT_SETTINGS_REQUESTED_SETTINGS_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_get_current_settings_requested_settings_get_string ((QmiWdsGetCurrentSettingsRequestedSettings)tmp));
#elif defined  __QMI_WDS_GET_CURRENT_SETTINGS_REQUESTED_SETTINGS_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_get_current_settings_requested_settings_build_string_from_mask ((QmiWdsGetCurrentSettingsRequestedSettings)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsGetCurrentSettingsRequestedSettings
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_profile_name_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PROFILE_NAME, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_pdp_type_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PDP_TYPE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_PDP_TYPE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_pdp_type_get_string ((QmiWdsPdpType)tmp));
#elif defined  __QMI_WDS_PDP_TYPE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_pdp_type_build_string_from_mask ((QmiWdsPdpType)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsPdpType
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_apn_name_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_APN_NAME, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_primary_ipv4_dns_address_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PRIMARY_IPV4_DNS_ADDRESS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_secondary_ipv4_dns_address_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_SECONDARY_IPV4_DNS_ADDRESS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_umts_granted_qos_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_UMTS_GRANTED_QOS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " traffic_class = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_TRAFFIC_CLASS_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_traffic_class_get_string ((QmiWdsTrafficClass)tmp));
#elif defined  __QMI_WDS_TRAFFIC_CLASS_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_traffic_class_build_string_from_mask ((QmiWdsTrafficClass)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsTrafficClass
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " max_uplink_bitrate = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " max_downlink_bitrate = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " guaranteed_uplink_bitrate = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " guaranteed_downlink_bitrate = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " qos_delivery_order = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_DELIVERY_ORDER_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_delivery_order_get_string ((QmiWdsDeliveryOrder)tmp));
#elif defined  __QMI_WDS_DELIVERY_ORDER_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_delivery_order_build_string_from_mask ((QmiWdsDeliveryOrder)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsDeliveryOrder
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " maximum_sdu_size = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " sdu_error_ratio = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_SDU_ERROR_RATIO_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_sdu_error_ratio_get_string ((QmiWdsSduErrorRatio)tmp));
#elif defined  __QMI_WDS_SDU_ERROR_RATIO_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_sdu_error_ratio_build_string_from_mask ((QmiWdsSduErrorRatio)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsSduErrorRatio
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " residual_bit_error_ratio = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_SDU_RESIDUAL_BIT_ERROR_RATIO_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_sdu_residual_bit_error_ratio_get_string ((QmiWdsSduResidualBitErrorRatio)tmp));
#elif defined  __QMI_WDS_SDU_RESIDUAL_BIT_ERROR_RATIO_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_sdu_residual_bit_error_ratio_build_string_from_mask ((QmiWdsSduResidualBitErrorRatio)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsSduResidualBitErrorRatio
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " delivery_erroneous_sdu = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_SDU_ERRONEOUS_DELIVERY_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_sdu_erroneous_delivery_get_string ((QmiWdsSduErroneousDelivery)tmp));
#elif defined  __QMI_WDS_SDU_ERRONEOUS_DELIVERY_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_sdu_erroneous_delivery_build_string_from_mask ((QmiWdsSduErroneousDelivery)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsSduErroneousDelivery
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " transfer_delay = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " traffic_handling_priority = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_gprs_granted_qos_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_GPRS_GRANTED_QOS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " precedence_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " delay_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " reliability_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " peak_throughput_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " mean_throughput_class = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_username_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_USERNAME, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gchar *tmp;

        if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &tmp, &error))
            goto out;
        g_string_append (printable, tmp);
        g_free (tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_authentication_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_AUTHENTICATION, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_AUTHENTICATION_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_authentication_get_string ((QmiWdsAuthentication)tmp));
#elif defined  __QMI_WDS_AUTHENTICATION_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_authentication_build_string_from_mask ((QmiWdsAuthentication)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsAuthentication
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_ipv4_address_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV4_ADDRESS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_profile_id_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PROFILE_ID, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " profile_type = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_PROFILE_TYPE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_profile_type_get_string ((QmiWdsProfileType)tmp));
#elif defined  __QMI_WDS_PROFILE_TYPE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_profile_type_build_string_from_mask ((QmiWdsProfileType)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsProfileType
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " profile_index = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_ipv4_gateway_address_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV4_GATEWAY_ADDRESS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_ipv4_gateway_subnet_mask_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV4_GATEWAY_SUBNET_MASK, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_pcscf_address_using_pco_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_PCO, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_pcscf_server_address_list_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PCSCF_SERVER_ADDRESS_LIST, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint pcscf_server_address_list_i;
        guint8 pcscf_server_address_list_n_items;

        /* Read number of items in the array */
        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(pcscf_server_address_list_n_items), &error))
            goto out;

        g_string_append (printable, "{");

        for (pcscf_server_address_list_i = 0; pcscf_server_address_list_i < pcscf_server_address_list_n_items; pcscf_server_address_list_i++) {
            g_string_append_printf (printable, " [%u] = '", pcscf_server_address_list_i);

            {
                guint32 tmp;

                if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_pcscf_domain_name_list_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PCSCF_DOMAIN_NAME_LIST, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint pcscf_domain_name_list_i;
        guint8 pcscf_domain_name_list_n_items;

        /* Read number of items in the array */
        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(pcscf_domain_name_list_n_items), &error))
            goto out;

        g_string_append (printable, "{");

        for (pcscf_domain_name_list_i = 0; pcscf_domain_name_list_i < pcscf_domain_name_list_n_items; pcscf_domain_name_list_i++) {
            g_string_append_printf (printable, " [%u] = '", pcscf_domain_name_list_i);

            {
                gchar *tmp;

                if (!qmi_message_tlv_read_string (message, init_offset, &offset, 2, 0, &tmp, &error))
                    goto out;
                g_string_append (printable, tmp);
                g_free (tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_ipv6_address_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_ADDRESS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " address = '");
    {
        guint address_i;
        guint16 address_n_items = 8;


        g_string_append (printable, "{");

        for (address_i = 0; address_i < address_n_items; address_i++) {
            g_string_append_printf (printable, " [%u] = '", address_i);

            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%" G_GUINT16_FORMAT "", tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }    g_string_append (printable, "'");
    g_string_append (printable, " prefix_length = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_ipv6_gateway_address_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_GATEWAY_ADDRESS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " address = '");
    {
        guint address_i;
        guint16 address_n_items = 8;


        g_string_append (printable, "{");

        for (address_i = 0; address_i < address_n_items; address_i++) {
            g_string_append_printf (printable, " [%u] = '", address_i);

            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%" G_GUINT16_FORMAT "", tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }    g_string_append (printable, "'");
    g_string_append (printable, " prefix_length = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%u", (guint)tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_ipv6_primary_dns_address_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_PRIMARY_DNS_ADDRESS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint ipv6_primary_dns_address_i;
        guint16 ipv6_primary_dns_address_n_items = 8;


        g_string_append (printable, "{");

        for (ipv6_primary_dns_address_i = 0; ipv6_primary_dns_address_i < ipv6_primary_dns_address_n_items; ipv6_primary_dns_address_i++) {
            g_string_append_printf (printable, " [%u] = '", ipv6_primary_dns_address_i);

            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%" G_GUINT16_FORMAT "", tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_ipv6_secondary_dns_address_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_SECONDARY_DNS_ADDRESS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint ipv6_secondary_dns_address_i;
        guint16 ipv6_secondary_dns_address_n_items = 8;


        g_string_append (printable, "{");

        for (ipv6_secondary_dns_address_i = 0; ipv6_secondary_dns_address_i < ipv6_secondary_dns_address_n_items; ipv6_secondary_dns_address_i++) {
            g_string_append_printf (printable, " [%u] = '", ipv6_secondary_dns_address_i);

            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%" G_GUINT16_FORMAT "", tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_mtu_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_MTU, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_domain_name_list_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_DOMAIN_NAME_LIST, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint domain_name_list_i;
        guint8 domain_name_list_n_items;

        /* Read number of items in the array */
        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(domain_name_list_n_items), &error))
            goto out;

        g_string_append (printable, "{");

        for (domain_name_list_i = 0; domain_name_list_i < domain_name_list_n_items; domain_name_list_i++) {
            g_string_append_printf (printable, " [%u] = '", domain_name_list_i);

            {
                gchar *tmp;

                if (!qmi_message_tlv_read_string (message, init_offset, &offset, 2, 0, &tmp, &error))
                    goto out;
                g_string_append (printable, tmp);
                g_free (tmp);
            }
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_ip_family_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IP_FAMILY, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_IP_FAMILY_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_ip_family_get_string ((QmiWdsIpFamily)tmp));
#elif defined  __QMI_WDS_IP_FAMILY_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_ip_family_build_string_from_mask ((QmiWdsIpFamily)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsIpFamily
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_imcn_flag_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IMCN_FLAG, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gint8 tmp;

        if (!qmi_message_tlv_read_gint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%d", (gint)tmp);
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_settings_output_extended_technology_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_EXTENDED_TECHNOLOGY_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint16 tmp;

        if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_EXTENDED_TECHNOLOGY_PREFERENCE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_extended_technology_preference_get_string ((QmiWdsExtendedTechnologyPreference)tmp));
#elif defined  __QMI_WDS_EXTENDED_TECHNOLOGY_PREFERENCE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_extended_technology_preference_build_string_from_mask ((QmiWdsExtendedTechnologyPreference)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsExtendedTechnologyPreference
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_get_current_settings_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_get_current_settings_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_get_current_settings_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_INPUT_TLV_REQUESTED_SETTINGS:
            tlv_type_str = "Requested Settings";
            translated_value = qmi_message_wds_get_current_settings_input_requested_settings_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PROFILE_NAME:
            tlv_type_str = "Profile Name";
            translated_value = qmi_message_wds_get_current_settings_output_profile_name_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PDP_TYPE:
            tlv_type_str = "PDP Type";
            translated_value = qmi_message_wds_get_current_settings_output_pdp_type_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_APN_NAME:
            tlv_type_str = "APN Name";
            translated_value = qmi_message_wds_get_current_settings_output_apn_name_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PRIMARY_IPV4_DNS_ADDRESS:
            tlv_type_str = "Primary IPv4 DNS Address";
            translated_value = qmi_message_wds_get_current_settings_output_primary_ipv4_dns_address_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_SECONDARY_IPV4_DNS_ADDRESS:
            tlv_type_str = "Secondary IPv4 DNS Address";
            translated_value = qmi_message_wds_get_current_settings_output_secondary_ipv4_dns_address_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_UMTS_GRANTED_QOS:
            tlv_type_str = "UMTS Granted QoS";
            translated_value = qmi_message_wds_get_current_settings_output_umts_granted_qos_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_GPRS_GRANTED_QOS:
            tlv_type_str = "GPRS Granted QoS";
            translated_value = qmi_message_wds_get_current_settings_output_gprs_granted_qos_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_USERNAME:
            tlv_type_str = "Username";
            translated_value = qmi_message_wds_get_current_settings_output_username_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_AUTHENTICATION:
            tlv_type_str = "Authentication";
            translated_value = qmi_message_wds_get_current_settings_output_authentication_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV4_ADDRESS:
            tlv_type_str = "IPv4 Address";
            translated_value = qmi_message_wds_get_current_settings_output_ipv4_address_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PROFILE_ID:
            tlv_type_str = "Profile ID";
            translated_value = qmi_message_wds_get_current_settings_output_profile_id_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV4_GATEWAY_ADDRESS:
            tlv_type_str = "IPv4 Gateway Address";
            translated_value = qmi_message_wds_get_current_settings_output_ipv4_gateway_address_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV4_GATEWAY_SUBNET_MASK:
            tlv_type_str = "IPv4 Gateway Subnet Mask";
            translated_value = qmi_message_wds_get_current_settings_output_ipv4_gateway_subnet_mask_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_PCO:
            tlv_type_str = "PCSCF Address Using PCO";
            translated_value = qmi_message_wds_get_current_settings_output_pcscf_address_using_pco_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PCSCF_SERVER_ADDRESS_LIST:
            tlv_type_str = "PCSCF Server Address List";
            translated_value = qmi_message_wds_get_current_settings_output_pcscf_server_address_list_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PCSCF_DOMAIN_NAME_LIST:
            tlv_type_str = "PCSCF Domain Name List";
            translated_value = qmi_message_wds_get_current_settings_output_pcscf_domain_name_list_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_ADDRESS:
            tlv_type_str = "IPv6 Address";
            translated_value = qmi_message_wds_get_current_settings_output_ipv6_address_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_GATEWAY_ADDRESS:
            tlv_type_str = "IPv6 Gateway Address";
            translated_value = qmi_message_wds_get_current_settings_output_ipv6_gateway_address_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_PRIMARY_DNS_ADDRESS:
            tlv_type_str = "IPv6 Primary DNS Address";
            translated_value = qmi_message_wds_get_current_settings_output_ipv6_primary_dns_address_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_SECONDARY_DNS_ADDRESS:
            tlv_type_str = "IPv6 Secondary DNS Address";
            translated_value = qmi_message_wds_get_current_settings_output_ipv6_secondary_dns_address_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_MTU:
            tlv_type_str = "MTU";
            translated_value = qmi_message_wds_get_current_settings_output_mtu_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_DOMAIN_NAME_LIST:
            tlv_type_str = "Domain Name List";
            translated_value = qmi_message_wds_get_current_settings_output_domain_name_list_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IP_FAMILY:
            tlv_type_str = "IP Family";
            translated_value = qmi_message_wds_get_current_settings_output_ip_family_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IMCN_FLAG:
            tlv_type_str = "IMCN Flag";
            translated_value = qmi_message_wds_get_current_settings_output_imcn_flag_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_EXTENDED_TECHNOLOGY_PREFERENCE:
            tlv_type_str = "Extended Technology Preference";
            translated_value = qmi_message_wds_get_current_settings_output_extended_technology_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_get_current_settings_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Get Current Settings\" (0x002D)\n",
                            line_prefix);

    {
        struct message_get_current_settings_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_get_current_settings_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsGetCurrentSettingsOutput *
__qmi_message_wds_get_current_settings_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsGetCurrentSettingsOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS, NULL);

    self = g_slice_new0 (QmiMessageWdsGetCurrentSettingsOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_get_current_settings_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_get_current_settings_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PROFILE_NAME, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_profile_name_out;
            }
            if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &(self->arg_profile_name), NULL))
                goto qmi_message_wds_get_current_settings_output_profile_name_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Profile Name' TLV", offset);
            }

            self->arg_profile_name_set = TRUE;

qmi_message_wds_get_current_settings_output_profile_name_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PDP_TYPE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_pdp_type_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_current_settings_output_pdp_type_out;
                self->arg_pdp_type = (QmiWdsPdpType)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'PDP Type' TLV", offset);
            }

            self->arg_pdp_type_set = TRUE;

qmi_message_wds_get_current_settings_output_pdp_type_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_APN_NAME, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_apn_name_out;
            }
            if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &(self->arg_apn_name), NULL))
                goto qmi_message_wds_get_current_settings_output_apn_name_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'APN Name' TLV", offset);
            }

            self->arg_apn_name_set = TRUE;

qmi_message_wds_get_current_settings_output_apn_name_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PRIMARY_IPV4_DNS_ADDRESS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_primary_ipv4_dns_address_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_primary_ipv4_dns_address), NULL))
                goto qmi_message_wds_get_current_settings_output_primary_ipv4_dns_address_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Primary IPv4 DNS Address' TLV", offset);
            }

            self->arg_primary_ipv4_dns_address_set = TRUE;

qmi_message_wds_get_current_settings_output_primary_ipv4_dns_address_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_SECONDARY_IPV4_DNS_ADDRESS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_secondary_ipv4_dns_address_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_secondary_ipv4_dns_address), NULL))
                goto qmi_message_wds_get_current_settings_output_secondary_ipv4_dns_address_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Secondary IPv4 DNS Address' TLV", offset);
            }

            self->arg_secondary_ipv4_dns_address_set = TRUE;

qmi_message_wds_get_current_settings_output_secondary_ipv4_dns_address_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_UMTS_GRANTED_QOS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_umts_granted_qos_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_current_settings_output_umts_granted_qos_out;
                self->arg_umts_granted_qos_traffic_class = (QmiWdsTrafficClass)tmp;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_umts_granted_qos_max_uplink_bitrate), NULL))
                goto qmi_message_wds_get_current_settings_output_umts_granted_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_umts_granted_qos_max_downlink_bitrate), NULL))
                goto qmi_message_wds_get_current_settings_output_umts_granted_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_umts_granted_qos_guaranteed_uplink_bitrate), NULL))
                goto qmi_message_wds_get_current_settings_output_umts_granted_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_umts_granted_qos_guaranteed_downlink_bitrate), NULL))
                goto qmi_message_wds_get_current_settings_output_umts_granted_qos_out;
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_current_settings_output_umts_granted_qos_out;
                self->arg_umts_granted_qos_qos_delivery_order = (QmiWdsDeliveryOrder)tmp;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_umts_granted_qos_maximum_sdu_size), NULL))
                goto qmi_message_wds_get_current_settings_output_umts_granted_qos_out;
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_current_settings_output_umts_granted_qos_out;
                self->arg_umts_granted_qos_sdu_error_ratio = (QmiWdsSduErrorRatio)tmp;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_current_settings_output_umts_granted_qos_out;
                self->arg_umts_granted_qos_residual_bit_error_ratio = (QmiWdsSduResidualBitErrorRatio)tmp;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_current_settings_output_umts_granted_qos_out;
                self->arg_umts_granted_qos_delivery_erroneous_sdu = (QmiWdsSduErroneousDelivery)tmp;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_umts_granted_qos_transfer_delay), NULL))
                goto qmi_message_wds_get_current_settings_output_umts_granted_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_umts_granted_qos_traffic_handling_priority), NULL))
                goto qmi_message_wds_get_current_settings_output_umts_granted_qos_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'UMTS Granted QoS' TLV", offset);
            }

            self->arg_umts_granted_qos_set = TRUE;

qmi_message_wds_get_current_settings_output_umts_granted_qos_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_GPRS_GRANTED_QOS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_gprs_granted_qos_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_granted_qos_precedence_class), NULL))
                goto qmi_message_wds_get_current_settings_output_gprs_granted_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_granted_qos_delay_class), NULL))
                goto qmi_message_wds_get_current_settings_output_gprs_granted_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_granted_qos_reliability_class), NULL))
                goto qmi_message_wds_get_current_settings_output_gprs_granted_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_granted_qos_peak_throughput_class), NULL))
                goto qmi_message_wds_get_current_settings_output_gprs_granted_qos_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_gprs_granted_qos_mean_throughput_class), NULL))
                goto qmi_message_wds_get_current_settings_output_gprs_granted_qos_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'GPRS Granted QoS' TLV", offset);
            }

            self->arg_gprs_granted_qos_set = TRUE;

qmi_message_wds_get_current_settings_output_gprs_granted_qos_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_USERNAME, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_username_out;
            }
            if (!qmi_message_tlv_read_string (message, init_offset, &offset, 0, 0, &(self->arg_username), NULL))
                goto qmi_message_wds_get_current_settings_output_username_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Username' TLV", offset);
            }

            self->arg_username_set = TRUE;

qmi_message_wds_get_current_settings_output_username_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_AUTHENTICATION, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_authentication_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_current_settings_output_authentication_out;
                self->arg_authentication = (QmiWdsAuthentication)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Authentication' TLV", offset);
            }

            self->arg_authentication_set = TRUE;

qmi_message_wds_get_current_settings_output_authentication_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV4_ADDRESS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_ipv4_address_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_ipv4_address), NULL))
                goto qmi_message_wds_get_current_settings_output_ipv4_address_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv4 Address' TLV", offset);
            }

            self->arg_ipv4_address_set = TRUE;

qmi_message_wds_get_current_settings_output_ipv4_address_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PROFILE_ID, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_profile_id_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_current_settings_output_profile_id_out;
                self->arg_profile_id_profile_type = (QmiWdsProfileType)tmp;
            }
            if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(self->arg_profile_id_profile_index), NULL))
                goto qmi_message_wds_get_current_settings_output_profile_id_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Profile ID' TLV", offset);
            }

            self->arg_profile_id_set = TRUE;

qmi_message_wds_get_current_settings_output_profile_id_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV4_GATEWAY_ADDRESS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_ipv4_gateway_address_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_ipv4_gateway_address), NULL))
                goto qmi_message_wds_get_current_settings_output_ipv4_gateway_address_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv4 Gateway Address' TLV", offset);
            }

            self->arg_ipv4_gateway_address_set = TRUE;

qmi_message_wds_get_current_settings_output_ipv4_gateway_address_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV4_GATEWAY_SUBNET_MASK, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_ipv4_gateway_subnet_mask_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_ipv4_gateway_subnet_mask), NULL))
                goto qmi_message_wds_get_current_settings_output_ipv4_gateway_subnet_mask_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv4 Gateway Subnet Mask' TLV", offset);
            }

            self->arg_ipv4_gateway_subnet_mask_set = TRUE;

qmi_message_wds_get_current_settings_output_ipv4_gateway_subnet_mask_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PCSCF_ADDRESS_USING_PCO, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_pcscf_address_using_pco_out;
            }
            if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(self->arg_pcscf_address_using_pco), NULL))
                goto qmi_message_wds_get_current_settings_output_pcscf_address_using_pco_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'PCSCF Address Using PCO' TLV", offset);
            }

            self->arg_pcscf_address_using_pco_set = TRUE;

qmi_message_wds_get_current_settings_output_pcscf_address_using_pco_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PCSCF_SERVER_ADDRESS_LIST, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_pcscf_server_address_list_out;
            }
            {
                guint pcscf_server_address_list_i;
                guint8 pcscf_server_address_list_n_items;

                /* Read number of items in the array */
                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(pcscf_server_address_list_n_items), NULL))
                    goto qmi_message_wds_get_current_settings_output_pcscf_server_address_list_out;

                self->arg_pcscf_server_address_list = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (guint32),
                    (guint)pcscf_server_address_list_n_items);

                for (pcscf_server_address_list_i = 0; pcscf_server_address_list_i < pcscf_server_address_list_n_items; pcscf_server_address_list_i++) {
                    guint32 pcscf_server_address_list_aux;

                    if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(pcscf_server_address_list_aux), NULL))
                        goto qmi_message_wds_get_current_settings_output_pcscf_server_address_list_out;
                    g_array_insert_val (self->arg_pcscf_server_address_list, pcscf_server_address_list_i, pcscf_server_address_list_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'PCSCF Server Address List' TLV", offset);
            }

            self->arg_pcscf_server_address_list_set = TRUE;

qmi_message_wds_get_current_settings_output_pcscf_server_address_list_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_PCSCF_DOMAIN_NAME_LIST, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_pcscf_domain_name_list_out;
            }
            {
                guint pcscf_domain_name_list_i;
                guint8 pcscf_domain_name_list_n_items;

                /* Read number of items in the array */
                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(pcscf_domain_name_list_n_items), NULL))
                    goto qmi_message_wds_get_current_settings_output_pcscf_domain_name_list_out;

                self->arg_pcscf_domain_name_list = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (gchar *),
                    (guint)pcscf_domain_name_list_n_items);

                g_array_set_clear_func (self->arg_pcscf_domain_name_list,
                                        (GDestroyNotify)pcscf_domain_name_list_gchar_pointer_clear);

                for (pcscf_domain_name_list_i = 0; pcscf_domain_name_list_i < pcscf_domain_name_list_n_items; pcscf_domain_name_list_i++) {
                    gchar * pcscf_domain_name_list_aux;

                    if (!qmi_message_tlv_read_string (message, init_offset, &offset, 2, 0, &(pcscf_domain_name_list_aux), NULL))
                        goto qmi_message_wds_get_current_settings_output_pcscf_domain_name_list_out;
                    g_array_insert_val (self->arg_pcscf_domain_name_list, pcscf_domain_name_list_i, pcscf_domain_name_list_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'PCSCF Domain Name List' TLV", offset);
            }

            self->arg_pcscf_domain_name_list_set = TRUE;

qmi_message_wds_get_current_settings_output_pcscf_domain_name_list_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_ADDRESS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_ipv6_address_out;
            }
            {
                guint address_i;
                guint16 address_n_items = 8;


                self->arg_ipv6_address_address = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (guint16),
                    (guint)address_n_items);

                for (address_i = 0; address_i < address_n_items; address_i++) {
                    guint16 address_aux;

                    if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &(address_aux), NULL))
                        goto qmi_message_wds_get_current_settings_output_ipv6_address_out;
                    g_array_insert_val (self->arg_ipv6_address_address, address_i, address_aux);
                }
            }
            if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(self->arg_ipv6_address_prefix_length), NULL))
                goto qmi_message_wds_get_current_settings_output_ipv6_address_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv6 Address' TLV", offset);
            }

            self->arg_ipv6_address_set = TRUE;

qmi_message_wds_get_current_settings_output_ipv6_address_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_GATEWAY_ADDRESS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_ipv6_gateway_address_out;
            }
            {
                guint address_i;
                guint16 address_n_items = 8;


                self->arg_ipv6_gateway_address_address = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (guint16),
                    (guint)address_n_items);

                for (address_i = 0; address_i < address_n_items; address_i++) {
                    guint16 address_aux;

                    if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &(address_aux), NULL))
                        goto qmi_message_wds_get_current_settings_output_ipv6_gateway_address_out;
                    g_array_insert_val (self->arg_ipv6_gateway_address_address, address_i, address_aux);
                }
            }
            if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(self->arg_ipv6_gateway_address_prefix_length), NULL))
                goto qmi_message_wds_get_current_settings_output_ipv6_gateway_address_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv6 Gateway Address' TLV", offset);
            }

            self->arg_ipv6_gateway_address_set = TRUE;

qmi_message_wds_get_current_settings_output_ipv6_gateway_address_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_PRIMARY_DNS_ADDRESS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_ipv6_primary_dns_address_out;
            }
            {
                guint ipv6_primary_dns_address_i;
                guint16 ipv6_primary_dns_address_n_items = 8;


                self->arg_ipv6_primary_dns_address = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (guint16),
                    (guint)ipv6_primary_dns_address_n_items);

                for (ipv6_primary_dns_address_i = 0; ipv6_primary_dns_address_i < ipv6_primary_dns_address_n_items; ipv6_primary_dns_address_i++) {
                    guint16 ipv6_primary_dns_address_aux;

                    if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &(ipv6_primary_dns_address_aux), NULL))
                        goto qmi_message_wds_get_current_settings_output_ipv6_primary_dns_address_out;
                    g_array_insert_val (self->arg_ipv6_primary_dns_address, ipv6_primary_dns_address_i, ipv6_primary_dns_address_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv6 Primary DNS Address' TLV", offset);
            }

            self->arg_ipv6_primary_dns_address_set = TRUE;

qmi_message_wds_get_current_settings_output_ipv6_primary_dns_address_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IPV6_SECONDARY_DNS_ADDRESS, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_ipv6_secondary_dns_address_out;
            }
            {
                guint ipv6_secondary_dns_address_i;
                guint16 ipv6_secondary_dns_address_n_items = 8;


                self->arg_ipv6_secondary_dns_address = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (guint16),
                    (guint)ipv6_secondary_dns_address_n_items);

                for (ipv6_secondary_dns_address_i = 0; ipv6_secondary_dns_address_i < ipv6_secondary_dns_address_n_items; ipv6_secondary_dns_address_i++) {
                    guint16 ipv6_secondary_dns_address_aux;

                    if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_BIG, &(ipv6_secondary_dns_address_aux), NULL))
                        goto qmi_message_wds_get_current_settings_output_ipv6_secondary_dns_address_out;
                    g_array_insert_val (self->arg_ipv6_secondary_dns_address, ipv6_secondary_dns_address_i, ipv6_secondary_dns_address_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IPv6 Secondary DNS Address' TLV", offset);
            }

            self->arg_ipv6_secondary_dns_address_set = TRUE;

qmi_message_wds_get_current_settings_output_ipv6_secondary_dns_address_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_MTU, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_mtu_out;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_mtu), NULL))
                goto qmi_message_wds_get_current_settings_output_mtu_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'MTU' TLV", offset);
            }

            self->arg_mtu_set = TRUE;

qmi_message_wds_get_current_settings_output_mtu_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_DOMAIN_NAME_LIST, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_domain_name_list_out;
            }
            {
                guint domain_name_list_i;
                guint8 domain_name_list_n_items;

                /* Read number of items in the array */
                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(domain_name_list_n_items), NULL))
                    goto qmi_message_wds_get_current_settings_output_domain_name_list_out;

                self->arg_domain_name_list = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (gchar *),
                    (guint)domain_name_list_n_items);

                g_array_set_clear_func (self->arg_domain_name_list,
                                        (GDestroyNotify)domain_name_list_gchar_pointer_clear);

                for (domain_name_list_i = 0; domain_name_list_i < domain_name_list_n_items; domain_name_list_i++) {
                    gchar * domain_name_list_aux;

                    if (!qmi_message_tlv_read_string (message, init_offset, &offset, 2, 0, &(domain_name_list_aux), NULL))
                        goto qmi_message_wds_get_current_settings_output_domain_name_list_out;
                    g_array_insert_val (self->arg_domain_name_list, domain_name_list_i, domain_name_list_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Domain Name List' TLV", offset);
            }

            self->arg_domain_name_list_set = TRUE;

qmi_message_wds_get_current_settings_output_domain_name_list_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IP_FAMILY, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_ip_family_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_current_settings_output_ip_family_out;
                self->arg_ip_family = (QmiWdsIpFamily)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IP Family' TLV", offset);
            }

            self->arg_ip_family_set = TRUE;

qmi_message_wds_get_current_settings_output_ip_family_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_IMCN_FLAG, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_imcn_flag_out;
            }
            if (!qmi_message_tlv_read_gint8 (message, init_offset, &offset, &(self->arg_imcn_flag), NULL))
                goto qmi_message_wds_get_current_settings_output_imcn_flag_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'IMCN Flag' TLV", offset);
            }

            self->arg_imcn_flag_set = TRUE;

qmi_message_wds_get_current_settings_output_imcn_flag_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS_OUTPUT_TLV_EXTENDED_TECHNOLOGY_PREFERENCE, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_settings_output_extended_technology_preference_out;
            }
            {
                guint16 tmp;

                if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, NULL))
                    goto qmi_message_wds_get_current_settings_output_extended_technology_preference_out;
                self->arg_extended_technology_preference = (QmiWdsExtendedTechnologyPreference)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Extended Technology Preference' TLV", offset);
            }

            self->arg_extended_technology_preference_set = TRUE;

qmi_message_wds_get_current_settings_output_extended_technology_preference_out:
            ;

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Get Autoconnect Settings */


/* --- Input -- */

/* Note: no fields in the Input container */

static QmiMessage *
__qmi_message_wds_get_autoconnect_settings_request_create (
    guint16 transaction_id,
    guint8 cid,
    gpointer unused,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS);

    return self;
}

/* --- Output -- */

struct _QmiMessageWdsGetAutoconnectSettingsOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;

    /* Status */
    gboolean arg_status_set;
    guint8 arg_status;

    /* Roaming */
    gboolean arg_roaming_set;
    guint8 arg_roaming;
};

#define QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_RESULT 0x02
#define QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_STATUS 0x01
#define QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_ROAMING 0x10

/**
 * qmi_message_wds_get_autoconnect_settings_output_get_result:
 * @self: a QmiMessageWdsGetAutoconnectSettingsOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_get_autoconnect_settings_output_get_result (
    QmiMessageWdsGetAutoconnectSettingsOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

/**
 * qmi_message_wds_get_autoconnect_settings_output_get_status:
 * @self: a #QmiMessageWdsGetAutoconnectSettingsOutput.
 * @value_status: a placeholder for the output #QmiWdsAutoconnectSetting, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Status' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_autoconnect_settings_output_get_status (
    QmiMessageWdsGetAutoconnectSettingsOutput *self,
    QmiWdsAutoconnectSetting *value_status,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_status_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Status' was not found in the message");
        return FALSE;
    }

    if (value_status)
        *value_status = (QmiWdsAutoconnectSetting)(self->arg_status);

    return TRUE;
}

/**
 * qmi_message_wds_get_autoconnect_settings_output_get_roaming:
 * @self: a #QmiMessageWdsGetAutoconnectSettingsOutput.
 * @value_roaming: a placeholder for the output #QmiWdsAutoconnectSettingRoaming, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Roaming' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_autoconnect_settings_output_get_roaming (
    QmiMessageWdsGetAutoconnectSettingsOutput *self,
    QmiWdsAutoconnectSettingRoaming *value_roaming,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_roaming_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Roaming' was not found in the message");
        return FALSE;
    }

    if (value_roaming)
        *value_roaming = (QmiWdsAutoconnectSettingRoaming)(self->arg_roaming);

    return TRUE;
}

GType
qmi_message_wds_get_autoconnect_settings_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetAutoconnectSettingsOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_autoconnect_settings_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_autoconnect_settings_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_autoconnect_settings_output_ref:
 * @self: a #QmiMessageWdsGetAutoconnectSettingsOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetAutoconnectSettingsOutput *
qmi_message_wds_get_autoconnect_settings_output_ref (QmiMessageWdsGetAutoconnectSettingsOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_autoconnect_settings_output_unref:
 * @self: a #QmiMessageWdsGetAutoconnectSettingsOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_autoconnect_settings_output_unref (QmiMessageWdsGetAutoconnectSettingsOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsGetAutoconnectSettingsOutput, self);
    }
}

static gchar *
qmi_message_wds_get_autoconnect_settings_output_status_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_STATUS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_AUTOCONNECT_SETTING_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_autoconnect_setting_get_string ((QmiWdsAutoconnectSetting)tmp));
#elif defined  __QMI_WDS_AUTOCONNECT_SETTING_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_autoconnect_setting_build_string_from_mask ((QmiWdsAutoconnectSetting)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsAutoconnectSetting
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_autoconnect_settings_output_roaming_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_ROAMING, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_AUTOCONNECT_SETTING_ROAMING_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_autoconnect_setting_roaming_get_string ((QmiWdsAutoconnectSettingRoaming)tmp));
#elif defined  __QMI_WDS_AUTOCONNECT_SETTING_ROAMING_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_autoconnect_setting_roaming_build_string_from_mask ((QmiWdsAutoconnectSettingRoaming)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsAutoconnectSettingRoaming
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_get_autoconnect_settings_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_get_autoconnect_settings_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_get_autoconnect_settings_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_STATUS:
            tlv_type_str = "Status";
            translated_value = qmi_message_wds_get_autoconnect_settings_output_status_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_ROAMING:
            tlv_type_str = "Roaming";
            translated_value = qmi_message_wds_get_autoconnect_settings_output_roaming_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_get_autoconnect_settings_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Get Autoconnect Settings\" (0x0034)\n",
                            line_prefix);

    {
        struct message_get_autoconnect_settings_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_get_autoconnect_settings_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsGetAutoconnectSettingsOutput *
__qmi_message_wds_get_autoconnect_settings_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsGetAutoconnectSettingsOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS, NULL);

    self = g_slice_new0 (QmiMessageWdsGetAutoconnectSettingsOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_get_autoconnect_settings_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_get_autoconnect_settings_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_STATUS, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Status TLV: ");
                qmi_message_wds_get_autoconnect_settings_output_unref (self);
                return NULL;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, error))
                    goto qmi_message_wds_get_autoconnect_settings_output_status_out;
                self->arg_status = (QmiWdsAutoconnectSetting)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Status' TLV", offset);
            }

            self->arg_status_set = TRUE;

qmi_message_wds_get_autoconnect_settings_output_status_out:
            if (!self->arg_status_set) {
                qmi_message_wds_get_autoconnect_settings_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_ROAMING, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_autoconnect_settings_output_roaming_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_autoconnect_settings_output_roaming_out;
                self->arg_roaming = (QmiWdsAutoconnectSettingRoaming)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Roaming' TLV", offset);
            }

            self->arg_roaming_set = TRUE;

qmi_message_wds_get_autoconnect_settings_output_roaming_out:
            ;

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Get Data Bearer Technology */


/* --- Input -- */

/* Note: no fields in the Input container */

static QmiMessage *
__qmi_message_wds_get_data_bearer_technology_request_create (
    guint16 transaction_id,
    guint8 cid,
    gpointer unused,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY);

    return self;
}

/* --- Output -- */

struct _QmiMessageWdsGetDataBearerTechnologyOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;

    /* Current */
    gboolean arg_current_set;
    gint8 arg_current;

    /* Last */
    gboolean arg_last_set;
    gint8 arg_last;
};

#define QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_RESULT 0x02
#define QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_CURRENT 0x01
#define QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_LAST 0x10

/**
 * qmi_message_wds_get_data_bearer_technology_output_get_result:
 * @self: a QmiMessageWdsGetDataBearerTechnologyOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_get_data_bearer_technology_output_get_result (
    QmiMessageWdsGetDataBearerTechnologyOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

/**
 * qmi_message_wds_get_data_bearer_technology_output_get_current:
 * @self: a #QmiMessageWdsGetDataBearerTechnologyOutput.
 * @value_current: a placeholder for the output #QmiWdsDataBearerTechnology, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Current' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_data_bearer_technology_output_get_current (
    QmiMessageWdsGetDataBearerTechnologyOutput *self,
    QmiWdsDataBearerTechnology *value_current,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_current_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Current' was not found in the message");
        return FALSE;
    }

    if (value_current)
        *value_current = (QmiWdsDataBearerTechnology)(self->arg_current);

    return TRUE;
}

/**
 * qmi_message_wds_get_data_bearer_technology_output_get_last:
 * @self: a #QmiMessageWdsGetDataBearerTechnologyOutput.
 * @value_last: a placeholder for the output #QmiWdsDataBearerTechnology, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Last' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_data_bearer_technology_output_get_last (
    QmiMessageWdsGetDataBearerTechnologyOutput *self,
    QmiWdsDataBearerTechnology *value_last,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_last_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Last' was not found in the message");
        return FALSE;
    }

    if (value_last)
        *value_last = (QmiWdsDataBearerTechnology)(self->arg_last);

    return TRUE;
}

GType
qmi_message_wds_get_data_bearer_technology_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetDataBearerTechnologyOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_data_bearer_technology_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_data_bearer_technology_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_data_bearer_technology_output_ref:
 * @self: a #QmiMessageWdsGetDataBearerTechnologyOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetDataBearerTechnologyOutput *
qmi_message_wds_get_data_bearer_technology_output_ref (QmiMessageWdsGetDataBearerTechnologyOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_data_bearer_technology_output_unref:
 * @self: a #QmiMessageWdsGetDataBearerTechnologyOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_data_bearer_technology_output_unref (QmiMessageWdsGetDataBearerTechnologyOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsGetDataBearerTechnologyOutput, self);
    }
}

static gchar *
qmi_message_wds_get_data_bearer_technology_output_current_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_CURRENT, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gint8 tmp;

        if (!qmi_message_tlv_read_gint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_DATA_BEARER_TECHNOLOGY_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_data_bearer_technology_get_string ((QmiWdsDataBearerTechnology)tmp));
#elif defined  __QMI_WDS_DATA_BEARER_TECHNOLOGY_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_data_bearer_technology_build_string_from_mask ((QmiWdsDataBearerTechnology)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsDataBearerTechnology
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_data_bearer_technology_output_last_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_LAST, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        gint8 tmp;

        if (!qmi_message_tlv_read_gint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_DATA_BEARER_TECHNOLOGY_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_data_bearer_technology_get_string ((QmiWdsDataBearerTechnology)tmp));
#elif defined  __QMI_WDS_DATA_BEARER_TECHNOLOGY_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_data_bearer_technology_build_string_from_mask ((QmiWdsDataBearerTechnology)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsDataBearerTechnology
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_get_data_bearer_technology_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_get_data_bearer_technology_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_get_data_bearer_technology_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_CURRENT:
            tlv_type_str = "Current";
            translated_value = qmi_message_wds_get_data_bearer_technology_output_current_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_LAST:
            tlv_type_str = "Last";
            translated_value = qmi_message_wds_get_data_bearer_technology_output_last_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_get_data_bearer_technology_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Get Data Bearer Technology\" (0x0037)\n",
                            line_prefix);

    {
        struct message_get_data_bearer_technology_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_get_data_bearer_technology_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsGetDataBearerTechnologyOutput *
__qmi_message_wds_get_data_bearer_technology_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsGetDataBearerTechnologyOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY, NULL);

    self = g_slice_new0 (QmiMessageWdsGetDataBearerTechnologyOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_get_data_bearer_technology_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_get_data_bearer_technology_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_CURRENT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Current TLV: ");
                qmi_message_wds_get_data_bearer_technology_output_unref (self);
                return NULL;
            }
            {
                gint8 tmp;

                if (!qmi_message_tlv_read_gint8 (message, init_offset, &offset, &tmp, error))
                    goto qmi_message_wds_get_data_bearer_technology_output_current_out;
                self->arg_current = (QmiWdsDataBearerTechnology)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Current' TLV", offset);
            }

            self->arg_current_set = TRUE;

qmi_message_wds_get_data_bearer_technology_output_current_out:
            if (!self->arg_current_set) {
                qmi_message_wds_get_data_bearer_technology_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status != QMI_STATUS_SUCCESS))
            break;
        /* Prerequisite.... */
        if (!(self->arg_result.error_code == QMI_PROTOCOL_ERROR_OUT_OF_CALL))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_LAST, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_data_bearer_technology_output_last_out;
            }
            {
                gint8 tmp;

                if (!qmi_message_tlv_read_gint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_data_bearer_technology_output_last_out;
                self->arg_last = (QmiWdsDataBearerTechnology)tmp;
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Last' TLV", offset);
            }

            self->arg_last_set = TRUE;

qmi_message_wds_get_data_bearer_technology_output_last_out:
            ;

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Get Current Data Bearer Technology */


/* --- Input -- */

/* Note: no fields in the Input container */

static QmiMessage *
__qmi_message_wds_get_current_data_bearer_technology_request_create (
    guint16 transaction_id,
    guint8 cid,
    gpointer unused,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY);

    return self;
}

/* --- Output -- */

struct _QmiMessageWdsGetCurrentDataBearerTechnologyOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;

    /* Last */
    gboolean arg_last_set;
    guint8 arg_last_network_type;
    guint32 arg_last_rat_mask;
    guint32 arg_last_so_mask;

    /* Current */
    gboolean arg_current_set;
    guint8 arg_current_network_type;
    guint32 arg_current_rat_mask;
    guint32 arg_current_so_mask;
};

#define QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_RESULT 0x02
#define QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_LAST 0x10
#define QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_CURRENT 0x01

/**
 * qmi_message_wds_get_current_data_bearer_technology_output_get_result:
 * @self: a QmiMessageWdsGetCurrentDataBearerTechnologyOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_get_current_data_bearer_technology_output_get_result (
    QmiMessageWdsGetCurrentDataBearerTechnologyOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

/**
 * qmi_message_wds_get_current_data_bearer_technology_output_get_last:
 * @self: a #QmiMessageWdsGetCurrentDataBearerTechnologyOutput.
 * @value_last_network_type: a placeholder for the output #QmiWdsNetworkType, or %NULL if not required.
 * @value_last_rat_mask: a placeholder for the output #guint32, or %NULL if not required.
 * @value_last_so_mask: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Last' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_data_bearer_technology_output_get_last (
    QmiMessageWdsGetCurrentDataBearerTechnologyOutput *self,
    QmiWdsNetworkType *value_last_network_type,
    guint32 *value_last_rat_mask,
    guint32 *value_last_so_mask,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_last_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Last' was not found in the message");
        return FALSE;
    }

    if (value_last_network_type)
        *value_last_network_type = (QmiWdsNetworkType)(self->arg_last_network_type);
    if (value_last_rat_mask)
        *value_last_rat_mask = self->arg_last_rat_mask;
    if (value_last_so_mask)
        *value_last_so_mask = self->arg_last_so_mask;

    return TRUE;
}

/**
 * qmi_message_wds_get_current_data_bearer_technology_output_get_current:
 * @self: a #QmiMessageWdsGetCurrentDataBearerTechnologyOutput.
 * @value_current_network_type: a placeholder for the output #QmiWdsNetworkType, or %NULL if not required.
 * @value_current_rat_mask: a placeholder for the output #guint32, or %NULL if not required.
 * @value_current_so_mask: a placeholder for the output #guint32, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Current' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_current_data_bearer_technology_output_get_current (
    QmiMessageWdsGetCurrentDataBearerTechnologyOutput *self,
    QmiWdsNetworkType *value_current_network_type,
    guint32 *value_current_rat_mask,
    guint32 *value_current_so_mask,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_current_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Current' was not found in the message");
        return FALSE;
    }

    if (value_current_network_type)
        *value_current_network_type = (QmiWdsNetworkType)(self->arg_current_network_type);
    if (value_current_rat_mask)
        *value_current_rat_mask = self->arg_current_rat_mask;
    if (value_current_so_mask)
        *value_current_so_mask = self->arg_current_so_mask;

    return TRUE;
}

GType
qmi_message_wds_get_current_data_bearer_technology_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetCurrentDataBearerTechnologyOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_current_data_bearer_technology_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_current_data_bearer_technology_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_current_data_bearer_technology_output_ref:
 * @self: a #QmiMessageWdsGetCurrentDataBearerTechnologyOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetCurrentDataBearerTechnologyOutput *
qmi_message_wds_get_current_data_bearer_technology_output_ref (QmiMessageWdsGetCurrentDataBearerTechnologyOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_current_data_bearer_technology_output_unref:
 * @self: a #QmiMessageWdsGetCurrentDataBearerTechnologyOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_current_data_bearer_technology_output_unref (QmiMessageWdsGetCurrentDataBearerTechnologyOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsGetCurrentDataBearerTechnologyOutput, self);
    }
}

static gchar *
qmi_message_wds_get_current_data_bearer_technology_output_last_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_LAST, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " network_type = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_NETWORK_TYPE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_network_type_get_string ((QmiWdsNetworkType)tmp));
#elif defined  __QMI_WDS_NETWORK_TYPE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_network_type_build_string_from_mask ((QmiWdsNetworkType)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsNetworkType
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " rat_mask = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " so_mask = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_current_data_bearer_technology_output_current_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_CURRENT, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    g_string_append (printable, "[");
    g_string_append (printable, " network_type = '");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_NETWORK_TYPE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_network_type_get_string ((QmiWdsNetworkType)tmp));
#elif defined  __QMI_WDS_NETWORK_TYPE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_network_type_build_string_from_mask ((QmiWdsNetworkType)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsNetworkType
#endif
    }
    g_string_append (printable, "'");
    g_string_append (printable, " rat_mask = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " so_mask = '");

    {
        guint32 tmp;

        if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
            goto out;
        g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
    }
    g_string_append (printable, "'");
    g_string_append (printable, " ]");

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_get_current_data_bearer_technology_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_get_current_data_bearer_technology_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_get_current_data_bearer_technology_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_LAST:
            tlv_type_str = "Last";
            translated_value = qmi_message_wds_get_current_data_bearer_technology_output_last_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_CURRENT:
            tlv_type_str = "Current";
            translated_value = qmi_message_wds_get_current_data_bearer_technology_output_current_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_get_current_data_bearer_technology_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Get Current Data Bearer Technology\" (0x0044)\n",
                            line_prefix);

    {
        struct message_get_current_data_bearer_technology_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_get_current_data_bearer_technology_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsGetCurrentDataBearerTechnologyOutput *
__qmi_message_wds_get_current_data_bearer_technology_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsGetCurrentDataBearerTechnologyOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY, NULL);

    self = g_slice_new0 (QmiMessageWdsGetCurrentDataBearerTechnologyOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_get_current_data_bearer_technology_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_get_current_data_bearer_technology_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_LAST, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_current_data_bearer_technology_output_last_out;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, NULL))
                    goto qmi_message_wds_get_current_data_bearer_technology_output_last_out;
                self->arg_last_network_type = (QmiWdsNetworkType)tmp;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_last_rat_mask), NULL))
                goto qmi_message_wds_get_current_data_bearer_technology_output_last_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_last_so_mask), NULL))
                goto qmi_message_wds_get_current_data_bearer_technology_output_last_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Last' TLV", offset);
            }

            self->arg_last_set = TRUE;

qmi_message_wds_get_current_data_bearer_technology_output_last_out:
            ;

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY_OUTPUT_TLV_CURRENT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Current TLV: ");
                qmi_message_wds_get_current_data_bearer_technology_output_unref (self);
                return NULL;
            }
            {
                guint8 tmp;

                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, error))
                    goto qmi_message_wds_get_current_data_bearer_technology_output_current_out;
                self->arg_current_network_type = (QmiWdsNetworkType)tmp;
            }
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_current_rat_mask), error))
                goto qmi_message_wds_get_current_data_bearer_technology_output_current_out;
            if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_current_so_mask), error))
                goto qmi_message_wds_get_current_data_bearer_technology_output_current_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Current' TLV", offset);
            }

            self->arg_current_set = TRUE;

qmi_message_wds_get_current_data_bearer_technology_output_current_out:
            if (!self->arg_current_set) {
                qmi_message_wds_get_current_data_bearer_technology_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Set IP Family */


/* --- Input -- */

struct _QmiMessageWdsSetIpFamilyInput {
    volatile gint ref_count;

    /* Preference */
    gboolean arg_preference_set;
    guint8 arg_preference;
};

#define QMI_MESSAGE_WDS_SET_IP_FAMILY_INPUT_TLV_PREFERENCE 0x01

/**
 * qmi_message_wds_set_ip_family_input_get_preference:
 * @self: a #QmiMessageWdsSetIpFamilyInput.
 * @value_preference: a placeholder for the output #QmiWdsIpFamily, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Preference' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_set_ip_family_input_get_preference (
    QmiMessageWdsSetIpFamilyInput *self,
    QmiWdsIpFamily *value_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_preference_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Preference' was not found in the message");
        return FALSE;
    }

    if (value_preference)
        *value_preference = (QmiWdsIpFamily)(self->arg_preference);

    return TRUE;
}

/**
 * qmi_message_wds_set_ip_family_input_set_preference:
 * @self: a #QmiMessageWdsSetIpFamilyInput.
 * @value_preference: a #QmiWdsIpFamily.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Preference' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_set_ip_family_input_set_preference (
    QmiMessageWdsSetIpFamilyInput *self,
    QmiWdsIpFamily value_preference,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_preference = (guint8)(value_preference);
    self->arg_preference_set = TRUE;

    return TRUE;
}

GType
qmi_message_wds_set_ip_family_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsSetIpFamilyInput"),
                                          (GBoxedCopyFunc) qmi_message_wds_set_ip_family_input_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_set_ip_family_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_set_ip_family_input_ref:
 * @self: a #QmiMessageWdsSetIpFamilyInput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsSetIpFamilyInput *
qmi_message_wds_set_ip_family_input_ref (QmiMessageWdsSetIpFamilyInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_set_ip_family_input_unref:
 * @self: a #QmiMessageWdsSetIpFamilyInput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_set_ip_family_input_unref (QmiMessageWdsSetIpFamilyInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsSetIpFamilyInput, self);
    }
}

/**
 * qmi_message_wds_set_ip_family_input_new:
 *
 * Allocates a new #QmiMessageWdsSetIpFamilyInput.
 *
 * Returns: the newly created #QmiMessageWdsSetIpFamilyInput. The returned value should be freed with qmi_message_wds_set_ip_family_input_unref().
 */
QmiMessageWdsSetIpFamilyInput *
qmi_message_wds_set_ip_family_input_new (void)
{
    QmiMessageWdsSetIpFamilyInput *self;

    self = g_slice_new0 (QmiMessageWdsSetIpFamilyInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_wds_set_ip_family_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageWdsSetIpFamilyInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_SET_IP_FAMILY);

    /* All TLVs are optional, we allow NULL input */
    if (!input)
        return self;

    /* Try to add the 'Preference' TLV */
    if (input->arg_preference_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_SET_IP_FAMILY_INPUT_TLV_PREFERENCE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Preference': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_preference;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Preference': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Preference': ");
            goto error_out;
        }
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

struct _QmiMessageWdsSetIpFamilyOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;
};

#define QMI_MESSAGE_WDS_SET_IP_FAMILY_OUTPUT_TLV_RESULT 0x02

/**
 * qmi_message_wds_set_ip_family_output_get_result:
 * @self: a QmiMessageWdsSetIpFamilyOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_set_ip_family_output_get_result (
    QmiMessageWdsSetIpFamilyOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

GType
qmi_message_wds_set_ip_family_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsSetIpFamilyOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_set_ip_family_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_set_ip_family_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_set_ip_family_output_ref:
 * @self: a #QmiMessageWdsSetIpFamilyOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsSetIpFamilyOutput *
qmi_message_wds_set_ip_family_output_ref (QmiMessageWdsSetIpFamilyOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_set_ip_family_output_unref:
 * @self: a #QmiMessageWdsSetIpFamilyOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_set_ip_family_output_unref (QmiMessageWdsSetIpFamilyOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsSetIpFamilyOutput, self);
    }
}

static gchar *
qmi_message_wds_set_ip_family_input_preference_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_SET_IP_FAMILY_INPUT_TLV_PREFERENCE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_IP_FAMILY_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_ip_family_get_string ((QmiWdsIpFamily)tmp));
#elif defined  __QMI_WDS_IP_FAMILY_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_ip_family_build_string_from_mask ((QmiWdsIpFamily)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsIpFamily
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_set_ip_family_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_set_ip_family_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_set_ip_family_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_WDS_SET_IP_FAMILY_INPUT_TLV_PREFERENCE:
            tlv_type_str = "Preference";
            translated_value = qmi_message_wds_set_ip_family_input_preference_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_SET_IP_FAMILY_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_set_ip_family_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Set IP Family\" (0x004D)\n",
                            line_prefix);

    {
        struct message_set_ip_family_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_set_ip_family_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsSetIpFamilyOutput *
__qmi_message_wds_set_ip_family_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsSetIpFamilyOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_SET_IP_FAMILY, NULL);

    self = g_slice_new0 (QmiMessageWdsSetIpFamilyOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_SET_IP_FAMILY_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_set_ip_family_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_set_ip_family_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Set Autoconnect Settings */


/* --- Input -- */

struct _QmiMessageWdsSetAutoconnectSettingsInput {
    volatile gint ref_count;

    /* Roaming */
    gboolean arg_roaming_set;
    guint8 arg_roaming;

    /* Status */
    gboolean arg_status_set;
    guint8 arg_status;
};

#define QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS_INPUT_TLV_ROAMING 0x10
#define QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS_INPUT_TLV_STATUS 0x01

/**
 * qmi_message_wds_set_autoconnect_settings_input_get_roaming:
 * @self: a #QmiMessageWdsSetAutoconnectSettingsInput.
 * @value_roaming: a placeholder for the output #QmiWdsAutoconnectSettingRoaming, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Roaming' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_set_autoconnect_settings_input_get_roaming (
    QmiMessageWdsSetAutoconnectSettingsInput *self,
    QmiWdsAutoconnectSettingRoaming *value_roaming,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_roaming_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Roaming' was not found in the message");
        return FALSE;
    }

    if (value_roaming)
        *value_roaming = (QmiWdsAutoconnectSettingRoaming)(self->arg_roaming);

    return TRUE;
}

/**
 * qmi_message_wds_set_autoconnect_settings_input_set_roaming:
 * @self: a #QmiMessageWdsSetAutoconnectSettingsInput.
 * @value_roaming: a #QmiWdsAutoconnectSettingRoaming.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Roaming' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_set_autoconnect_settings_input_set_roaming (
    QmiMessageWdsSetAutoconnectSettingsInput *self,
    QmiWdsAutoconnectSettingRoaming value_roaming,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_roaming = (guint8)(value_roaming);
    self->arg_roaming_set = TRUE;

    return TRUE;
}

/**
 * qmi_message_wds_set_autoconnect_settings_input_get_status:
 * @self: a #QmiMessageWdsSetAutoconnectSettingsInput.
 * @value_status: a placeholder for the output #QmiWdsAutoconnectSetting, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Status' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_set_autoconnect_settings_input_get_status (
    QmiMessageWdsSetAutoconnectSettingsInput *self,
    QmiWdsAutoconnectSetting *value_status,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_status_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Status' was not found in the message");
        return FALSE;
    }

    if (value_status)
        *value_status = (QmiWdsAutoconnectSetting)(self->arg_status);

    return TRUE;
}

/**
 * qmi_message_wds_set_autoconnect_settings_input_set_status:
 * @self: a #QmiMessageWdsSetAutoconnectSettingsInput.
 * @value_status: a #QmiWdsAutoconnectSetting.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Status' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_set_autoconnect_settings_input_set_status (
    QmiMessageWdsSetAutoconnectSettingsInput *self,
    QmiWdsAutoconnectSetting value_status,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_status = (guint8)(value_status);
    self->arg_status_set = TRUE;

    return TRUE;
}

GType
qmi_message_wds_set_autoconnect_settings_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsSetAutoconnectSettingsInput"),
                                          (GBoxedCopyFunc) qmi_message_wds_set_autoconnect_settings_input_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_set_autoconnect_settings_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_set_autoconnect_settings_input_ref:
 * @self: a #QmiMessageWdsSetAutoconnectSettingsInput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsSetAutoconnectSettingsInput *
qmi_message_wds_set_autoconnect_settings_input_ref (QmiMessageWdsSetAutoconnectSettingsInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_set_autoconnect_settings_input_unref:
 * @self: a #QmiMessageWdsSetAutoconnectSettingsInput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_set_autoconnect_settings_input_unref (QmiMessageWdsSetAutoconnectSettingsInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsSetAutoconnectSettingsInput, self);
    }
}

/**
 * qmi_message_wds_set_autoconnect_settings_input_new:
 *
 * Allocates a new #QmiMessageWdsSetAutoconnectSettingsInput.
 *
 * Returns: the newly created #QmiMessageWdsSetAutoconnectSettingsInput. The returned value should be freed with qmi_message_wds_set_autoconnect_settings_input_unref().
 */
QmiMessageWdsSetAutoconnectSettingsInput *
qmi_message_wds_set_autoconnect_settings_input_new (void)
{
    QmiMessageWdsSetAutoconnectSettingsInput *self;

    self = g_slice_new0 (QmiMessageWdsSetAutoconnectSettingsInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_wds_set_autoconnect_settings_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageWdsSetAutoconnectSettingsInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS);

    /* All TLVs are optional, we allow NULL input */
    if (!input)
        return self;

    /* Try to add the 'Roaming' TLV */
    if (input->arg_roaming_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS_INPUT_TLV_ROAMING, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Roaming': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_roaming;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Roaming': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Roaming': ");
            goto error_out;
        }
    }

    /* Try to add the 'Status' TLV */
    if (input->arg_status_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS_INPUT_TLV_STATUS, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Status': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_status;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Status': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Status': ");
            goto error_out;
        }
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

struct _QmiMessageWdsSetAutoconnectSettingsOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;
};

#define QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_RESULT 0x02

/**
 * qmi_message_wds_set_autoconnect_settings_output_get_result:
 * @self: a QmiMessageWdsSetAutoconnectSettingsOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_set_autoconnect_settings_output_get_result (
    QmiMessageWdsSetAutoconnectSettingsOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

GType
qmi_message_wds_set_autoconnect_settings_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsSetAutoconnectSettingsOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_set_autoconnect_settings_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_set_autoconnect_settings_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_set_autoconnect_settings_output_ref:
 * @self: a #QmiMessageWdsSetAutoconnectSettingsOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsSetAutoconnectSettingsOutput *
qmi_message_wds_set_autoconnect_settings_output_ref (QmiMessageWdsSetAutoconnectSettingsOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_set_autoconnect_settings_output_unref:
 * @self: a #QmiMessageWdsSetAutoconnectSettingsOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_set_autoconnect_settings_output_unref (QmiMessageWdsSetAutoconnectSettingsOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsSetAutoconnectSettingsOutput, self);
    }
}

static gchar *
qmi_message_wds_set_autoconnect_settings_input_roaming_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS_INPUT_TLV_ROAMING, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_AUTOCONNECT_SETTING_ROAMING_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_autoconnect_setting_roaming_get_string ((QmiWdsAutoconnectSettingRoaming)tmp));
#elif defined  __QMI_WDS_AUTOCONNECT_SETTING_ROAMING_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_autoconnect_setting_roaming_build_string_from_mask ((QmiWdsAutoconnectSettingRoaming)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsAutoconnectSettingRoaming
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_set_autoconnect_settings_input_status_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS_INPUT_TLV_STATUS, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_AUTOCONNECT_SETTING_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_autoconnect_setting_get_string ((QmiWdsAutoconnectSetting)tmp));
#elif defined  __QMI_WDS_AUTOCONNECT_SETTING_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_autoconnect_setting_build_string_from_mask ((QmiWdsAutoconnectSetting)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsAutoconnectSetting
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_set_autoconnect_settings_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_set_autoconnect_settings_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_set_autoconnect_settings_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS_INPUT_TLV_ROAMING:
            tlv_type_str = "Roaming";
            translated_value = qmi_message_wds_set_autoconnect_settings_input_roaming_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS_INPUT_TLV_STATUS:
            tlv_type_str = "Status";
            translated_value = qmi_message_wds_set_autoconnect_settings_input_status_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_set_autoconnect_settings_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Set Autoconnect Settings\" (0x0051)\n",
                            line_prefix);

    {
        struct message_set_autoconnect_settings_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_set_autoconnect_settings_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsSetAutoconnectSettingsOutput *
__qmi_message_wds_set_autoconnect_settings_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsSetAutoconnectSettingsOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS, NULL);

    self = g_slice_new0 (QmiMessageWdsSetAutoconnectSettingsOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_set_autoconnect_settings_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_set_autoconnect_settings_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* REQUEST/RESPONSE: Qmi Message WDS Get PDN Throttle Info */


/* --- Input -- */

struct _QmiMessageWdsGetPdnThrottleInfoInput {
    volatile gint ref_count;

    /* Network Type */
    gboolean arg_network_type_set;
    guint8 arg_network_type;
};

#define QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO_INPUT_TLV_NETWORK_TYPE 0x01

/**
 * qmi_message_wds_get_pdn_throttle_info_input_get_network_type:
 * @self: a #QmiMessageWdsGetPdnThrottleInfoInput.
 * @value_network_type: a placeholder for the output #QmiWdsDataSystemNetworkType, or %NULL if not required.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Network Type' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_pdn_throttle_info_input_get_network_type (
    QmiMessageWdsGetPdnThrottleInfoInput *self,
    QmiWdsDataSystemNetworkType *value_network_type,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_network_type_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Network Type' was not found in the message");
        return FALSE;
    }

    if (value_network_type)
        *value_network_type = (QmiWdsDataSystemNetworkType)(self->arg_network_type);

    return TRUE;
}

/**
 * qmi_message_wds_get_pdn_throttle_info_input_set_network_type:
 * @self: a #QmiMessageWdsGetPdnThrottleInfoInput.
 * @value_network_type: a #QmiWdsDataSystemNetworkType.
 * @error: Return location for error or %NULL.
 *
 * Set the 'Network Type' field in the message.
 *
 * Returns: %TRUE if @value was successfully set, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_pdn_throttle_info_input_set_network_type (
    QmiMessageWdsGetPdnThrottleInfoInput *self,
    QmiWdsDataSystemNetworkType value_network_type,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    self->arg_network_type = (guint8)(value_network_type);
    self->arg_network_type_set = TRUE;

    return TRUE;
}

GType
qmi_message_wds_get_pdn_throttle_info_input_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetPdnThrottleInfoInput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_pdn_throttle_info_input_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_pdn_throttle_info_input_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_pdn_throttle_info_input_ref:
 * @self: a #QmiMessageWdsGetPdnThrottleInfoInput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetPdnThrottleInfoInput *
qmi_message_wds_get_pdn_throttle_info_input_ref (QmiMessageWdsGetPdnThrottleInfoInput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_pdn_throttle_info_input_unref:
 * @self: a #QmiMessageWdsGetPdnThrottleInfoInput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_pdn_throttle_info_input_unref (QmiMessageWdsGetPdnThrottleInfoInput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        g_slice_free (QmiMessageWdsGetPdnThrottleInfoInput, self);
    }
}

/**
 * qmi_message_wds_get_pdn_throttle_info_input_new:
 *
 * Allocates a new #QmiMessageWdsGetPdnThrottleInfoInput.
 *
 * Returns: the newly created #QmiMessageWdsGetPdnThrottleInfoInput. The returned value should be freed with qmi_message_wds_get_pdn_throttle_info_input_unref().
 */
QmiMessageWdsGetPdnThrottleInfoInput *
qmi_message_wds_get_pdn_throttle_info_input_new (void)
{
    QmiMessageWdsGetPdnThrottleInfoInput *self;

    self = g_slice_new0 (QmiMessageWdsGetPdnThrottleInfoInput);
    self->ref_count = 1;
    return self;
}

static QmiMessage *
__qmi_message_wds_get_pdn_throttle_info_request_create (
    guint16 transaction_id,
    guint8 cid,
    QmiMessageWdsGetPdnThrottleInfoInput *input,
    GError **error)
{
    QmiMessage *self;

    self = qmi_message_new (QMI_SERVICE_WDS,
                            cid,
                            transaction_id,
                            QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO);

    /* All TLVs are optional, we allow NULL input */
    if (!input)
        return self;

    /* Try to add the 'Network Type' TLV */
    if (input->arg_network_type_set) {
        gsize tlv_offset;

        if (!(tlv_offset = qmi_message_tlv_write_init (self, (guint8)QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO_INPUT_TLV_NETWORK_TYPE, error))) {
            g_prefix_error (error, "Cannot initialize TLV 'Network Type': ");
            goto error_out;
        }

        {
            guint8 tmp;

            tmp = (guint8) input->arg_network_type;
            /* Write the guint8 variable to the buffer */
            if (!qmi_message_tlv_write_guint8 (self, tmp, error)) {
                g_prefix_error (error, "Cannot write enum in TLV 'Network Type': ");
                goto error_out;
            }
        }

        if (!qmi_message_tlv_write_complete (self, tlv_offset, error)) {
            g_prefix_error (error, "Cannot complete TLV 'Network Type': ");
            goto error_out;
        }
    }

    return self;

error_out:
    qmi_message_unref (self);
    return NULL;
}

/* --- Output -- */

static void
info_qmimessagewdsgetpdnthrottleinfooutputinfoelement_clear (QmiMessageWdsGetPdnThrottleInfoOutputInfoElement *p)
{
    g_free ((*p).apn);
}

struct _QmiMessageWdsGetPdnThrottleInfoOutput {
    volatile gint ref_count;

    /* Result */
    gboolean arg_result_set;
    QmiMessageResult arg_result;

    /* Info */
    gboolean arg_info_set;
    GArray *arg_info;
};

#define QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO_OUTPUT_TLV_RESULT 0x02
#define QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO_OUTPUT_TLV_INFO 0x10

/**
 * qmi_message_wds_get_pdn_throttle_info_output_get_result:
 * @self: a QmiMessageWdsGetPdnThrottleInfoOutput.
 * @error: Return location for error or %NULL.
 *
 * Get the result of the QMI operation.
 *
 * Returns: %TRUE if the QMI operation succeeded, %FALSE if @error is set.
 */
gboolean
qmi_message_wds_get_pdn_throttle_info_output_get_result (
    QmiMessageWdsGetPdnThrottleInfoOutput *self,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    /* We should always have a result set in the response message */
    if (!self->arg_result_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_INVALID_MESSAGE,
                     "No 'Result' field given in the message");
        return FALSE;
    }

    if (self->arg_result.error_status == QMI_STATUS_SUCCESS) {
        /* Operation succeeded */
        return TRUE;
    }

    /* Report a QMI protocol error */
    g_set_error (error,
                 QMI_PROTOCOL_ERROR,
                 (QmiProtocolError) self->arg_result.error_code,
                 "QMI protocol error (%u): '%s'",
                 self->arg_result.error_code,
                 qmi_protocol_error_get_string ((QmiProtocolError) self->arg_result.error_code));
    return FALSE;
}

/**
 * qmi_message_wds_get_pdn_throttle_info_output_get_info:
 * @self: a #QmiMessageWdsGetPdnThrottleInfoOutput.
 * @value_info: a placeholder for the output #GArray of #QmiMessageWdsGetPdnThrottleInfoOutputInfoElement elements, or %NULL if not required. Do not free it, it is owned by @self.
 * @error: Return location for error or %NULL.
 *
 * Get the 'Info' field from @self.
 *
 * Returns: %TRUE if the field is found, %FALSE otherwise.
 */
gboolean
qmi_message_wds_get_pdn_throttle_info_output_get_info (
    QmiMessageWdsGetPdnThrottleInfoOutput *self,
    GArray **value_info,
    GError **error)
{
    g_return_val_if_fail (self != NULL, FALSE);

    if (!self->arg_info_set) {
        g_set_error (error,
                     QMI_CORE_ERROR,
                     QMI_CORE_ERROR_TLV_NOT_FOUND,
                     "Field 'Info' was not found in the message");
        return FALSE;
    }

    if (value_info)
        *value_info = self->arg_info;

    return TRUE;
}

GType
qmi_message_wds_get_pdn_throttle_info_output_get_type (void)
{
    static volatile gsize g_define_type_id__volatile = 0;

    if (g_once_init_enter (&g_define_type_id__volatile)) {
        GType g_define_type_id =
            g_boxed_type_register_static (g_intern_static_string ("QmiMessageWdsGetPdnThrottleInfoOutput"),
                                          (GBoxedCopyFunc) qmi_message_wds_get_pdn_throttle_info_output_ref,
                                          (GBoxedFreeFunc) qmi_message_wds_get_pdn_throttle_info_output_unref);

        g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

    return g_define_type_id__volatile;
}

/**
 * qmi_message_wds_get_pdn_throttle_info_output_ref:
 * @self: a #QmiMessageWdsGetPdnThrottleInfoOutput.
 *
 * Atomically increments the reference count of @self by one.
 *
 * Returns: the new reference to @self.
 */
QmiMessageWdsGetPdnThrottleInfoOutput *
qmi_message_wds_get_pdn_throttle_info_output_ref (QmiMessageWdsGetPdnThrottleInfoOutput *self)
{
    g_return_val_if_fail (self != NULL, NULL);

    g_atomic_int_inc (&self->ref_count);
    return self;
}

/**
 * qmi_message_wds_get_pdn_throttle_info_output_unref:
 * @self: a #QmiMessageWdsGetPdnThrottleInfoOutput.
 *
 * Atomically decrements the reference count of @self by one.
 * If the reference count drops to 0, @self is completely disposed.
 */
void
qmi_message_wds_get_pdn_throttle_info_output_unref (QmiMessageWdsGetPdnThrottleInfoOutput *self)
{
    g_return_if_fail (self != NULL);

    if (g_atomic_int_dec_and_test (&self->ref_count)) {
        if (self->arg_info)
            g_array_unref (self->arg_info);
        g_slice_free (QmiMessageWdsGetPdnThrottleInfoOutput, self);
    }
}

static gchar *
qmi_message_wds_get_pdn_throttle_info_input_network_type_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO_INPUT_TLV_NETWORK_TYPE, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");

    {
        guint8 tmp;

        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &tmp, &error))
            goto out;
#if defined  __QMI_WDS_DATA_SYSTEM_NETWORK_TYPE_IS_ENUM__
        g_string_append_printf (printable, "%s", qmi_wds_data_system_network_type_get_string ((QmiWdsDataSystemNetworkType)tmp));
#elif defined  __QMI_WDS_DATA_SYSTEM_NETWORK_TYPE_IS_FLAGS__
        {
            gchar *flags_str;

            flags_str = qmi_wds_data_system_network_type_build_string_from_mask ((QmiWdsDataSystemNetworkType)tmp);
            g_string_append_printf (printable, "%s", flags_str);
            g_free (flags_str);
        }
#else
# error unexpected public format: QmiWdsDataSystemNetworkType
#endif
    }

    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

static gchar *
qmi_message_wds_get_pdn_throttle_info_output_info_get_printable (
    QmiMessage *message,
    const gchar *line_prefix)
{
    gsize offset = 0;
    gsize init_offset;
    GString *printable;
    GError *error = NULL;

    if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO_OUTPUT_TLV_INFO, NULL, NULL)) == 0)
        return NULL;

    printable = g_string_new ("");
    {
        guint info_i;
        guint8 info_n_items;

        /* Read number of items in the array */
        if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(info_n_items), &error))
            goto out;

        g_string_append (printable, "{");

        for (info_i = 0; info_i < info_n_items; info_i++) {
            g_string_append_printf (printable, " [%u] = '", info_i);
            g_string_append (printable, "[");
            g_string_append (printable, " ipv4_throttled = '");

            {
                gint8 tmp;

                if (!qmi_message_tlv_read_gint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
            }
            g_string_append (printable, "'");
            g_string_append (printable, " ipv6_throttled = '");

            {
                gint8 tmp;

                if (!qmi_message_tlv_read_gint8 (message, init_offset, &offset, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%s", tmp ? "yes" : "no");
            }
            g_string_append (printable, "'");
            g_string_append (printable, " ipv4_throttle_time_left_ms = '");

            {
                guint32 tmp;

                if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
            }
            g_string_append (printable, "'");
            g_string_append (printable, " ipv6_throttle_time_left_ms = '");

            {
                guint32 tmp;

                if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &tmp, &error))
                    goto out;
                g_string_append_printf (printable, "%" G_GUINT32_FORMAT "", tmp);
            }
            g_string_append (printable, "'");
            g_string_append (printable, " apn = '");

            {
                gchar *tmp;

                if (!qmi_message_tlv_read_string (message, init_offset, &offset, 1, 0, &tmp, &error))
                    goto out;
                g_string_append (printable, tmp);
                g_free (tmp);
            }
            g_string_append (printable, "'");
            g_string_append (printable, " ]");
            g_string_append (printable, " '");
        }

        g_string_append (printable, "}");
    }
    if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0)
        g_string_append_printf (printable, "Additional unexpected '%" G_GSIZE_FORMAT "' bytes", offset);

out:
    if (error)
        g_string_append_printf (printable, " ERROR: %s", error->message);
    return g_string_free (printable, FALSE);
}

struct message_get_pdn_throttle_info_context {
    QmiMessage *self;
    const gchar *line_prefix;
    GString *printable;
};

static void
message_get_pdn_throttle_info_get_tlv_printable (
    guint8 type,
    const guint8 *value,
    gsize length,
    struct message_get_pdn_throttle_info_context *ctx)
{
    const gchar *tlv_type_str = NULL;
    gchar *translated_value;

    if (!qmi_message_is_response (ctx->self)) {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO_INPUT_TLV_NETWORK_TYPE:
            tlv_type_str = "Network Type";
            translated_value = qmi_message_wds_get_pdn_throttle_info_input_network_type_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    } else {
        switch (type) {
        case QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO_OUTPUT_TLV_RESULT:
            tlv_type_str = "Result";
            translated_value = qmi_message_result_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        case QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO_OUTPUT_TLV_INFO:
            tlv_type_str = "Info";
            translated_value = qmi_message_wds_get_pdn_throttle_info_output_info_get_printable (
                                   ctx->self,
                                   ctx->line_prefix);
            break;
        default:
            break;
        }
    }

    if (!tlv_type_str) {
        gchar *value_str = NULL;

        value_str = qmi_message_get_tlv_printable (ctx->self,
                                                   ctx->line_prefix,
                                                   type,
                                                   value,
                                                   length);
        g_string_append (ctx->printable, value_str);
        g_free (value_str);
    } else {
        gchar *value_hex;

        value_hex = __qmi_utils_str_hex (value, length, ':');
        g_string_append_printf (ctx->printable,
                                "%sTLV:\n"
                                "%s  type       = \"%s\" (0x%02x)\n"
                                "%s  length     = %" G_GSIZE_FORMAT "\n"
                                "%s  value      = %s\n"
                                "%s  translated = %s\n",
                                ctx->line_prefix,
                                ctx->line_prefix, tlv_type_str, type,
                                ctx->line_prefix, length,
                                ctx->line_prefix, value_hex,
                                ctx->line_prefix, translated_value ? translated_value : "");
        g_free (value_hex);
        g_free (translated_value);
    }
}

static gchar *
message_get_pdn_throttle_info_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    GString *printable;

    printable = g_string_new ("");
    g_string_append_printf (printable,
                            "%s  message     = \"Get PDN Throttle Info\" (0x006C)\n",
                            line_prefix);

    {
        struct message_get_pdn_throttle_info_context ctx;
        ctx.self = self;
        ctx.line_prefix = line_prefix;
        ctx.printable = printable;
        qmi_message_foreach_raw_tlv (self,
                                     (QmiMessageForeachRawTlvFn)message_get_pdn_throttle_info_get_tlv_printable,
                                     &ctx);
    }

    return g_string_free (printable, FALSE);
}

static QmiMessageWdsGetPdnThrottleInfoOutput *
__qmi_message_wds_get_pdn_throttle_info_response_parse (
    QmiMessage *message,
    GError **error)
{
    QmiMessageWdsGetPdnThrottleInfoOutput *self;

    g_return_val_if_fail (qmi_message_get_message_id (message) == QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO, NULL);

    self = g_slice_new0 (QmiMessageWdsGetPdnThrottleInfoOutput);
    self->ref_count = 1;

    do {
        /* No Prerequisites for field */

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO_OUTPUT_TLV_RESULT, NULL, error)) == 0) {
                g_prefix_error (error, "Couldn't get the mandatory Result TLV: ");
                qmi_message_wds_get_pdn_throttle_info_output_unref (self);
                return NULL;
            }
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_status), error))
                goto qmi_message_result_out;
            if (!qmi_message_tlv_read_guint16 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(self->arg_result.error_code), error))
                goto qmi_message_result_out;

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Result' TLV", offset);
            }

            self->arg_result_set = TRUE;

qmi_message_result_out:
            if (!self->arg_result_set) {
                qmi_message_wds_get_pdn_throttle_info_output_unref (self);
                return NULL;
            }

        }
    } while (0);

    do {
        /* Prerequisite.... */
        if (!(self->arg_result.error_status == QMI_STATUS_SUCCESS))
            break;

        {
            gsize offset = 0;
            gsize init_offset;

            if ((init_offset = qmi_message_tlv_read_init (message, QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO_OUTPUT_TLV_INFO, NULL, NULL)) == 0) {
                goto qmi_message_wds_get_pdn_throttle_info_output_info_out;
            }
            {
                guint info_i;
                guint8 info_n_items;

                /* Read number of items in the array */
                if (!qmi_message_tlv_read_guint8 (message, init_offset, &offset, &(info_n_items), NULL))
                    goto qmi_message_wds_get_pdn_throttle_info_output_info_out;

                self->arg_info = g_array_sized_new (
                    FALSE,
                    FALSE,
                    sizeof (QmiMessageWdsGetPdnThrottleInfoOutputInfoElement),
                    (guint)info_n_items);

                g_array_set_clear_func (self->arg_info,
                                        (GDestroyNotify)info_qmimessagewdsgetpdnthrottleinfooutputinfoelement_clear);

                for (info_i = 0; info_i < info_n_items; info_i++) {
                    QmiMessageWdsGetPdnThrottleInfoOutputInfoElement info_aux;

                    {
                        gint8 tmp;

                        if (!qmi_message_tlv_read_gint8 (message, init_offset, &offset, &tmp, NULL))
                            goto qmi_message_wds_get_pdn_throttle_info_output_info_out;
                        info_aux.ipv4_throttled = (gboolean)tmp;
                    }
                    {
                        gint8 tmp;

                        if (!qmi_message_tlv_read_gint8 (message, init_offset, &offset, &tmp, NULL))
                            goto qmi_message_wds_get_pdn_throttle_info_output_info_out;
                        info_aux.ipv6_throttled = (gboolean)tmp;
                    }
                    if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(info_aux.ipv4_throttle_time_left_ms), NULL))
                        goto qmi_message_wds_get_pdn_throttle_info_output_info_out;
                    if (!qmi_message_tlv_read_guint32 (message, init_offset, &offset, QMI_ENDIAN_LITTLE, &(info_aux.ipv6_throttle_time_left_ms), NULL))
                        goto qmi_message_wds_get_pdn_throttle_info_output_info_out;
                    if (!qmi_message_tlv_read_string (message, init_offset, &offset, 1, 0, &(info_aux.apn), NULL))
                        goto qmi_message_wds_get_pdn_throttle_info_output_info_out;
                    g_array_insert_val (self->arg_info, info_i, info_aux);
                }
            }

            /* The remaining size of the buffer needs to be 0 if we successfully read the TLV */
            if ((offset = __qmi_message_tlv_read_remaining_size (message, init_offset, offset)) > 0) {
                g_warning ("Left '%" G_GSIZE_FORMAT "' bytes unread when getting the 'Info' TLV", offset);
            }

            self->arg_info_set = TRUE;

qmi_message_wds_get_pdn_throttle_info_output_info_out:
            ;

        }
    } while (0);

    return self;
}

/*****************************************************************************/
/* Service-specific printable: WDS */


gchar *
__qmi_message_wds_get_printable (
    QmiMessage *self,
    const gchar *line_prefix)
{
    if (qmi_message_is_indication (self)) {
        switch (qmi_message_get_message_id (self)) {
        case QMI_INDICATION_WDS_PACKET_SERVICE_STATUS:
            return indication_packet_service_status_get_printable (self, line_prefix);
         default:
             return NULL;
        }
    } else {
        switch (qmi_message_get_message_id (self)) {
        case QMI_MESSAGE_WDS_RESET:
            return message_reset_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_ABORT:
            return message_abort_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_GET_SUPPORTED_MESSAGES:
            return message_get_supported_messages_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_START_NETWORK:
            return message_start_network_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_STOP_NETWORK:
            return message_stop_network_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_GET_PACKET_SERVICE_STATUS:
            return message_get_packet_service_status_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS:
            return message_get_packet_statistics_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_GET_PROFILE_LIST:
            return message_get_profile_list_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS:
            return message_get_profile_settings_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS:
            return message_get_default_settings_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS:
            return message_get_current_settings_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS:
            return message_get_autoconnect_settings_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY:
            return message_get_data_bearer_technology_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY:
            return message_get_current_data_bearer_technology_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_SET_IP_FAMILY:
            return message_set_ip_family_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS:
            return message_set_autoconnect_settings_get_printable (self, line_prefix);
        case QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO:
            return message_get_pdn_throttle_info_get_printable (self, line_prefix);
         default:
             return NULL;
        }
    }
}

gboolean
__qmi_message_wds_get_version_introduced (
    QmiMessage *self,
    guint *major,
    guint *minor)
{
    switch (qmi_message_get_message_id (self)) {
    case QMI_MESSAGE_WDS_RESET:
        *major = 1;
        *minor = 0;
        return TRUE;
    case QMI_MESSAGE_WDS_ABORT:
        *major = 1;
        *minor = 0;
        return TRUE;
    case QMI_MESSAGE_WDS_GET_SUPPORTED_MESSAGES:
        *major = 1;
        *minor = 38;
        return TRUE;
    case QMI_MESSAGE_WDS_START_NETWORK:
        *major = 1;
        *minor = 0;
        return TRUE;
    case QMI_MESSAGE_WDS_STOP_NETWORK:
        *major = 1;
        *minor = 0;
        return TRUE;
    case QMI_MESSAGE_WDS_GET_PACKET_SERVICE_STATUS:
        *major = 1;
        *minor = 0;
        return TRUE;
    case QMI_MESSAGE_WDS_GET_PACKET_STATISTICS:
        *major = 1;
        *minor = 0;
        return TRUE;
    case QMI_MESSAGE_WDS_GET_PROFILE_LIST:
        *major = 1;
        *minor = 1;
        return TRUE;
    case QMI_MESSAGE_WDS_GET_PROFILE_SETTINGS:
        *major = 1;
        *minor = 1;
        return TRUE;
    case QMI_MESSAGE_WDS_GET_DEFAULT_SETTINGS:
        *major = 1;
        *minor = 1;
        return TRUE;
    case QMI_MESSAGE_WDS_GET_CURRENT_SETTINGS:
        *major = 1;
        *minor = 2;
        return TRUE;
    case QMI_MESSAGE_WDS_GET_AUTOCONNECT_SETTINGS:
        *major = 1;
        *minor = 12;
        return TRUE;
    case QMI_MESSAGE_WDS_GET_DATA_BEARER_TECHNOLOGY:
        *major = 1;
        *minor = 12;
        return TRUE;
    case QMI_MESSAGE_WDS_GET_CURRENT_DATA_BEARER_TECHNOLOGY:
        *major = 1;
        *minor = 4;
        return TRUE;
    case QMI_MESSAGE_WDS_SET_IP_FAMILY:
        *major = 1;
        *minor = 9;
        return TRUE;
    case QMI_MESSAGE_WDS_SET_AUTOCONNECT_SETTINGS:
        *major = 1;
        *minor = 12;
        return TRUE;
    case QMI_MESSAGE_WDS_GET_PDN_THROTTLE_INFO:
        *major = 1;
        *minor = 12;
        return TRUE;
    default:
        return FALSE;
    }
}

/*****************************************************************************/
/* CLIENT: QMI Client WDS */


/**
 * SECTION: qmi-client-wds
 * @title: QmiClientWds
 * @short_description: #QmiClient for the WDS service.
 *
 * #QmiClient which handles operations in the WDS service.
 */

G_DEFINE_TYPE (QmiClientWds, qmi_client_wds, QMI_TYPE_CLIENT);

enum {
    SIGNAL_PACKET_SERVICE_STATUS,
    SIGNAL_LAST
};

static guint signals[SIGNAL_LAST] = { 0 };

static void
process_indication (QmiClient *self,
                    QmiMessage *message)
{
    switch (qmi_message_get_message_id (message)) {
        case QMI_INDICATION_WDS_PACKET_SERVICE_STATUS: {
            QmiIndicationWdsPacketServiceStatusOutput *output;
            GError *error = NULL;

            /* Parse indication */
            output = __qmi_indication_wds_packet_service_status_indication_parse (message, &error);
            if (!output) {
                g_warning ("Couldn't parse 'Packet Service Status' indication: %s",
                           error ? error->message : "Unknown error");
                if (error)
                    g_error_free (error);
            } else {
                g_signal_emit (self, signals[SIGNAL_PACKET_SERVICE_STATUS], 0, output);
                qmi_indication_wds_packet_service_status_output_unref (output);
            }
            break;
        }
        default:
            break;
    }
}

static void
qmi_client_wds_init (QmiClientWds *self)
{
}

static void
qmi_client_wds_class_init (QmiClientWdsClass *klass)
{
    QmiClientClass *client_class = QMI_CLIENT_CLASS (klass);

    client_class->process_indication = process_indication;

    /**
     * QmiClientWds::packet-service-status:
     * @object: A #QmiClientWds.
     * @output: A #QmiIndicationWdsPacketServiceStatusOutput.
     *
     * The ::packet-service-status signal gets emitted when a '<link linkend="libqmi-glib-WDS-Packet-Service-Status.top_of_page">Packet Service Status</link>' indication is received.
     */
    signals[SIGNAL_PACKET_SERVICE_STATUS] =
        g_signal_new ("packet-service-status",
                      G_OBJECT_CLASS_TYPE (G_OBJECT_CLASS (klass)),
                      G_SIGNAL_RUN_LAST,
                      0,
                      NULL,
                      NULL,
                      NULL,
                      G_TYPE_NONE,
                      1,
                      QMI_TYPE_INDICATION_WDS_PACKET_SERVICE_STATUS_OUTPUT);
}


/**
 * qmi_client_wds_reset_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_reset().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_reset().
 *
 * Returns: a #QmiMessageWdsResetOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_reset_output_unref().
 */
QmiMessageWdsResetOutput *
qmi_client_wds_reset_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_reset_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
reset_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsResetOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_reset_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_reset_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_reset:
 * @self: a #QmiClientWds.
 * @unused: %NULL. This message doesn't have any input bundle.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Reset request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_reset_finish() to get the result of the operation.
 */
void
qmi_client_wds_reset (
    QmiClientWds *self,
    gpointer unused,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_reset);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_reset_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  NULL,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)reset_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_get_supported_messages_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_get_supported_messages().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_get_supported_messages().
 *
 * Returns: a #QmiMessageWdsGetSupportedMessagesOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_get_supported_messages_output_unref().
 */
QmiMessageWdsGetSupportedMessagesOutput *
qmi_client_wds_get_supported_messages_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_get_supported_messages_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
get_supported_messages_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsGetSupportedMessagesOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_get_supported_messages_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_get_supported_messages_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_get_supported_messages:
 * @self: a #QmiClientWds.
 * @unused: %NULL. This message doesn't have any input bundle.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Get Supported Messages request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_get_supported_messages_finish() to get the result of the operation.
 */
void
qmi_client_wds_get_supported_messages (
    QmiClientWds *self,
    gpointer unused,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_get_supported_messages);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_get_supported_messages_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  NULL,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)get_supported_messages_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_start_network_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_start_network().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_start_network().
 *
 * Returns: a #QmiMessageWdsStartNetworkOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_start_network_output_unref().
 */
QmiMessageWdsStartNetworkOutput *
qmi_client_wds_start_network_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_start_network_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
start_network_abort_ready (
    QmiDevice *device,
    GAsyncResult *res)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsAbortOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (reply) {
        output = __qmi_message_wds_abort_response_parse (reply, &error);
        if (output)
            qmi_message_wds_abort_output_unref (output);
        qmi_message_unref (reply);
    }

    if (error) {
        g_debug ("Operation to abort 'Start Network' failed: %s", error->message);
        g_error_free (error);
    }
}

static void
start_network_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsStartNetworkOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        if (g_error_matches (error, QMI_CORE_ERROR, QMI_CORE_ERROR_TIMEOUT) ||
            g_error_matches (error, QMI_PROTOCOL_ERROR, QMI_PROTOCOL_ERROR_ABORTED)) {
                QmiMessage *abort;
                GObject *self;
                guint16 transaction_id;
                QmiMessageWdsAbortInput *input;

                self = g_async_result_get_source_object (G_ASYNC_RESULT (simple));
                g_assert (self != NULL);

                transaction_id = (guint16) GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (simple),
                                                                                "transaction-id"));
                g_assert (transaction_id != 0);

                input = qmi_message_wds_abort_input_new ();
                qmi_message_wds_abort_input_set_transaction_id (
                    input,
                    transaction_id,
                    NULL);
                abort = __qmi_message_wds_abort_request_create (
                            qmi_client_get_next_transaction_id (QMI_CLIENT (self)),
                            qmi_client_get_cid (QMI_CLIENT (self)),
                            input,
                            NULL);
                g_assert (abort != NULL);
                qmi_device_command (device,
                                    abort,
                                    30,
                                    NULL,
                                    (GAsyncReadyCallback)start_network_abort_ready,
                                    NULL);
                qmi_message_wds_abort_input_unref (input);
                qmi_message_unref (abort);
                g_object_unref (self);
            }

        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_start_network_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_start_network_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_start_network:
 * @self: a #QmiClientWds.
 * @input: a #QmiMessageWdsStartNetworkInput.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Start Network request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_start_network_finish() to get the result of the operation.
 */
void
qmi_client_wds_start_network (
    QmiClientWds *self,
    QmiMessageWdsStartNetworkInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_start_network);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_start_network_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    g_object_set_data (G_OBJECT (result),
                       "transaction-id",
                       GUINT_TO_POINTER (transaction_id));

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)start_network_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_stop_network_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_stop_network().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_stop_network().
 *
 * Returns: a #QmiMessageWdsStopNetworkOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_stop_network_output_unref().
 */
QmiMessageWdsStopNetworkOutput *
qmi_client_wds_stop_network_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_stop_network_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
stop_network_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsStopNetworkOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_stop_network_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_stop_network_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_stop_network:
 * @self: a #QmiClientWds.
 * @input: a #QmiMessageWdsStopNetworkInput.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Stop Network request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_stop_network_finish() to get the result of the operation.
 */
void
qmi_client_wds_stop_network (
    QmiClientWds *self,
    QmiMessageWdsStopNetworkInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_stop_network);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_stop_network_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)stop_network_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_get_packet_service_status_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_get_packet_service_status().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_get_packet_service_status().
 *
 * Returns: a #QmiMessageWdsGetPacketServiceStatusOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_get_packet_service_status_output_unref().
 */
QmiMessageWdsGetPacketServiceStatusOutput *
qmi_client_wds_get_packet_service_status_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_get_packet_service_status_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
get_packet_service_status_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsGetPacketServiceStatusOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_get_packet_service_status_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_get_packet_service_status_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_get_packet_service_status:
 * @self: a #QmiClientWds.
 * @unused: %NULL. This message doesn't have any input bundle.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Get Packet Service Status request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_get_packet_service_status_finish() to get the result of the operation.
 */
void
qmi_client_wds_get_packet_service_status (
    QmiClientWds *self,
    gpointer unused,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_get_packet_service_status);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_get_packet_service_status_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  NULL,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)get_packet_service_status_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_get_packet_statistics_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_get_packet_statistics().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_get_packet_statistics().
 *
 * Returns: a #QmiMessageWdsGetPacketStatisticsOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_get_packet_statistics_output_unref().
 */
QmiMessageWdsGetPacketStatisticsOutput *
qmi_client_wds_get_packet_statistics_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_get_packet_statistics_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
get_packet_statistics_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsGetPacketStatisticsOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_get_packet_statistics_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_get_packet_statistics_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_get_packet_statistics:
 * @self: a #QmiClientWds.
 * @input: a #QmiMessageWdsGetPacketStatisticsInput.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Get Packet Statistics request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_get_packet_statistics_finish() to get the result of the operation.
 */
void
qmi_client_wds_get_packet_statistics (
    QmiClientWds *self,
    QmiMessageWdsGetPacketStatisticsInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_get_packet_statistics);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_get_packet_statistics_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)get_packet_statistics_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_get_profile_list_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_get_profile_list().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_get_profile_list().
 *
 * Returns: a #QmiMessageWdsGetProfileListOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_get_profile_list_output_unref().
 */
QmiMessageWdsGetProfileListOutput *
qmi_client_wds_get_profile_list_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_get_profile_list_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
get_profile_list_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsGetProfileListOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_get_profile_list_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_get_profile_list_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_get_profile_list:
 * @self: a #QmiClientWds.
 * @input: a #QmiMessageWdsGetProfileListInput.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Get Profile List request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_get_profile_list_finish() to get the result of the operation.
 */
void
qmi_client_wds_get_profile_list (
    QmiClientWds *self,
    QmiMessageWdsGetProfileListInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_get_profile_list);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_get_profile_list_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)get_profile_list_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_get_profile_settings_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_get_profile_settings().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_get_profile_settings().
 *
 * Returns: a #QmiMessageWdsGetProfileSettingsOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_get_profile_settings_output_unref().
 */
QmiMessageWdsGetProfileSettingsOutput *
qmi_client_wds_get_profile_settings_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_get_profile_settings_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
get_profile_settings_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsGetProfileSettingsOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_get_profile_settings_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_get_profile_settings_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_get_profile_settings:
 * @self: a #QmiClientWds.
 * @input: a #QmiMessageWdsGetProfileSettingsInput.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Get Profile Settings request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_get_profile_settings_finish() to get the result of the operation.
 */
void
qmi_client_wds_get_profile_settings (
    QmiClientWds *self,
    QmiMessageWdsGetProfileSettingsInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_get_profile_settings);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_get_profile_settings_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)get_profile_settings_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_get_default_settings_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_get_default_settings().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_get_default_settings().
 *
 * Returns: a #QmiMessageWdsGetDefaultSettingsOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_get_default_settings_output_unref().
 */
QmiMessageWdsGetDefaultSettingsOutput *
qmi_client_wds_get_default_settings_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_get_default_settings_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
get_default_settings_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsGetDefaultSettingsOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_get_default_settings_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_get_default_settings_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_get_default_settings:
 * @self: a #QmiClientWds.
 * @input: a #QmiMessageWdsGetDefaultSettingsInput.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Get Default Settings request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_get_default_settings_finish() to get the result of the operation.
 */
void
qmi_client_wds_get_default_settings (
    QmiClientWds *self,
    QmiMessageWdsGetDefaultSettingsInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_get_default_settings);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_get_default_settings_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)get_default_settings_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_get_current_settings_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_get_current_settings().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_get_current_settings().
 *
 * Returns: a #QmiMessageWdsGetCurrentSettingsOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_get_current_settings_output_unref().
 */
QmiMessageWdsGetCurrentSettingsOutput *
qmi_client_wds_get_current_settings_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_get_current_settings_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
get_current_settings_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsGetCurrentSettingsOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_get_current_settings_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_get_current_settings_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_get_current_settings:
 * @self: a #QmiClientWds.
 * @input: a #QmiMessageWdsGetCurrentSettingsInput.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Get Current Settings request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_get_current_settings_finish() to get the result of the operation.
 */
void
qmi_client_wds_get_current_settings (
    QmiClientWds *self,
    QmiMessageWdsGetCurrentSettingsInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_get_current_settings);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_get_current_settings_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)get_current_settings_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_get_autoconnect_settings_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_get_autoconnect_settings().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_get_autoconnect_settings().
 *
 * Returns: a #QmiMessageWdsGetAutoconnectSettingsOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_get_autoconnect_settings_output_unref().
 */
QmiMessageWdsGetAutoconnectSettingsOutput *
qmi_client_wds_get_autoconnect_settings_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_get_autoconnect_settings_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
get_autoconnect_settings_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsGetAutoconnectSettingsOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_get_autoconnect_settings_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_get_autoconnect_settings_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_get_autoconnect_settings:
 * @self: a #QmiClientWds.
 * @unused: %NULL. This message doesn't have any input bundle.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Get Autoconnect Settings request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_get_autoconnect_settings_finish() to get the result of the operation.
 */
void
qmi_client_wds_get_autoconnect_settings (
    QmiClientWds *self,
    gpointer unused,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_get_autoconnect_settings);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_get_autoconnect_settings_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  NULL,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)get_autoconnect_settings_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_get_data_bearer_technology_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_get_data_bearer_technology().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_get_data_bearer_technology().
 *
 * Returns: a #QmiMessageWdsGetDataBearerTechnologyOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_get_data_bearer_technology_output_unref().
 */
QmiMessageWdsGetDataBearerTechnologyOutput *
qmi_client_wds_get_data_bearer_technology_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_get_data_bearer_technology_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
get_data_bearer_technology_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsGetDataBearerTechnologyOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_get_data_bearer_technology_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_get_data_bearer_technology_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_get_data_bearer_technology:
 * @self: a #QmiClientWds.
 * @unused: %NULL. This message doesn't have any input bundle.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Get Data Bearer Technology request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_get_data_bearer_technology_finish() to get the result of the operation.
 */
void
qmi_client_wds_get_data_bearer_technology (
    QmiClientWds *self,
    gpointer unused,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_get_data_bearer_technology);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_get_data_bearer_technology_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  NULL,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)get_data_bearer_technology_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_get_current_data_bearer_technology_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_get_current_data_bearer_technology().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_get_current_data_bearer_technology().
 *
 * Returns: a #QmiMessageWdsGetCurrentDataBearerTechnologyOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_get_current_data_bearer_technology_output_unref().
 */
QmiMessageWdsGetCurrentDataBearerTechnologyOutput *
qmi_client_wds_get_current_data_bearer_technology_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_get_current_data_bearer_technology_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
get_current_data_bearer_technology_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsGetCurrentDataBearerTechnologyOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_get_current_data_bearer_technology_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_get_current_data_bearer_technology_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_get_current_data_bearer_technology:
 * @self: a #QmiClientWds.
 * @unused: %NULL. This message doesn't have any input bundle.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Get Current Data Bearer Technology request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_get_current_data_bearer_technology_finish() to get the result of the operation.
 */
void
qmi_client_wds_get_current_data_bearer_technology (
    QmiClientWds *self,
    gpointer unused,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_get_current_data_bearer_technology);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_get_current_data_bearer_technology_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  NULL,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)get_current_data_bearer_technology_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_set_ip_family_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_set_ip_family().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_set_ip_family().
 *
 * Returns: a #QmiMessageWdsSetIpFamilyOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_set_ip_family_output_unref().
 */
QmiMessageWdsSetIpFamilyOutput *
qmi_client_wds_set_ip_family_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_set_ip_family_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
set_ip_family_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsSetIpFamilyOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_set_ip_family_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_set_ip_family_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_set_ip_family:
 * @self: a #QmiClientWds.
 * @input: a #QmiMessageWdsSetIpFamilyInput.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Set IP Family request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_set_ip_family_finish() to get the result of the operation.
 */
void
qmi_client_wds_set_ip_family (
    QmiClientWds *self,
    QmiMessageWdsSetIpFamilyInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_set_ip_family);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_set_ip_family_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)set_ip_family_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_set_autoconnect_settings_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_set_autoconnect_settings().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_set_autoconnect_settings().
 *
 * Returns: a #QmiMessageWdsSetAutoconnectSettingsOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_set_autoconnect_settings_output_unref().
 */
QmiMessageWdsSetAutoconnectSettingsOutput *
qmi_client_wds_set_autoconnect_settings_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_set_autoconnect_settings_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
set_autoconnect_settings_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsSetAutoconnectSettingsOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_set_autoconnect_settings_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_set_autoconnect_settings_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_set_autoconnect_settings:
 * @self: a #QmiClientWds.
 * @input: a #QmiMessageWdsSetAutoconnectSettingsInput.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Set Autoconnect Settings request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_set_autoconnect_settings_finish() to get the result of the operation.
 */
void
qmi_client_wds_set_autoconnect_settings (
    QmiClientWds *self,
    QmiMessageWdsSetAutoconnectSettingsInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_set_autoconnect_settings);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_set_autoconnect_settings_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)set_autoconnect_settings_ready,
                        result);
    qmi_message_unref (request);
}


/**
 * qmi_client_wds_get_pdn_throttle_info_finish:
 * @self: a #QmiClientWds.
 * @res: the #GAsyncResult obtained from the #GAsyncReadyCallback passed to qmi_client_wds_get_pdn_throttle_info().
 * @error: Return location for error or %NULL.
 *
 * Finishes an async operation started with qmi_client_wds_get_pdn_throttle_info().
 *
 * Returns: a #QmiMessageWdsGetPdnThrottleInfoOutput, or %NULL if @error is set. The returned value should be freed with qmi_message_wds_get_pdn_throttle_info_output_unref().
 */
QmiMessageWdsGetPdnThrottleInfoOutput *
qmi_client_wds_get_pdn_throttle_info_finish (
    QmiClientWds *self,
    GAsyncResult *res,
    GError **error)
{
   if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
       return NULL;

   return qmi_message_wds_get_pdn_throttle_info_output_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
}

static void
get_pdn_throttle_info_ready (
    QmiDevice *device,
    GAsyncResult *res,
    GSimpleAsyncResult *simple)
{
    GError *error = NULL;
    QmiMessage *reply;
    QmiMessageWdsGetPdnThrottleInfoOutput *output;

    reply = qmi_device_command_finish (device, res, &error);
    if (!reply) {
        g_simple_async_result_take_error (simple, error);
        g_simple_async_result_complete (simple);
        g_object_unref (simple);
        return;
    }

    /* Parse reply */
    output = __qmi_message_wds_get_pdn_throttle_info_response_parse (reply, &error);
    if (!output)
        g_simple_async_result_take_error (simple, error);
    else
        g_simple_async_result_set_op_res_gpointer (simple,
                                                   output,
                                                   (GDestroyNotify)qmi_message_wds_get_pdn_throttle_info_output_unref);
    g_simple_async_result_complete (simple);
    g_object_unref (simple);
    qmi_message_unref (reply);
}

/**
 * qmi_client_wds_get_pdn_throttle_info:
 * @self: a #QmiClientWds.
 * @input: a #QmiMessageWdsGetPdnThrottleInfoInput.
 * @timeout: maximum time to wait for the method to complete, in seconds.
 * @cancellable: a #GCancellable or %NULL.
 * @callback: a #GAsyncReadyCallback to call when the request is satisfied.
 * @user_data: user data to pass to @callback.
 *
 * Asynchronously sends a Get PDN Throttle Info request to the device.
 *
 * When the operation is finished, @callback will be invoked in the thread-default main loop of the thread you are calling this method from.
 *
 * You can then call qmi_client_wds_get_pdn_throttle_info_finish() to get the result of the operation.
 */
void
qmi_client_wds_get_pdn_throttle_info (
    QmiClientWds *self,
    QmiMessageWdsGetPdnThrottleInfoInput *input,
    guint timeout,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data)
{
    GSimpleAsyncResult *result;
    QmiMessage *request;
    GError *error = NULL;
    guint16 transaction_id;

    result = g_simple_async_result_new (G_OBJECT (self),
                                        callback,
                                        user_data,
                                        qmi_client_wds_get_pdn_throttle_info);

    transaction_id = qmi_client_get_next_transaction_id (QMI_CLIENT (self));

    request = __qmi_message_wds_get_pdn_throttle_info_request_create (
                  transaction_id,
                  qmi_client_get_cid (QMI_CLIENT (self)),
                  input,
                  &error);
    if (!request) {
        g_prefix_error (&error, "Couldn't create request message: ");
        g_simple_async_result_take_error (result, error);
        g_simple_async_result_complete_in_idle (result);
        g_object_unref (result);
        return;
    }

    qmi_device_command (QMI_DEVICE (qmi_client_peek_device (QMI_CLIENT (self))),
                        request,
                        timeout,
                        cancellable,
                        (GAsyncReadyCallback)get_pdn_throttle_info_ready,
                        result);
    qmi_message_unref (request);
}

